/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QSettings>
#include <QString>
#include <QtDebug>
#include <QCoreApplication>
#include "Ws2355.h"
#include "Configuration.h"



Configuration::Configuration()
{
    readSettings();

    QString myAppDirPath =  QCoreApplication::applicationDirPath();

    translatorEn.load( "WsMeteo_en", myAppDirPath );
    translatorFr.load( "WsMeteo_fr", myAppDirPath );

    if( language == 0 )
    {
        QCoreApplication::installTranslator( &translatorFr );
    }
    if( language == 1 )
    {
        QCoreApplication::installTranslator( &translatorEn );
    }
}

Configuration::~Configuration()
{
    writeSettings();
}

void Configuration::readSettings( void )
{
    QSettings settings("Gil", "WsMeteo");

    dataFilesDirectory = settings.value( "dataFilesDirectory", QDir::currentPath().toAscii() ).toString();
    windSpeedUnit = settings.value( "windSpeedUnit", KILOMETERS_PER_HOUR ).toDouble();
    temperatureUnit = settings.value( "temperatureUnit", CELCIUS ).toInt();
    rainUnit = settings.value( "rainUnit", MILLIMETERS ).toDouble();
    pressureUnit = settings.value( "pressureUnit", MILLIBARS ).toDouble();
    frequencyOfReading = settings.value( "frequencyOfReading", 1 ).toInt();
    language = settings.value( "programLanguage", 0 ).toInt();

}

void Configuration::writeSettings( void )
{
    QSettings settings("Gil", "WsMeteo");

    settings.setValue( "dataFilesDirectory", dataFilesDirectory.path() );
    settings.setValue( "windSpeedUnit", windSpeedUnit );
    settings.setValue( "temperatureUnit", temperatureUnit );
    settings.setValue( "rainUnit", rainUnit );
    settings.setValue( "pressureUnit", pressureUnit );
    settings.setValue( "programLanguage", language );
    settings.setValue( "frequencyOfReading", frequencyOfReading );
}

void Configuration::setProgramLanguage (int val)
{
    if( val == 0 )
    {
        QCoreApplication::removeTranslator( &translatorEn );
        QCoreApplication::installTranslator( &translatorFr );
    }
    if( val == 1 )
    {
        QCoreApplication::removeTranslator( &translatorFr );
        QCoreApplication::installTranslator( &translatorEn );
    }
    language = val;
}

QString Configuration::getFullDataFileName( void )
{
    QString strFile( dataFilesDirectory.path() );
    strFile.append( "/currentDataFile.txt" );

    return strFile;
}

/**
 * Returns the wind speed unit (KILOMETERS_PER_HOUR, METERS_PER_SECOND, MILES_PER_HOUR)
 *
 * @return val : Unit
 */
double Configuration::getWindSpeedUnit( void )
{
    double val = KILOMETERS_PER_HOUR;

    if( windSpeedUnit == 0 )
        val = KILOMETERS_PER_HOUR;
    else if( windSpeedUnit == 1 )
        val = METERS_PER_SECOND;
    else if( windSpeedUnit == 2 )
        val = MILES_PER_HOUR;

    return val;
}

/**
 * Returns the temperature unit (CELCIUS, FAHRENHEIT)
 *
 * @return val : Unit
 */
int Configuration::getTemperatureUnit( void )
{
    int val = CELCIUS;

    if( temperatureUnit == 0 )
        val = CELCIUS;
    else if( temperatureUnit == 1 )
        val = FAHRENHEIT;

    return val;
}
/**
 * Returns the rain unit (MILLIMETERS, INCHES)
 *
 * @return val : unit
 */
double Configuration::getRainUnit( void )
{
    double val = MILLIMETERS;

    if( rainUnit == 0 )
        val = MILLIMETERS;
    else if( rainUnit == 1 )
        val = INCHES;

    return val;
}

/**
 * Returns the pressure unit (HECTOPASCAL, MILLIBARS, INCHES_HG)
 *
 * @return
 */
double Configuration::getPressureUnit( void )
{
    double val = HECTOPASCAL;

    if( pressureUnit == 0 )
        val = HECTOPASCAL;
    else if( pressureUnit == 1 )
        val = MILLIBARS;
    else if( pressureUnit == 2 )
        val = INCHES_HG;

    return val;
}
