/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CONFIGURATION_H
#define CONFIGURATION_H
#include <QDir>
#include <QString>
#include <QObject>
#include <QTranslator>

class Configuration
{
public:
    Configuration();
    ~Configuration();

public:

    QDir                getDataFilesDirectory( void ) { return dataFilesDirectory; }
    void                setDataFilesDirectory( QDir val ) { dataFilesDirectory = val; }
    QString             getFullDataFileName( void );

    double              getWindSpeedUnit( void );
    int                 getWindSpeedUnitId( void ) { return windSpeedUnit; }
    void                setWindSpeedUnit( int val ) { windSpeedUnit = val; }
    int                 getTemperatureUnit( void );
    void                setTemperatureUnit( int val ) { temperatureUnit = val; }
    int                 getTemperatureUnitId( void ) { return temperatureUnit; }
    double              getRainUnit( void );
    void                setRainUnit( int val ) { rainUnit = val; }
    int                 getRainUnitId( void ) { return rainUnit; }
    double              getPressureUnit( void );
    void                setPressureUnit( int val ) { pressureUnit = val; }
    int                 getPressureUnitId( void ) { return pressureUnit; }
    int                 getfrequencyOfReading( void ) { return frequencyOfReading; }
    void                setfrequencyOfReading( int val ) { frequencyOfReading = val; }
    int                 getProgramLanguage( void ) { return language; }
    void                setProgramLanguage( int val );

private:
    int                 windSpeedUnit;      // from m/s to km/h or miles/hour
    int                 temperatureUnit;    // 0=Celcius, 1=Fahrenheit
    int                 rainUnit;           // from mm to inch
    int                 pressureUnit;       // from hPa (=millibar) to mmHg

    int                 language;           // Program interface language

    QDir                dataFilesDirectory;     

    int                 frequencyOfReading; // Read data from station every x hours

    void                writeSettings( void );
    void                readSettings( void );
    QTranslator         translatorEn;
    QTranslator         translatorFr;

};

#endif // CONFIGURATION_H
