/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qwt_compass.h>
#include <qwt_compass_rose.h>
#include <qwt_dial_needle.h>
#include <QSpacerItem>
#include <QDir>
#include <QFile>
#include <QFont>

#include "Configuration.h"
#include "ConfigurationTab.h"
#include "WsMeteo.h"

ConfigurationTab::ConfigurationTab( QWidget *parent, Ws2355 *station, Configuration *config ) :
    QFrame(parent)
{
    theStation = station;
    theConfiguration = config;



    createTab();
}

ConfigurationTab::~ConfigurationTab()
{

}

void ConfigurationTab::changeEvent(QEvent *event) {
    if (event->type() == QEvent::LanguageChange)
    {
        // Set all texts
        wsMeteoConfigurationGroup->setTitle( trUtf8( "WsMétéo configuration" ));
        label1->setText( trUtf8( "Répertoire des fichiers de relevés" ));
        filesConfigurationGroup->setTitle( trUtf8( "Relevés des données" ));
        label3->setText( trUtf8( "Relevé des données, toutes les " ));
        label4->setText( trUtf8( "heures" ));
        serialPortConfigurationGroup->setTitle( trUtf8( "Port série" ));
        label2->setText( trUtf8( "Nom du port série" ));
        unitConfigurationGroup->setTitle( trUtf8( "Unités/Pays" ));
        label6->setText( trUtf8( "Langage" ));
        label7->setText( trUtf8( "Unité de vitesse du vent" ));
        label8->setText( trUtf8( "Unité de température" ));
        label9->setText( trUtf8( "Unité de pression" ));
        label10->setText( trUtf8("Unité de précipitation" ));
        briefLabel->setText( trUtf8( "WSMétéo est un programme de relevé <br>d'informations météo en communiquant <br>avec une station météo personnelle<br>de type WS2355 Lacrosse Technology</b>." ));
        stationConfigurationGroup->setTitle( trUtf8( "Status des alarmes" ));
        temperaturesGroup->setTitle( trUtf8( "Températures" ));
        inTemperatureMinAlarm->setText( trUtf8( "T° Intérieure Min" ));
        inTemperatureMaxAlarm->setText( trUtf8( "T° Intérieure Max" ));
        outTemperatureMinAlarm->setText( trUtf8( "T° Extérieure Min" ));
        outTemperatureMaxAlarm->setText( trUtf8( "T° Extérieure Max" ));
        windChillTemperatureMinAlarm->setText( trUtf8( "T° Ressentie Min" ));
        windChillTemperatureMaxAlarm->setText( trUtf8( "T° Ressentie Max" ));
        dewPointTemperatureMinAlarm->setText( trUtf8( "T° du point de rosée Min" ));
        dewPointTemperatureMaxAlarm->setText( trUtf8( "T° du point de rosée Max" ));
        rainGroup->setTitle( trUtf8( "Précipitations" ));
        rain1HAlarm->setText( trUtf8( "Précipitation 1H" ));
        rain1DayAlarm->setText( trUtf8( "Précipitation 24H" ));
        humidityGroup->setTitle( trUtf8( "Humidité" ));
        inHumidityMinAlarm->setText( trUtf8( "Humidité Intérieure Min" ));
        inHumidityMaxAlarm->setText( trUtf8( "Humidité Intérieure Max" ));
        outHumidityMinAlarm->setText( trUtf8( "Humidité Extérieure Min" ));
        outHumidityMaxAlarm->setText( trUtf8( "Humidité Extérieure Max" ));
        pressureGroup->setTitle( trUtf8( "Pression" ));
        pressureMinAlarm->setText( trUtf8( "Pression Min" ));
        pressureMaxAlarm->setText( trUtf8( "Pression Max" ));
        windGroup->setTitle( trUtf8( "Vent" ));
        windSpeedMinAlarm->setText( trUtf8( "Vitesse Min" ));
        windSpeedMaxAlarm->setText( trUtf8( "Vitesse Max" ));
        windDirectionAlarm->setText( trUtf8( "Direction" ));

    }
    else
    {
        QWidget::changeEvent(event);
    }
}

void ConfigurationTab::createTab( void )
{
    // WsMeteo configuration
    wsMeteoConfigurationGroup = new QGroupBox( trUtf8( "WsMétéo configuration" ));

    // Files
    filesConfigurationGroup = new QGroupBox( trUtf8( "Relevés des données" ));
    label1 = new QLabel( trUtf8( "Répertoire des fichiers de relevés" ));
    dataFilesDirectoryLineEdit = new QLineEdit;
    dataFilesDirectoryLineEdit->setText( theConfiguration->getDataFilesDirectory().path() );

    label3 = new QLabel( trUtf8( "Relevé des données, toutes les " ));
    label4 = new QLabel( trUtf8( "heures" ));
    frequencyOfReadingSpinBox = new QSpinBox;
    frequencyOfReadingSpinBox->setMinimum( 1 );
    frequencyOfReadingSpinBox->setMaximum( 175 );
    frequencyOfReadingSpinBox->setValue( theConfiguration->getfrequencyOfReading() );
    frequencyOfReadingSpinBox->setFixedWidth( 50 );
    frequencyOfReadingSpinBox->setAlignment( Qt::AlignCenter );

    // Serial port
    serialPortConfigurationGroup = new QGroupBox( trUtf8( "Port série" ));
    label2 = new QLabel( trUtf8( "Nom du port série" ));
    deviceNameLineEdit = new QLineEdit;
    deviceNameLineEdit->setFont( QFont( "Arial", 10, QFont::Bold ));
    deviceNameLineEdit->setText( theStation->getTheConnection()->getDeviceName() );
    deviceNameLineEdit->setMaximumWidth( 200 );
    dataFilesDirectoryButton = new QPushButton;
    dataFilesDirectoryButton->setText( "..." );
    dataFilesDirectoryButton->setFixedWidth( 60 );

    // Serial port
    unitConfigurationGroup = new QGroupBox( trUtf8( "Unités/Pays" ));
    label6 = new QLabel( trUtf8( "Langage" ));
    label7 = new QLabel( trUtf8( "Unité de vitesse du vent" ));
    label8 = new QLabel( trUtf8( "Unité de température" ));
    label9 = new QLabel( trUtf8( "Unité de pression" ));
    label10 = new QLabel( trUtf8( "Unité de précipitation" ));
    languageBox = new QComboBox;
    languageBox->addItem( trUtf8( "Français" ));
    languageBox->addItem( trUtf8( "Anglais" ));
    languageBox->setCurrentIndex( theConfiguration->getProgramLanguage() );
    windSpeedUnitBox = new QComboBox;
    windSpeedUnitBox->addItem( trUtf8( "km/h" ));
    windSpeedUnitBox->addItem( trUtf8( "m/s" ));
    windSpeedUnitBox->addItem( trUtf8( "miles/h" ));
    windSpeedUnitBox->setCurrentIndex( theConfiguration->getWindSpeedUnitId() );
    temperatureUnitBox = new QComboBox;
    temperatureUnitBox->addItem( trUtf8( "Celcius" ));
    temperatureUnitBox->addItem( trUtf8( "Fahrenheit" ));
    temperatureUnitBox->setCurrentIndex( theConfiguration->getTemperatureUnitId() );
    pressureUnitBox = new QComboBox;
    pressureUnitBox->addItem( trUtf8( "hPa" ));
    pressureUnitBox->addItem( trUtf8( "millibar" ));
    pressureUnitBox->addItem( trUtf8( "inches Hg" ));
    pressureUnitBox->setCurrentIndex( theConfiguration->getPressureUnitId() );
    rainUnitBox = new QComboBox;
    rainUnitBox->addItem( trUtf8( "mm" ));
    rainUnitBox->addItem( trUtf8( "inches" ));
    rainUnitBox->setCurrentIndex( theConfiguration->getRainUnitId() );

    // Info program
    QGroupBox *infoProgramGroup = new QGroupBox( trUtf8( "WS Météo" ));
    QLabel *label5 = new QLabel();
    QPixmap imageLogo( ":/Images/Logo-WSmeteo.png" );
    label5->setPixmap( imageLogo );
    QLabel *programLabel = new QLabel( GUIStrings::WSMETEO_PROGRAM_NAME );
    programLabel->setFont( QFont( "Arial", 10, QFont::Bold ));
    QLabel *copyrightLabel = new QLabel( GUIStrings::WSMETEO_PROGRAM_COPYRIGHT );
    QLabel *versionLabel = new QLabel( GUIStrings::WSMETEO_PROGRAM_VERSION );
    briefLabel = new QLabel( trUtf8( "WSMétéo est un programme de relevé <br>d'informations météo en communiquant <br>avec une station météo personnelle<br>de type WS2355 Lacrosse Technology</b>." ));

    // Station configuration
    stationConfigurationGroup = new QGroupBox( trUtf8( "Status des alarmes" ));

    // Temperatures
    temperaturesGroup = new QGroupBox( trUtf8( "Températures" ));

    inTemperatureMin = new QDoubleSpinBox();
    inTemperatureMin->setMinimum( WSMETEO_TEMPERATURE_MIN );
    inTemperatureMin->setMaximum( WSMETEO_TEMPERATURE_MAX );
    inTemperatureMin->setDisabled( true);
    inTemperatureMin->setAlignment( Qt::AlignCenter );
    inTemperatureMin->setFixedWidth( 50 );

    inTemperatureMinAlarm = new QCheckBox();
    inTemperatureMinAlarm->setText( trUtf8( "T° Intérieure Min" ));

    inTemperatureMax = new QDoubleSpinBox();
    inTemperatureMax->setMinimum( WSMETEO_TEMPERATURE_MIN );
    inTemperatureMax->setMaximum( WSMETEO_TEMPERATURE_MAX );
    inTemperatureMax->setDisabled( true);
    inTemperatureMax->setAlignment( Qt::AlignCenter );
    inTemperatureMax->setFixedWidth( 50 );

    inTemperatureMaxAlarm = new QCheckBox();
    inTemperatureMaxAlarm->setText( trUtf8( "T° Intérieure Max" ));

    outTemperatureMin = new QDoubleSpinBox();
    outTemperatureMin->setMinimum( WSMETEO_TEMPERATURE_MIN );
    outTemperatureMin->setMaximum( WSMETEO_TEMPERATURE_MAX );
    outTemperatureMin->setDisabled( true);
    outTemperatureMin->setAlignment( Qt::AlignCenter );
    outTemperatureMin->setFixedWidth( 50 );

    outTemperatureMinAlarm = new QCheckBox();
    outTemperatureMinAlarm->setText( trUtf8( "T° Extérieure Min" ));

    outTemperatureMax = new QDoubleSpinBox();
    outTemperatureMax->setMinimum( WSMETEO_TEMPERATURE_MIN );
    outTemperatureMax->setMaximum( WSMETEO_TEMPERATURE_MAX );
    outTemperatureMax->setDisabled( true);
    outTemperatureMax->setAlignment( Qt::AlignCenter );
    outTemperatureMax->setFixedWidth( 50 );

    outTemperatureMaxAlarm = new QCheckBox();
    outTemperatureMaxAlarm->setText( trUtf8( "T° Extérieure Max" ));

    windChillTemperatureMin = new QDoubleSpinBox();
    windChillTemperatureMin->setMinimum( WSMETEO_TEMPERATURE_MIN );
    windChillTemperatureMin->setMaximum( WSMETEO_TEMPERATURE_MAX );
    windChillTemperatureMin->setDisabled( true);
    windChillTemperatureMin->setAlignment( Qt::AlignCenter );
    windChillTemperatureMin->setFixedWidth( 50 );

    windChillTemperatureMinAlarm = new QCheckBox();
    windChillTemperatureMinAlarm->setText( trUtf8( "T° Ressentie Min" ));

    windChillTemperatureMax = new QDoubleSpinBox();
    windChillTemperatureMax->setMinimum( WSMETEO_TEMPERATURE_MIN );
    windChillTemperatureMax->setMaximum( WSMETEO_TEMPERATURE_MAX );
    windChillTemperatureMax->setDisabled( true);
    windChillTemperatureMax->setAlignment( Qt::AlignCenter );
    windChillTemperatureMax->setFixedWidth( 50 );

    windChillTemperatureMaxAlarm = new QCheckBox();
    windChillTemperatureMaxAlarm->setText( trUtf8( "T° Ressentie Max" ));

    dewPointTemperatureMin = new QDoubleSpinBox();
    dewPointTemperatureMin->setMinimum( WSMETEO_TEMPERATURE_MIN );
    dewPointTemperatureMin->setMaximum( WSMETEO_TEMPERATURE_MAX );
    dewPointTemperatureMin->setDisabled( true);
    dewPointTemperatureMin->setAlignment( Qt::AlignCenter );
    dewPointTemperatureMin->setFixedWidth( 50 );

    dewPointTemperatureMinAlarm = new QCheckBox();
    dewPointTemperatureMinAlarm->setText( trUtf8( "T° du point de rosée Min" ));

    dewPointTemperatureMax = new QDoubleSpinBox();
    dewPointTemperatureMax->setMinimum( WSMETEO_TEMPERATURE_MIN );
    dewPointTemperatureMax->setMaximum( WSMETEO_TEMPERATURE_MAX );
    dewPointTemperatureMax->setDisabled( true);
    dewPointTemperatureMax->setAlignment( Qt::AlignCenter );
    dewPointTemperatureMax->setFixedWidth( 50 );

    dewPointTemperatureMaxAlarm = new QCheckBox();
    dewPointTemperatureMaxAlarm->setText( trUtf8( "T° du point de rosée Max" ));

    QFrame *separ1 = new QFrame;
    separ1->setFrameStyle(QFrame::HLine | QFrame::Raised);
    QFrame *separ2 = new QFrame;
    separ2->setFrameStyle(QFrame::HLine | QFrame::Raised);
    QFrame *separ3 = new QFrame;
    separ3->setFrameStyle(QFrame::HLine | QFrame::Raised);

    // Rain
    rainGroup = new QGroupBox( trUtf8( "Précipitations" ));

    rain1H = new QDoubleSpinBox();
    rain1H->setDisabled( true );
    rain1H->setAlignment( Qt::AlignCenter );
    rain1H->setFixedWidth( 50 );

    rain1HAlarm = new QCheckBox();
    rain1HAlarm->setText( trUtf8( "Précipitation 1H" ));

    rain1Day = new QDoubleSpinBox();
    rain1Day->setDisabled( true );
    rain1Day->setAlignment( Qt::AlignCenter );
    rain1Day->setFixedWidth( 50 );

    rain1DayAlarm = new QCheckBox();
    rain1DayAlarm->setText( trUtf8( "Précipitation 24H" ));

    // Humidity
    humidityGroup = new QGroupBox( trUtf8( "Humidité" ));

    inHumidityMin = new QSpinBox();
    inHumidityMin->setMinimum( 0 );         // 0%
    inHumidityMin->setMaximum( 100 );       // 100%
    inHumidityMin->setDisabled( true );
    inHumidityMin->setAlignment( Qt::AlignCenter );
    inHumidityMin->setFixedWidth( 50 );

    inHumidityMinAlarm = new QCheckBox();
    inHumidityMinAlarm->setText( trUtf8( "Humidité Intérieure Min" ));

    inHumidityMax = new QSpinBox();
    inHumidityMax->setMinimum( 0 );         // 0%
    inHumidityMax->setMaximum( 100 );       // 100%
    inHumidityMax->setDisabled( true );
    inHumidityMax->setAlignment( Qt::AlignCenter );
    inHumidityMax->setFixedWidth( 50 );

    inHumidityMaxAlarm = new QCheckBox();
    inHumidityMaxAlarm->setText( trUtf8( "Humidité Intérieure Max" ));

    QFrame *separ4 = new QFrame;
    separ4->setFrameStyle(QFrame::HLine | QFrame::Raised);

    outHumidityMin = new QSpinBox();
    outHumidityMin->setMinimum( 0 );         // 0%
    outHumidityMin->setMaximum( 100 );       // 100%
    outHumidityMin->setDisabled( true );
    outHumidityMin->setAlignment( Qt::AlignCenter );
    outHumidityMin->setFixedWidth( 50 );

    outHumidityMinAlarm = new QCheckBox();
    outHumidityMinAlarm->setText( trUtf8( "Humidité Extérieure Min" ));

    outHumidityMax = new QSpinBox();
    outHumidityMax->setMinimum( 0 );         // 0%
    outHumidityMax->setMaximum( 100 );       // 100%
    outHumidityMax->setDisabled( true );
    outHumidityMax->setAlignment( Qt::AlignCenter );
    outHumidityMax->setFixedWidth( 50 );

    outHumidityMaxAlarm = new QCheckBox();
    outHumidityMaxAlarm->setText( trUtf8( "Humidité Extérieure Max" ));

    // Pressure
    pressureGroup = new QGroupBox( trUtf8( "Pression" ));

    pressureMin = new QDoubleSpinBox();
    pressureMin->setDisabled( true );
    pressureMin->setAlignment( Qt::AlignCenter );
    pressureMin->setFixedWidth( 70 );
    pressureMin->setMaximum( 1500 );

    pressureMinAlarm = new QCheckBox();
    pressureMinAlarm->setText( trUtf8( "Pression Min" ));

    pressureMax = new QDoubleSpinBox();
    pressureMax->setDisabled( true );
    pressureMax->setAlignment( Qt::AlignCenter );
    pressureMax->setFixedWidth( 70 );
    pressureMax->setMaximum( 1500 );

    pressureMaxAlarm = new QCheckBox();
    pressureMaxAlarm->setText( trUtf8( "Pression Max" ));

    // Wind
    windGroup = new QGroupBox( trUtf8( "Vent" ));

    windSpeedMin = new QSpinBox();
    windSpeedMin->setDisabled( true );
    windSpeedMin->setAlignment( Qt::AlignCenter );
    windSpeedMin->setFixedWidth( 50 );

    windSpeedMinAlarm = new QCheckBox();
    windSpeedMinAlarm->setText( trUtf8( "Vitesse Min" ));

    windSpeedMax = new QSpinBox();
    windSpeedMax->setDisabled( true );
    windSpeedMax->setAlignment( Qt::AlignCenter );
    windSpeedMax->setFixedWidth( 50 );
    windSpeedMax->setMaximum( 1000 );

    windSpeedMaxAlarm = new QCheckBox();
    windSpeedMaxAlarm->setText( trUtf8( "Vitesse Max" ));

    windDirection = new QwtCompass( this );
    QPalette colorGroup;
    for ( int c = 0; c < QPalette::NColorRoles; c++ )
        colorGroup.setColor( ( QPalette::ColorRole ) c, QColor() );

    colorGroup.setColor( QPalette::Base, palette().color( backgroundRole() ).light( 120 ));
    colorGroup.setColor( QPalette::WindowText, colorGroup.color( QPalette::Base ));

    windDirection->setLineWidth( 5 );
    windDirection->setFrameShadow(  QwtCompass::Raised);
    windDirection->setScaleComponents( QwtAbstractScaleDraw::Ticks | QwtAbstractScaleDraw::Labels );
    windDirection->setScaleTicks(0, 0, 3);
    windDirection->setNeedle(new QwtCompassMagnetNeedle(  QwtCompassMagnetNeedle::TriangleStyle, Qt::white, Qt::red ));
    windDirection->setMinimumWidth( 150 );
    windDirection->setStep( 30 );

    windDirectionAlarm = new QCheckBox();
    windDirectionAlarm->setText( trUtf8( "Direction" ));

    // Set up layout
    QGridLayout *filesConfigurationLayout = new QGridLayout;
    filesConfigurationLayout->addWidget( label1, 0, 0 );
    filesConfigurationLayout->addWidget( dataFilesDirectoryLineEdit, 1, 0, 1, 3 );
    filesConfigurationLayout->addWidget( dataFilesDirectoryButton, 1, 4 );
    filesConfigurationLayout->addWidget( label3, 2, 0 );
    filesConfigurationLayout->addWidget( frequencyOfReadingSpinBox, 2, 1 );
    filesConfigurationLayout->addWidget( label4, 2, 2 );

    filesConfigurationGroup->setLayout( filesConfigurationLayout );


    QVBoxLayout *serialPortConfigurationLayout = new QVBoxLayout;
    serialPortConfigurationLayout->addWidget( label2 );
    serialPortConfigurationLayout->addWidget( deviceNameLineEdit );
    serialPortConfigurationGroup->setLayout( serialPortConfigurationLayout );

    QGridLayout *unitConfigurationLayout = new QGridLayout;
    unitConfigurationLayout->addWidget( label6, 0, 0 );
    unitConfigurationLayout->addWidget( languageBox, 1, 0 );
    unitConfigurationLayout->addWidget( label10, 2, 0 );
    unitConfigurationLayout->addWidget( label7, 2, 1 );
    unitConfigurationLayout->addWidget( label8, 2, 2 );
    unitConfigurationLayout->addWidget( label9, 2, 3 );
    unitConfigurationLayout->addWidget( rainUnitBox, 3, 0);
    unitConfigurationLayout->addWidget( windSpeedUnitBox, 3, 1);
    unitConfigurationLayout->addWidget( temperatureUnitBox, 3, 2);
    unitConfigurationLayout->addWidget( pressureUnitBox, 3, 3 );
    unitConfigurationGroup->setLayout( unitConfigurationLayout );

    QGridLayout *infoProgramLayout = new QGridLayout;
    infoProgramLayout->addWidget( label5, 0, 0, 4, 0 );
    infoProgramLayout->addWidget( programLabel, 0, 2 );
    infoProgramLayout->addWidget( versionLabel, 1, 2 );
    infoProgramLayout->addWidget( copyrightLabel, 2, 2 );
    infoProgramLayout->addWidget( briefLabel, 3, 2 );
    infoProgramGroup->setLayout( infoProgramLayout );

    QGridLayout *rainLayout = new QGridLayout;
    rainLayout->addWidget( rain1H, 0, 0 );
    rainLayout->addWidget( rain1HAlarm, 0, 1 );
    rainLayout->addWidget( rain1Day, 1, 0 );
    rainLayout->addWidget( rain1DayAlarm, 1, 1 );
    rainGroup->setLayout( rainLayout );

    QGridLayout *temperaturesLayout = new QGridLayout;
    temperaturesLayout->addWidget( inTemperatureMin, 0, 0 );
    temperaturesLayout->addWidget( inTemperatureMinAlarm, 0, 1 );
    temperaturesLayout->addWidget( inTemperatureMax, 1, 0 );
    temperaturesLayout->addWidget( inTemperatureMaxAlarm, 1, 1 );
    temperaturesLayout->addWidget( separ1, 2, 0, 1, 2 );
    temperaturesLayout->addWidget( outTemperatureMin, 3, 0 );
    temperaturesLayout->addWidget( outTemperatureMinAlarm, 3, 1 );
    temperaturesLayout->addWidget( outTemperatureMax, 4, 0 );
    temperaturesLayout->addWidget( outTemperatureMaxAlarm, 4, 1 );
    temperaturesLayout->addWidget( separ2, 5, 0, 1, 2 );
    temperaturesLayout->addWidget( windChillTemperatureMin, 6, 0 );
    temperaturesLayout->addWidget( windChillTemperatureMinAlarm, 6, 1 );
    temperaturesLayout->addWidget( windChillTemperatureMax, 7, 0 );
    temperaturesLayout->addWidget( windChillTemperatureMaxAlarm, 7, 1 );
    temperaturesLayout->addWidget( separ3, 8, 0, 1, 2 );
    temperaturesLayout->addWidget( dewPointTemperatureMin, 9, 0 );
    temperaturesLayout->addWidget( dewPointTemperatureMinAlarm, 9, 1 );
    temperaturesLayout->addWidget( dewPointTemperatureMax, 10, 0 );
    temperaturesLayout->addWidget( dewPointTemperatureMaxAlarm, 10, 1 );
    temperaturesGroup->setLayout( temperaturesLayout );

    QGridLayout *humidityLayout = new QGridLayout;
    humidityLayout->addWidget( inHumidityMin, 0, 0 );
    humidityLayout->addWidget( inHumidityMinAlarm, 0, 1 );
    humidityLayout->addWidget( inHumidityMax, 1, 0 );
    humidityLayout->addWidget( inHumidityMaxAlarm, 1, 1 );
    humidityLayout->addWidget( separ4, 2, 0, 1, 2 );
    humidityLayout->addWidget( outHumidityMin, 3, 0 );
    humidityLayout->addWidget( outHumidityMinAlarm, 3, 1 );
    humidityLayout->addWidget( outHumidityMax, 4, 0 );
    humidityLayout->addWidget( outHumidityMaxAlarm, 4, 1 );
    humidityGroup->setLayout( humidityLayout );

    QGridLayout *pressureLayout = new QGridLayout;
    pressureLayout->addWidget( pressureMin, 0, 0 );
    pressureLayout->addWidget( pressureMinAlarm, 0, 1 );
    pressureLayout->addWidget( pressureMax, 1, 0 );
    pressureLayout->addWidget( pressureMaxAlarm, 1, 1 );
    pressureGroup->setLayout( pressureLayout );

    QGridLayout *windLayout = new QGridLayout;
    windLayout->addWidget( windSpeedMin, 0, 0 );
    windLayout->addWidget( windSpeedMinAlarm, 0, 1 );
    windLayout->addWidget( windSpeedMax, 1, 0 );
    windLayout->addWidget( windSpeedMaxAlarm, 1, 1 );
    windLayout->addWidget( windDirection, 0, 2 );
    windLayout->addWidget( windDirectionAlarm, 1, 2 );
    windGroup->setLayout( windLayout );

    QGridLayout *stationConfigurationLayout = new QGridLayout;
    stationConfigurationLayout->addWidget( temperaturesGroup, 0, 0, 3, 1 );
    stationConfigurationLayout->addWidget( humidityGroup, 0, 1 );
    stationConfigurationLayout->addWidget( pressureGroup, 1, 1 );
    stationConfigurationLayout->addWidget( rainGroup, 2, 1 );
    stationConfigurationLayout->addWidget( windGroup, 3, 0, 1, 2 );
    stationConfigurationGroup->setLayout( stationConfigurationLayout );

    QSpacerItem* spacer1 = new QSpacerItem( 20, 20, QSizePolicy::Minimum,
       QSizePolicy::Expanding );

    QVBoxLayout *wsMeteoConfigurationLayout = new QVBoxLayout;
    wsMeteoConfigurationLayout->addWidget( filesConfigurationGroup );
    wsMeteoConfigurationLayout->addWidget( serialPortConfigurationGroup );
    wsMeteoConfigurationLayout->addWidget( unitConfigurationGroup );
    wsMeteoConfigurationLayout->addWidget( infoProgramGroup );
    wsMeteoConfigurationLayout->addItem( spacer1 );
    wsMeteoConfigurationGroup->setLayout( wsMeteoConfigurationLayout );

    QGridLayout *currentLayout = new QGridLayout();
    currentLayout->addWidget( wsMeteoConfigurationGroup, 0, 0 );
    currentLayout->addWidget( stationConfigurationGroup, 0, 1 );
    this->setLayout( currentLayout );

    // Set up slots
    connect( dataFilesDirectoryLineEdit, SIGNAL( textChanged( QString )), this, SLOT( dataFilesDirectoryChanged( QString )));
    connect( deviceNameLineEdit, SIGNAL( textChanged( QString )), this, SLOT( deviceNameLineEditChanged( QString )));
    connect( dataFilesDirectoryButton, SIGNAL( clicked() ), this, SLOT( dataFilesDirectoryButtonPushed()));
    connect( frequencyOfReadingSpinBox, SIGNAL( valueChanged( int ) ), this, SLOT( frequencyOfReadingChanged( int )));

    connect( languageBox, SIGNAL( currentIndexChanged( int ) ), this, SLOT( languageChanged ( int )));
    connect( windSpeedUnitBox, SIGNAL( currentIndexChanged( int ) ), this, SLOT( windSpeedUnitChanged ( int )));
    connect( temperatureUnitBox, SIGNAL( currentIndexChanged( int ) ), this, SLOT( temperatureUnitChanged ( int )));
    connect( pressureUnitBox, SIGNAL( currentIndexChanged( int ) ), this, SLOT( pressureUnitChanged ( int )));
    connect( rainUnitBox, SIGNAL( currentIndexChanged( int ) ), this, SLOT( rainUnitChanged ( int )));

    connect( inTemperatureMin, SIGNAL( valueChanged( double ) ), this, SLOT( inTemperatureMinChanged( double ) ));
    connect( inTemperatureMinAlarm, SIGNAL( clicked() ), this, SLOT( inTemperatureMinAlarmChanged()));
    connect( inTemperatureMax, SIGNAL( valueChanged( double ) ), this, SLOT( inTemperatureMaxChanged( double ) ));
    connect( inTemperatureMaxAlarm, SIGNAL( clicked() ), this, SLOT( inTemperatureMaxAlarmChanged() ));
    connect( outTemperatureMin, SIGNAL( valueChanged( double ) ), this, SLOT( outTemperatureMinChanged( double ) ));
    connect( outTemperatureMinAlarm, SIGNAL( clicked() ), this, SLOT( outTemperatureMinAlarmChanged() ));
    connect( outTemperatureMax, SIGNAL( valueChanged( double ) ), this, SLOT( outTemperatureMaxChanged( double ) ));
    connect( outTemperatureMaxAlarm, SIGNAL( clicked() ), this, SLOT( outTemperatureMaxAlarmChanged() ));
    connect( windChillTemperatureMin, SIGNAL( valueChanged( double ) ), this, SLOT( windChillTemperatureMinChanged( double ) ));
    connect( windChillTemperatureMinAlarm, SIGNAL( clicked() ), this, SLOT( windChillTemperatureMinAlarmChanged() ));
    connect( windChillTemperatureMax, SIGNAL( valueChanged( double ) ), this, SLOT( windChillTemperatureMaxChanged( double ) ));
    connect( windChillTemperatureMaxAlarm, SIGNAL( clicked() ), this, SLOT( windChillTemperatureMaxAlarmChanged() ));
    connect( dewPointTemperatureMin, SIGNAL( valueChanged( double ) ), this, SLOT( dewPointTemperatureMinChanged( double ) ));
    connect( dewPointTemperatureMinAlarm, SIGNAL( clicked() ), this, SLOT( dewPointTemperatureMinAlarmChanged() ));
    connect( dewPointTemperatureMax, SIGNAL( valueChanged( double ) ), this, SLOT( dewPointTemperatureMaxChanged( double ) ));
    connect( dewPointTemperatureMaxAlarm, SIGNAL( clicked() ), this, SLOT( dewPointTemperatureMaxAlarmChanged( ) ));
    connect( rain1H, SIGNAL( valueChanged( double ) ), this, SLOT( rain1HChanged( double ) ));
    connect( rain1HAlarm, SIGNAL( clicked() ), this, SLOT( rain1HAlarmChanged() ));
    connect( rain1Day, SIGNAL( valueChanged( double ) ), this, SLOT( rain1DayChanged( double ) ));
    connect( rain1DayAlarm, SIGNAL( clicked() ), this, SLOT( rain1DayAlarmChanged() ));
    connect( inHumidityMin, SIGNAL( valueChanged( int ) ), this, SLOT( inHumidityMinChanged( int ) ));
    connect( inHumidityMinAlarm, SIGNAL( clicked() ), this, SLOT( inHumidityMinAlarmChanged() ));
    connect( inHumidityMax, SIGNAL( valueChanged( int ) ), this, SLOT( inHumidityMaxChanged( int ) ));
    connect( inHumidityMaxAlarm, SIGNAL( clicked() ), this, SLOT( inHumidityMaxAlarmChanged() ));
    connect( outHumidityMin, SIGNAL( valueChanged( int ) ), this, SLOT( outHumidityMinChanged( int ) ));
    connect( outHumidityMinAlarm, SIGNAL( clicked() ), this, SLOT( outHumidityMinAlarmChanged() ));
    connect( outHumidityMax, SIGNAL( valueChanged( int ) ), this, SLOT( outHumidityMaxChanged( int ) ));
    connect( outHumidityMaxAlarm, SIGNAL( clicked() ), this, SLOT( outHumidityMaxAlarmChanged() ));
    connect( pressureMin, SIGNAL( valueChanged( double ) ), this, SLOT( pressureMinChanged( double ) ));
    connect( pressureMinAlarm, SIGNAL( clicked() ), this, SLOT( pressureMinAlarmChanged() ));
    connect( pressureMax, SIGNAL( valueChanged( double ) ), this, SLOT(pressureMaxChanged( double ) ));
    connect( pressureMaxAlarm, SIGNAL( clicked() ), this, SLOT( pressureMaxAlarmChanged() ));
    connect( windSpeedMin, SIGNAL( valueChanged( int ) ), this, SLOT( windSpeedMinChanged( int ) ));
    connect( windSpeedMinAlarm, SIGNAL( clicked() ), this, SLOT( windSpeedMinAlarmChanged() ));
    connect( windSpeedMax, SIGNAL( valueChanged( int ) ), this, SLOT( windSpeedMaxChanged( int ) ));
    connect( windSpeedMaxAlarm, SIGNAL( clicked() ), this, SLOT( windSpeedMaxAlarmChanged() ));
    connect( windDirection, SIGNAL( valueChanged( double ) ), this, SLOT( windDirectionChanged( double ) ));
    connect( windDirectionAlarm, SIGNAL( clicked() ), this, SLOT( windDirectionAlarmChanged() ));

    updateValues();
}

void ConfigurationTab::updateValues( void )
{
    inTemperatureMin->setValue( theStation->getInTemperatureMinConf() );
    inTemperatureMinAlarm->setChecked( theStation->getInTemperatureMinAlarmConf() );
    inTemperatureMax->setValue( theStation->getInTemperatureMaxConf() );
    inTemperatureMaxAlarm->setChecked( theStation->getInTemperatureMaxAlarmConf() );
    outTemperatureMin->setValue( theStation->getOutTemperatureMinConf() );
    outTemperatureMinAlarm->setChecked( theStation->getOutTemperatureMinAlarmConf() );
    outTemperatureMax->setValue( theStation->getOutTemperatureMaxConf() );
    outTemperatureMaxAlarm->setChecked( theStation->getOutTemperatureMaxAlarmConf() );
    windChillTemperatureMin->setValue( theStation->getWindChillTemperatureMinConf() );
    windChillTemperatureMinAlarm->setChecked( theStation->getWindChillTemperatureMinAlarmConf() );
    windChillTemperatureMax->setValue( theStation->getWindChillTemperatureMaxConf() );
    windChillTemperatureMaxAlarm->setChecked( theStation->getWindChillTemperatureMaxAlarmConf() );
    dewPointTemperatureMin->setValue( theStation->getDewPointTemperatureMinConf() );
    dewPointTemperatureMinAlarm->setChecked( theStation->getDewPointTemperatureMinAlarmConf() );
    dewPointTemperatureMax->setValue( theStation->getDewPointTemperatureMaxConf() );
    dewPointTemperatureMaxAlarm->setChecked( theStation->getDewPointTemperatureMaxAlarmConf() );

    rain1H->setValue( theStation->getRain1HConf() );
    rain1HAlarm->setChecked( theStation->getRain1HAlarmConf() );
    rain1Day->setValue( theStation->getRain1DayConf() );
    rain1DayAlarm->setChecked( theStation->getRain1DayAlarmConf() );

    inHumidityMin->setValue( theStation->getInHumidityMinConf() );
    inHumidityMinAlarm->setChecked( theStation->getInHumidityMinAlarmConf() );
    inHumidityMax->setValue( theStation->getInHumidityMaxConf() );
    inHumidityMaxAlarm->setChecked( theStation->getInHumidityMaxAlarmConf() );

    outHumidityMin->setValue( theStation->getOutHumidityMinConf() );
    outHumidityMinAlarm->setChecked( theStation->getOutHumidityMinAlarmConf() );
    outHumidityMax->setValue( theStation->getOutHumidityMaxConf() );
    outHumidityMaxAlarm->setChecked( theStation->getOutHumidityMaxAlarmConf() );

    pressureMin->setValue( theStation->getPressureMinConf() );
    pressureMinAlarm->setChecked( theStation->getPressureMinAlarmConf() );
    pressureMax->setValue( theStation->getPressureMaxConf() );
    pressureMaxAlarm->setChecked( theStation->getPressureMaxAlarmConf() );

    windSpeedMin->setValue( theStation->getWindSpeedMinConf() );
    windSpeedMinAlarm->setChecked( theStation->getWindSpeedMinAlarmConf() );
    windSpeedMax->setValue( theStation->getWindSpeedMaxConf() );
    windSpeedMaxAlarm->setChecked( theStation->getWindSpeedMaxAlarmConf() );

    windDirection->setValue( theStation->getWindDirectionConf() );

    windDirectionAlarm->setChecked( theStation->getWindDirectionAlarmConf() );

}

void ConfigurationTab::languageChanged (int val)
{
    theConfiguration->setProgramLanguage( val );
}

void ConfigurationTab::windSpeedUnitChanged ( int val)
{
    theConfiguration->setWindSpeedUnit( val );
}

void ConfigurationTab::rainUnitChanged (int val)
{
    theConfiguration->setRainUnit( val );
}

void ConfigurationTab::temperatureUnitChanged ( int val)
{
    theConfiguration->setTemperatureUnit( val );
}

void ConfigurationTab::pressureUnitChanged ( int val)
{
    theConfiguration->setPressureUnit( val );
}

void ConfigurationTab::inTemperatureMinChanged( double val )
{
    theStation->setInTemperatureMinConf( val );
}

void ConfigurationTab::inTemperatureMaxChanged( double val )
{
    theStation->setInTemperatureMaxConf( val );
}

void ConfigurationTab::inTemperatureMinAlarmChanged()
{
    if( this->inTemperatureMinAlarm->isChecked() )
        this->inTemperatureMinAlarm->setChecked( false );
    else
        this->inTemperatureMinAlarm->setChecked( true );
}

void ConfigurationTab::inTemperatureMaxAlarmChanged()
{
    if( this->inTemperatureMaxAlarm->isChecked() )
        this->inTemperatureMaxAlarm->setChecked( false );
    else
        this->inTemperatureMaxAlarm->setChecked( true );
}

void ConfigurationTab::outTemperatureMinChanged( double val )
{
    theStation->setOutTemperatureMinConf( val );
}

void ConfigurationTab::outTemperatureMaxChanged( double val )
{
    theStation->setOutTemperatureMaxConf( val );
}

void ConfigurationTab::outTemperatureMinAlarmChanged()
{
    if( this->outTemperatureMinAlarm->isChecked() )
        this->outTemperatureMinAlarm->setChecked( false );
    else
        this->outTemperatureMinAlarm->setChecked( true );
}

void ConfigurationTab::outTemperatureMaxAlarmChanged()
{
    if( this->outTemperatureMaxAlarm->isChecked() )
        this->outTemperatureMaxAlarm->setChecked( false );
    else
        this->outTemperatureMaxAlarm->setChecked( true );
}

void ConfigurationTab::windChillTemperatureMinChanged( double val )
{
    theStation->setWindChillTemperatureMinConf( val );
}

void ConfigurationTab::windChillTemperatureMaxChanged( double val )
{
    theStation->setWindChillTemperatureMaxConf( val );
}

void ConfigurationTab::windChillTemperatureMinAlarmChanged()
{
    if( this->windChillTemperatureMinAlarm->isChecked() )
        this->windChillTemperatureMinAlarm->setChecked( false );
    else
        this->windChillTemperatureMinAlarm->setChecked( true );
}

void ConfigurationTab::windChillTemperatureMaxAlarmChanged()
{
    if( this->windChillTemperatureMaxAlarm->isChecked() )
        this->windChillTemperatureMaxAlarm->setChecked( false );
    else
        this->windChillTemperatureMaxAlarm->setChecked( true );
 }

void ConfigurationTab::dewPointTemperatureMinChanged( double val )
{
    theStation->setDewPointTemperatureMinConf( val );
}

void ConfigurationTab::dewPointTemperatureMaxChanged( double val )
{
    theStation->setDewPointTemperatureMaxConf( val );
}

void ConfigurationTab::dewPointTemperatureMinAlarmChanged()
{
    if( this->dewPointTemperatureMinAlarm->isChecked() )
        this->dewPointTemperatureMinAlarm->setChecked( false );
    else
        this->dewPointTemperatureMinAlarm->setChecked( true );

}

void ConfigurationTab::dewPointTemperatureMaxAlarmChanged()
{
    if( this->dewPointTemperatureMaxAlarm->isChecked() )
        this->dewPointTemperatureMaxAlarm->setChecked( false );
    else
        this->dewPointTemperatureMaxAlarm->setChecked( true );
}

void ConfigurationTab::pressureMinChanged( double val )
{
    theStation->setPressureMinConf( val );
}

void ConfigurationTab::pressureMaxChanged( double val )
{
    theStation->setPressureMaxConf( val );
}

void ConfigurationTab::pressureMinAlarmChanged()
{
    if( this->pressureMinAlarm->isChecked() )
        this->pressureMinAlarm->setChecked( false );
    else
        this->pressureMinAlarm->setChecked( true );
}

void ConfigurationTab::pressureMaxAlarmChanged()
{
    if( this->pressureMaxAlarm->isChecked() )
        this->pressureMaxAlarm->setChecked( false );
    else
        this->pressureMaxAlarm->setChecked( true );
}

void ConfigurationTab::inHumidityMinChanged( int val )
{
    theStation->setInHumidityMinConf( val );
}

void ConfigurationTab::inHumidityMaxChanged( int val )
{
    theStation->setInHumidityMaxConf( val );
}

void ConfigurationTab::inHumidityMinAlarmChanged()
{
    if( this->inHumidityMinAlarm->isChecked() )
        this->inHumidityMinAlarm->setChecked( false );
    else
        this->inHumidityMinAlarm->setChecked( true );
}

void ConfigurationTab::inHumidityMaxAlarmChanged()
{
    if( this->inHumidityMaxAlarm->isChecked() )
        this->inHumidityMaxAlarm->setChecked( false );
    else
        this->inHumidityMaxAlarm->setChecked( true );
}

void ConfigurationTab::outHumidityMinChanged( int val )
{
    theStation->setOutHumidityMinConf( val );
}

void ConfigurationTab::outHumidityMaxChanged( int val )
{
    theStation->setOutHumidityMaxConf( val );
}

void ConfigurationTab::outHumidityMinAlarmChanged()
{
    if( this->outHumidityMinAlarm->isChecked() )
        this->outHumidityMinAlarm->setChecked( false );
    else
        this->outHumidityMinAlarm->setChecked( true );
}

void ConfigurationTab::outHumidityMaxAlarmChanged()
{
    if( this->outHumidityMaxAlarm->isChecked() )
        this->outHumidityMaxAlarm->setChecked( false );
    else
        this->outHumidityMaxAlarm->setChecked( true );
}

void ConfigurationTab::rain1HChanged( double val )
{
    theStation->setRain1HConf( val );
}

void ConfigurationTab::rain1HAlarmChanged()
{
    if( this->rain1HAlarm->isChecked() )
        this->rain1HAlarm->setChecked( false );
    else
        this->rain1HAlarm->setChecked( true );
}

void ConfigurationTab::rain1DayChanged( double val )
{
    theStation->setRain1DayConf( val );
}

void ConfigurationTab::rain1DayAlarmChanged()
{
    if( this->rain1DayAlarm->isChecked() )
        this->rain1DayAlarm->setChecked( false );
    else
        this->rain1DayAlarm->setChecked( true );
}

void ConfigurationTab::windSpeedMinChanged( int val )
{
    theStation->setWindSpeedMinConf( val );
}

void ConfigurationTab::windSpeedMaxChanged( int val )
{
    theStation->setWindSpeedMaxConf( val );
}

void ConfigurationTab::windSpeedMinAlarmChanged()
{
    if( this->windSpeedMinAlarm->isChecked() )
        this->windSpeedMinAlarm->setChecked( false );
    else
        this->windSpeedMinAlarm->setChecked( true );
}

void ConfigurationTab::windSpeedMaxAlarmChanged()
{
    if( this->windSpeedMaxAlarm->isChecked() )
        this->windSpeedMaxAlarm->setChecked( false );
    else
        this->windSpeedMaxAlarm->setChecked( true );
}

void ConfigurationTab::windDirectionChanged( double val )
{
    theStation->setWindDirectionConf ( val );
}

void ConfigurationTab::windDirectionAlarmChanged()
{
    if( this->windDirectionAlarm->isChecked() )
        this->windDirectionAlarm->setChecked( false );
    else
        this->windDirectionAlarm->setChecked( true );
}

void ConfigurationTab::dataFilesDirectoryChanged( QString dir )
{
    QDir d = dir;

    if( d.exists() )
        theConfiguration->setDataFilesDirectory( dir );

}

void ConfigurationTab::deviceNameLineEditChanged( QString name )
{
    QFile f( name );

    if( f.exists() )
    {
        theStation->getTheConnection()->setDeviceName( name );
        deviceNameLineEdit->setFont( QFont( "Arial", 10, QFont::Bold ));
    }
    else
    {
        deviceNameLineEdit->setFont( QFont( "Arial", 10, QFont:: Normal ));

    }
}

void ConfigurationTab::dataFilesDirectoryButtonPushed()
{
    QString path = QFileDialog::getExistingDirectory (this, trUtf8( "Répertoire" ), theConfiguration->getDataFilesDirectory().path() );

    if ( path.isNull() == false )
    {
        theConfiguration->setDataFilesDirectory( path );
        dataFilesDirectoryLineEdit->setText( path );
    }
}

void ConfigurationTab::frequencyOfReadingChanged( int val )
{
    theConfiguration->setfrequencyOfReading( val );
}
