/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CONFIGURATIONTAB_H
#define CONFIGURATIONTAB_H

#include <QFrame>
#include <QtGui>
#include "Ws2355.h"
#include "Configuration.h"
#include <qwt_compass.h>

class ConfigurationTab : public QFrame
{
    Q_OBJECT
public:
    explicit ConfigurationTab( QWidget *parent = 0, Ws2355 *station = NULL, Configuration *config = NULL );
    ~ConfigurationTab();
    void            updateValues( void );
signals:

public slots:
    void                changeEvent(QEvent *event);

private slots:
    void                dataFilesDirectoryChanged( QString );
    void                deviceNameLineEditChanged( QString );
    void                dataFilesDirectoryButtonPushed();
    void                frequencyOfReadingChanged( int );

    void                languageChanged ( int );
    void                windSpeedUnitChanged ( int );
    void                temperatureUnitChanged ( int );
    void                pressureUnitChanged ( int );
    void                rainUnitChanged ( int );

    void                inTemperatureMinChanged( double );
    void                inTemperatureMaxChanged( double );
    void                inTemperatureMinAlarmChanged();
    void                inTemperatureMaxAlarmChanged();

    void                outTemperatureMinChanged( double );
    void                outTemperatureMaxChanged( double );
    void                outTemperatureMinAlarmChanged();
    void                outTemperatureMaxAlarmChanged();

    void                windChillTemperatureMinChanged( double );
    void                windChillTemperatureMaxChanged( double );
    void                windChillTemperatureMinAlarmChanged();
    void                windChillTemperatureMaxAlarmChanged();

    void                dewPointTemperatureMinChanged( double );
    void                dewPointTemperatureMaxChanged( double );
    void                dewPointTemperatureMinAlarmChanged();
    void                dewPointTemperatureMaxAlarmChanged();

    void                pressureMinChanged( double );
    void                pressureMaxChanged( double );
    void                pressureMinAlarmChanged();
    void                pressureMaxAlarmChanged();

    void                inHumidityMinChanged( int );
    void                inHumidityMaxChanged( int );
    void                inHumidityMinAlarmChanged();
    void                inHumidityMaxAlarmChanged();

    void                outHumidityMinChanged( int );
    void                outHumidityMaxChanged( int );
    void                outHumidityMinAlarmChanged();
    void                outHumidityMaxAlarmChanged();

    void                rain1HChanged( double );
    void                rain1HAlarmChanged();

    void                rain1DayChanged( double );
    void                rain1DayAlarmChanged();

    void                windSpeedMinChanged( int );
    void                windSpeedMaxChanged( int );
    void                windSpeedMinAlarmChanged();
    void                windSpeedMaxAlarmChanged();

    void                windDirectionChanged( double );
    void                windDirectionAlarmChanged();

private:
    void            createTab( void );


    Ws2355          *theStation;
    Configuration   *theConfiguration;

    QLineEdit       *dataFilesDirectoryLineEdit;
    QPushButton     *dataFilesDirectoryButton;

    QComboBox       *languageBox;
    QComboBox       *windSpeedUnitBox;
    QComboBox       *temperatureUnitBox;
    QComboBox       *pressureUnitBox;
    QComboBox       *rainUnitBox;

    QSpinBox        *frequencyOfReadingSpinBox;

    QLineEdit       *deviceNameLineEdit;

    QDoubleSpinBox  *inTemperatureMin;
    QDoubleSpinBox  *inTemperatureMax;
    QCheckBox       *inTemperatureMinAlarm;
    QCheckBox       *inTemperatureMaxAlarm;

    QDoubleSpinBox  *outTemperatureMin;
    QDoubleSpinBox  *outTemperatureMax;
    QCheckBox       *outTemperatureMinAlarm;
    QCheckBox       *outTemperatureMaxAlarm;

    QDoubleSpinBox  *windChillTemperatureMin;
    QDoubleSpinBox  *windChillTemperatureMax;
    QCheckBox       *windChillTemperatureMinAlarm;
    QCheckBox       *windChillTemperatureMaxAlarm;

    QDoubleSpinBox  *dewPointTemperatureMin;
    QDoubleSpinBox  *dewPointTemperatureMax;
    QCheckBox       *dewPointTemperatureMinAlarm;
    QCheckBox       *dewPointTemperatureMaxAlarm;

    QDoubleSpinBox  *pressureMin;
    QDoubleSpinBox  *pressureMax;
    QCheckBox       *pressureMinAlarm;
    QCheckBox       *pressureMaxAlarm;

    QSpinBox        *inHumidityMin;
    QSpinBox        *inHumidityMax;
    QCheckBox       *inHumidityMinAlarm;
    QCheckBox       *inHumidityMaxAlarm;

    QSpinBox        *outHumidityMin;
    QSpinBox        *outHumidityMax;
    QCheckBox       *outHumidityMinAlarm;
    QCheckBox       *outHumidityMaxAlarm;


    QDoubleSpinBox  *rain1H;
    QCheckBox       *rain1HAlarm;

    QDoubleSpinBox  *rain1Day;
    QCheckBox       *rain1DayAlarm;

    QSpinBox        *windSpeedMin;
    QSpinBox        *windSpeedMax;
    QCheckBox       *windSpeedMinAlarm;
    QCheckBox       *windSpeedMaxAlarm;

    QwtCompass      *windDirection;
    QCheckBox       *windDirectionAlarm;


    QGroupBox       *wsMeteoConfigurationGroup;
    QLabel          *label1;
    QGroupBox       *filesConfigurationGroup;
    QLabel          *label3;
    QLabel          *label4;
    QGroupBox       *serialPortConfigurationGroup;
    QLabel          *label2;
    QGroupBox       *unitConfigurationGroup;
    QLabel          *label6;
    QLabel          *label7;
    QLabel          *label8;
    QLabel          *label9;
    QLabel          *label10;
    QLabel          *briefLabel;
    QGroupBox       *stationConfigurationGroup;
    QGroupBox       *temperaturesGroup;
    QGroupBox       *rainGroup;
    QGroupBox       *humidityGroup;
    QGroupBox       *pressureGroup;
    QGroupBox       *windGroup;
};

#endif // CONFIGURATIONTAB_H
