/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>

#include <termios.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netdb.h>
#include "WsMeteo.h"
#include <QSettings>
#include <QDebug>
#include <QFile>
#include "Connection.h"

Connection::Connection()
{
    readSettings();
}

Connection::~Connection()
{
    writeSettings();
}

void Connection::readSettings( void )
{
    QSettings settings("Gil", "WsMeteo");
    QString str;

    str = settings.value( "deviceName" ).toString();
    if( str == "" )
    {
        deviceName = DEFAULT_DEVICENAME;
    }
    else
    {
        deviceName = str;
    }

}

void Connection::writeSettings( void )
{
    QSettings settings("Gil", "WsMeteo");

    settings.setValue( "deviceName", deviceName.toAscii() );
}

void Connection::update( void )
{
    if( isConnected() != 0 )
    {
        qDebug() << QObject::trUtf8( "Port déconnecté" );
    }
    else
    {
        qDebug() << "Port OK";
    }
}

/**
 * Test if a station is connected to the serial port
 *
 * @param deviceName (ie. /dev/ttyUSB0 etc.)
 *
 * @return status = true if a station is connected else status is false
 */
bool Connection::isConnected( void )
{
    QFile f( deviceName.toAscii() );

    if( f.exists() )
        return true;
    else
        return false;
    /*
    int fd;
    bool status;

    fd = open( deviceName.toAscii(), O_RDWR | O_NOCTTY | O_NDELAY );

    if (ioctl(fd, TIOCMGET, &status) == -1)
    {
        status = false;
    }
    else
    {
        if (status & TIOCM_DTR)
        {
            // The station is connected
            status = true;
        }
        else
        {
            // The station is disconnected
            status = false;
        }
    }
//    close(fd);

    return status;*/
}

/**
 * Opens the serial link to communicate with the station
 *
 * @return status = OK
 *                  ERROR_UNABLE_TO_OPEN_SERIAL_PORT
 *                  ERROR_SERIAL_PORT_LOCKED
 *                  ERROR_COULDN_T_RESET_NON_BLOCKING_MODE
 *                  ERROR_ENABLE_TO_INITIALIZE_SERIAL_PORT
 */
int Connection::openCommunication( int &stationDescriptor  )
{
    struct termios adtio;
    int portstatus, fdflags;

    //Setup serial port
    if (( stationDescriptor = open( deviceName.toAscii(), O_RDWR | O_NONBLOCK )) < 0 )
    {
        return( ERROR_UNABLE_TO_OPEN_SERIAL_PORT );
    }

    if ( flock( stationDescriptor, LOCK_EX | LOCK_NB ) < 0 )
    {
        return( ERROR_SERIAL_PORT_LOCKED );
    }

    if (( fdflags = fcntl( stationDescriptor, F_GETFL )) == -1 ||
          fcntl( stationDescriptor, F_SETFL, fdflags & ~ O_NONBLOCK ) < 0)
    {
        return( ERROR_COULDN_T_RESET_NON_BLOCKING_MODE );
    }

    //We want full control of what is set and simply reset the entire adtio struct
    memset( &adtio, 0, sizeof( adtio ));

    //tcgetattr(ws2300, &adtio);   // Commented out and replaced by the memset above

    // Serial control options
    adtio.c_cflag &= ~PARENB;      // No parity
    adtio.c_cflag &= ~CSTOPB;      // One stop bit
    adtio.c_cflag &= ~CSIZE;       // Character size mask
    adtio.c_cflag |= CS8;          // Character size 8 bits
    adtio.c_cflag |= CREAD;        // Enable Receiver
    adtio.c_cflag &= ~HUPCL;       // No "hangup"
    adtio.c_cflag &= ~CRTSCTS;     // No flowcontrol
    adtio.c_cflag |= CLOCAL;       // Ignore modem control lines

    // Baudrate, for newer systems
    cfsetispeed( &adtio, DEFAULT_BAUDRATE );
    cfsetospeed( &adtio, DEFAULT_BAUDRATE );

    // Serial local options: adtio.c_lflag
    // Raw input = clear ICANON, ECHO, ECHOE, and ISIG
    // Disable misc other local features = clear FLUSHO, NOFLSH, TOSTOP, PENDIN, and IEXTEN
    // So we actually clear all flags in adtio.c_lflag
    adtio.c_lflag = 0;

    // Serial input options: adtio.c_iflag
    // Disable parity check = clear INPCK, PARMRK, and ISTRIP
    // Disable software flow control = clear IXON, IXOFF, and IXANY
    // Disable any translation of CR and LF = clear INLCR, IGNCR, and ICRNL
    // Ignore break condition on input = set IGNBRK
    // Ignore parity errors just in case = set IGNPAR;
    // So we can clear all flags except IGNBRK and IGNPAR
    adtio.c_iflag = IGNBRK | IGNPAR;

    // Serial output options
    // Raw output should disable all other output options
    adtio.c_oflag &= ~OPOST;

    adtio.c_cc[ VTIME ] = 10;		// timer 1s
    adtio.c_cc[ VMIN ] = 0;		// blocking read until 1 char

    if ( tcsetattr( stationDescriptor, TCSANOW, &adtio ) < 0 )
    {
        return( ERROR_ENABLE_TO_INITIALIZE_SERIAL_PORT );
    }

    tcflush( stationDescriptor, TCIOFLUSH );

    // Set DTR low and RTS high and leave other ctrl lines untouched
    ioctl( stationDescriptor, TIOCMGET, &portstatus );	// get current port status
    portstatus &= ~TIOCM_DTR;
    portstatus |= TIOCM_RTS;
    ioctl( stationDescriptor, TIOCMSET, &portstatus );	// set current port status

    return OK;
}
