/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CURRENTDATA_H
#define CURRENTDATA_H

#include <QString>
#include <QLabel>

// define the readings collector
typedef struct
{
    float           indoorTemperature;
    float           outdoorTemperature;
    float           indoorHumidity;
    float           outdoorHumidity;
    float           windChillTemperature;
    float           dewPointTemperature;
    float           windDirection;
    float           windSpeed;
    float           pressureAbsolute;
    float           pressureRelative;
    float           rain1Hour;
    float           rain1Day;
    float           rainTotal;
    unsigned int    weatherForecast;    // 0:"Rainy",  1:"Cloudy", 2:"Sunny"
    unsigned int    weatherTendency;    // 0:"Steady", 1:"Rising", 2:"Falling"

} ws2355DataType;

#define CURRENT_DATA_STATE_DEFAULT          0
#define CURRENT_DATA_STATE_DATA_READING     1

class CurrentData
{
public:
    CurrentData();
    ~CurrentData();

    void                startDataReading( void ) { state = CURRENT_DATA_STATE_DATA_READING; }
    void                stopDataReading( void ) { state = CURRENT_DATA_STATE_DEFAULT; }
    bool                dataReadingStateIsStarted ( void ) { return state == CURRENT_DATA_STATE_DATA_READING; }

    float               getIndoorTemperature( void ) { return data.indoorTemperature; }
    float               getOutdoorTemperature( void ) { return data.outdoorTemperature; }
    float               getIndoorHumidity( void ) { return data.indoorHumidity; }
    float               getOutdoorHumidity( void ) { return data.outdoorHumidity; }
    float               getWindChillTemperature( void ) { return data.windChillTemperature; }
    float               getDewPointTemperature( void ) { return data.dewPointTemperature; }
    float               getWindDirection( void ) { return data.windDirection; }
    float               getWindSpeed( void ) { return data.windSpeed; }
    float               getPressureAbsolute( void ) { return data.pressureAbsolute; }
    float               getPressureRelative( void ) { return data.pressureRelative; }
    float               getRain1Hour( void ) { return data.rain1Hour; }
    float               getRain1Day( void ) { return data.rain1Day; }
    float               getRainTotal( void ) { return data.rainTotal; }

    unsigned int        getWeatherForecast( void ) { return data.weatherForecast; }
    unsigned int        getWeatherTendency( void ) { return data.weatherTendency; }

    void                setIndoorTemperature( float val  ) { data.indoorTemperature = val; }
    void                setOutdoorTemperature( float val ) { data.outdoorTemperature = val; }
    void                setIndoorHumidity( float val ) { data.indoorHumidity = val; }
    void                setOutdoorHumidity( float val ) { data.outdoorHumidity = val; }
    void                setWindChillTemperature( float val ) { data.windChillTemperature = val; }
    void                setDewPointTemperature( float val ) { data.dewPointTemperature = val; }

    void                setWeatherForecast( unsigned int val ) { data.weatherForecast = val; }
    void                setWeatherTendency( unsigned int val ) { data.weatherTendency = val; }

    void                setWindDirection( float val ) { data.windDirection = val; }
    void                setWindSpeed( float val ) { data.windSpeed = val; }
    void                setPressureAbsolute( float val ) { data.pressureAbsolute = val; }
    void                setPressureRelative( float val ) { data.pressureRelative = val; }
    void                setRain1Hour( float val ) { data.rain1Hour = val; }
    void                setRain1Day( float val ) { data.rain1Day = val; }
    void                setRainTotal( float val ) { data.rainTotal = val; }

private:
    ws2355DataType      data;
    unsigned int        state;
};

#endif // CURRENTDATA_H
