/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <QSlider>
#include <QPixmap>
#include <QTextFormat>

#include <qwt_compass.h>
#include <qwt_compass_rose.h>
#include <qwt_dial_needle.h>

#include "CurrentDataTab.h"
#include "RotatedQLabel.h"
#include "CurrentData.h"
#include "Ws2355.h"
#include "WsMeteo.h"

CurrentDataTab::CurrentDataTab( QWidget *parent, Ws2355 *station, Configuration *config ) :
    QFrame(parent)
{
    theStation = station;
    theCurrentData = theStation->getTheCurrentData();
    theGlobalConfiguration = config;

    createTab();
    updateSensorsValues( );
}

/**
 * Destructor
 */
CurrentDataTab::~CurrentDataTab()
{
    delete currentIndoorTemperatureSlider;
    delete currentOutdoorTemperatureSlider;
    delete currentWindChillTemperatureSlider;
    delete currentDewPointTemperatureSlider;

    delete currentIndoorTemperatureLCDNumber;
    delete currentOutdoorTemperatureLCDNumber;
    delete currentWindChillTemperatureLCDNumber;
    delete currentDewPointTemperatureLCDNumber;

    delete currentRain1HourProgressBar;
    delete currentRain1DayProgressBar;
    delete currentRainTotalProgressBar;

    delete currentRain1HourLabel;
    delete currentRain1DayLabel;
    delete currentRainTotalLabel;

    delete currentIndoorHumiditySlider;
    delete currentOutdoorHumiditySlider;
    delete currentIndoorHumidityLCDNumber;
    delete currentOutdoorHumidityLCDNumber;

    delete currentPressureLabel;
    delete currentTendencyLabel;
    delete currentForecastlLabel;

    delete currentWindDirectionLabel;
    delete currentWindDirectionDial;
    delete currentWindSpeedLabel;

    delete firstDataReadingDateLabel;
    delete lastDataReadingDateLabel;
    delete nextDataReadingDateLabel;
    delete connectionStatusLabel;

    delete currentTempGroup;
    delete label1;
    delete label2;
    delete label3;
    delete label4;
    delete currentRainGroup;
    delete label10;
    delete label11;
    delete label12;
    delete currentHumidityGroup;
    delete currentForecastGroup;
    delete label30;
    delete currentWindGroup;
    delete dataReadingGroup;
    delete label40;
    delete label41;
    delete label42;
    delete label20;
    delete label21;
}


void CurrentDataTab::setDataReadingButtonText( QString newText )
{
    dataReadingButton->setText( newText );
}

/**
 * When the language change, update all text fields.
 * Capture QEvent::LanguageChange and forward all other events
 *
 * @param event
 */
void CurrentDataTab::changeEvent(QEvent *event) {
    if (event->type() == QEvent::LanguageChange)
    {
        // Set all texts
        currentTempGroup->setTitle( trUtf8( "Températures" ));
        label1->setText( trUtf8( "Intérieure" ));
        label2->setText( trUtf8( "Extérieure" ));
        label3->setText( trUtf8( "Ressentie" ));
        label4->setText( trUtf8( "Point de rosée" ));
        currentRainGroup->setTitle( trUtf8( "Précipitations" ));
        label10->setText( trUtf8( "1 heure" ));
        label11->setText( trUtf8( "1 jour" ));
        label12->setText( trUtf8( "Total" ));
        currentHumidityGroup->setTitle( trUtf8( "Humidité" ));
        label20->setText( trUtf8( "Intérieure" ));
        label21->setText( trUtf8( "Extérieure" ));
        currentForecastGroup->setTitle( trUtf8( "Pression/Prévisions" ));
        label30->setText( tr( "Pression" ));
        currentWindGroup->setTitle( trUtf8( "Vent" ));
        dataReadingGroup->setTitle( trUtf8( "Relevés" ));
        label40->setText( trUtf8( "Premier relevé le " ));
        label41->setText( trUtf8( "Dernier relevé le " ));
        label42->setText( trUtf8( "Prochain relevé dans " ));
        dataReadingButton->setText( trUtf8( "Relevé" ));
    }
    else
    {
        QWidget::changeEvent(event);
    }
}


/**
 * Create all widgets display in this tab
 */
void CurrentDataTab::createTab( void )
{
    // Temperature
    currentTempGroup = new QGroupBox( trUtf8( "Températures" ));

    currentIndoorTemperatureSlider = new QSlider( Qt::Vertical );
    currentIndoorTemperatureSlider->setTickInterval( 5 );
    currentIndoorTemperatureSlider->setTickPosition( QSlider::TicksRight );
    currentIndoorTemperatureSlider->setRange( -20, 50 );

    currentOutdoorTemperatureSlider = new QSlider( Qt::Vertical );
    currentOutdoorTemperatureSlider->setTickInterval( 5 );
    currentOutdoorTemperatureSlider->setTickPosition( QSlider::TicksRight );
    currentOutdoorTemperatureSlider->setRange( -20, 50 );

    currentWindChillTemperatureSlider = new QSlider( Qt::Vertical );
    currentWindChillTemperatureSlider->setTickInterval( 5 );
    currentWindChillTemperatureSlider->setTickPosition( QSlider::TicksRight );
    currentWindChillTemperatureSlider->setRange( -20, 50 );

    currentDewPointTemperatureSlider = new QSlider( Qt::Vertical );
    currentDewPointTemperatureSlider->setTickInterval( 5 );
    currentDewPointTemperatureSlider->setTickPosition( QSlider::TicksRight );
    currentDewPointTemperatureSlider->setRange( -20, 50 );

    currentIndoorTemperatureLCDNumber = new QLCDNumber( 5 );
    currentOutdoorTemperatureLCDNumber = new QLCDNumber( 5 );
    currentWindChillTemperatureLCDNumber = new QLCDNumber( 5 );
    currentDewPointTemperatureLCDNumber = new QLCDNumber( 5 );

    label1 = new RotatedQLabel();
    label1->setFixedWidth( 20);
    label1->setText( trUtf8( "Intérieure" ));
    label1->setAlignment( Qt::AlignHCenter );
    label2 = new RotatedQLabel();
    label2->setFixedWidth( 20);
    label2->setText( trUtf8( "Extérieure" ));
    label2->setAlignment( Qt::AlignHCenter );
    label3 = new RotatedQLabel();
    label3->setFixedWidth( 20);
    label3->setText( trUtf8( "Ressentie" ));
    label3->setAlignment( Qt::AlignHCenter );
    label4 = new RotatedQLabel();
    label4->setFixedWidth( 20);
    label4->setText( trUtf8( "Point de rosée" ));
    label4->setAlignment( Qt::AlignHCenter );

    // Rain
    currentRainGroup = new QGroupBox( trUtf8( "Précipitations" ));
    currentRainGroup->setMinimumHeight( 300);
    currentRain1HourProgressBar =  new QProgressBar();
    currentRain1HourProgressBar->setOrientation( Qt::Vertical );
    currentRain1HourProgressBar->setRange( 0, 80 );
    currentRain1DayProgressBar =  new QProgressBar();
    currentRain1DayProgressBar->setOrientation( Qt::Vertical );
    currentRain1DayProgressBar->setRange( 0, 200 );
    currentRainTotalProgressBar =  new QProgressBar();
    currentRainTotalProgressBar->setOrientation( Qt::Vertical );
    currentRainTotalProgressBar->setRange( 0, 1000 );

    label10 = new QLabel("1 heure");
    label10->setAlignment( Qt::AlignHCenter );
    label11 = new QLabel("1 jour");
    label11->setAlignment( Qt::AlignHCenter );
    label12 = new QLabel("Total");
    label12->setAlignment( Qt::AlignHCenter );

    currentRain1HourLabel = new QLabel();
    currentRain1DayLabel = new QLabel();
    currentRainTotalLabel = new QLabel();

    // Humidity
    currentHumidityGroup = new QGroupBox( trUtf8( "Humidité" ));

    currentIndoorHumiditySlider = new QSlider( Qt::Vertical );
    currentIndoorHumiditySlider->setTickInterval( 10 );
    currentIndoorHumiditySlider->setTickPosition( QSlider::TicksRight );
    currentIndoorHumiditySlider->setRange( 0, 100 );

    currentOutdoorHumiditySlider = new QSlider( Qt::Vertical );
    currentOutdoorHumiditySlider->setTickInterval( 10 );
    currentOutdoorHumiditySlider->setTickPosition( QSlider::TicksRight );
    currentOutdoorHumiditySlider->setRange( 0, 100 );

    currentIndoorHumidityLCDNumber = new QLCDNumber( 5 );
    currentOutdoorHumidityLCDNumber = new QLCDNumber( 5 );

    label20 = new RotatedQLabel();
    label20->setFixedWidth( 20);
    label20->setText( trUtf8( "Intérieure" ));
    label20->setAlignment( Qt::AlignHCenter );
    label21 = new RotatedQLabel();
    label21->setFixedWidth( 20);
    label21->setText( trUtf8( "Extérieure" ));
    label21->setAlignment( Qt::AlignHCenter );

    //Forecast/Tendency/Pressure
    currentForecastGroup = new QGroupBox( trUtf8( "Pression/Prévisions" ));

    currentPressureLabel = new QLabel();
    currentPressureLabel->setAlignment( Qt::AlignHCenter );
    currentPressureLabel->setFont(QFont("Arial", 16, QFont::Bold));
    currentTendencyLabel = new QLabel();
    currentForecastlLabel = new QLabel();

    label30 = new QLabel( tr( "Pression" ));
    label30->setAlignment( Qt::AlignHCenter | Qt::AlignBottom );

    //Wind
    currentWindGroup = new QGroupBox( trUtf8( "Vent" ));

    currentWindDirectionLabel = new QLabel();
    currentWindDirectionLabel->setAlignment( Qt::AlignHCenter );
    currentWindDirectionDial = new QwtCompass( this );

    QPalette colorGroup;
    for ( int c = 0; c < QPalette::NColorRoles; c++ )
        colorGroup.setColor( ( QPalette::ColorRole ) c, QColor() );

    colorGroup.setColor( QPalette::Base, palette().color( backgroundRole() ).light( 120 ));
    colorGroup.setColor( QPalette::WindowText, colorGroup.color( QPalette::Base ));

    currentWindDirectionDial->setLineWidth( 5 );
    currentWindDirectionDial->setFrameShadow(  QwtCompass::Raised);
    currentWindDirectionDial->setScaleComponents( QwtAbstractScaleDraw::Ticks | QwtAbstractScaleDraw::Labels );
    currentWindDirectionDial->setScaleTicks(0, 0, 3);
    currentWindDirectionDial->setNeedle(new QwtCompassMagnetNeedle(  QwtCompassMagnetNeedle::TriangleStyle, Qt::white, Qt::red ));
    currentWindDirectionDial->setMinimumWidth( 150 );

    currentWindSpeedLabel = new QLabel();
    currentWindSpeedLabel->setAlignment( Qt::AlignHCenter );
    currentWindSpeedLabel->setFont( QFont( "Arial", 16, QFont::Bold ));

    // DataReading
    dataReadingGroup = new QGroupBox( trUtf8( "Relevés" ));

    label40 = new QLabel( trUtf8( "Premier relevé le " ));
    label41 = new QLabel( trUtf8( "Dernier relevé le " ));
    label42 = new QLabel( trUtf8( "Prochain relevé dans " ));
    firstDataReadingDateLabel = new QLabel;
    firstDataReadingDateLabel->setFont( QFont( "Arial", 10, QFont::Bold ));
    lastDataReadingDateLabel = new QLabel;
    lastDataReadingDateLabel->setFont( QFont( "Arial", 10, QFont::Bold ));
    nextDataReadingDateLabel = new QLabel;
    nextDataReadingDateLabel->setFont( QFont( "Arial", 10, QFont::Bold ));
    connectionStatusLabel = new QLabel;
    connectionStatusLabel->setFont( QFont( "Arial", 10, QFont::Bold ));

    dataReadingButton = new QPushButton( trUtf8( "Relevé" ));
    connect( dataReadingButton, SIGNAL( clicked() ), this, SLOT( dataReadingButtonClicked()));

    // Set up layout
    QGridLayout *currentTempLayout = new QGridLayout();
    currentTempLayout->addWidget( label1, 0, 0 );
    currentTempLayout->addWidget( currentIndoorTemperatureSlider, 0, 1 );
    currentTempLayout->addWidget( label2, 0, 2 );
    currentTempLayout->addWidget( currentOutdoorTemperatureSlider, 0, 3 );
    currentTempLayout->addWidget( label3, 0, 4 );
    currentTempLayout->addWidget( currentWindChillTemperatureSlider, 0, 5 );
    currentTempLayout->addWidget( label4, 0, 6 );
    currentTempLayout->addWidget( currentDewPointTemperatureSlider, 0, 7 );
    currentTempLayout->addWidget( currentIndoorTemperatureLCDNumber, 1, 1 );
    currentTempLayout->addWidget( currentOutdoorTemperatureLCDNumber, 1, 3 );
    currentTempLayout->addWidget( currentWindChillTemperatureLCDNumber, 1, 5 );
    currentTempLayout->addWidget( currentDewPointTemperatureLCDNumber, 1, 7 );
    currentTempGroup->setLayout(currentTempLayout);

    QGridLayout *currentRainLayout = new QGridLayout();
    currentRainLayout->addWidget( currentRain1HourProgressBar, 0, 0 );
    currentRainLayout->addWidget( label10, 1, 0 );
    currentRainLayout->addWidget( currentRain1HourLabel, 2, 0 );
    currentRainLayout->addWidget( currentRain1DayProgressBar, 0, 1 );
    currentRainLayout->addWidget( label11, 1, 1 );
    currentRainLayout->addWidget( currentRain1DayLabel, 2, 1 );
    currentRainLayout->addWidget( currentRainTotalProgressBar, 0, 2 );
    currentRainLayout->addWidget( label12, 1, 2 );
    currentRainLayout->addWidget( currentRainTotalLabel, 2, 2 );
    currentRainGroup->setLayout(currentRainLayout);

    QGridLayout *currentHumidityLayout = new QGridLayout();
    currentHumidityLayout->addWidget( label20, 0, 0 );
    currentHumidityLayout->addWidget( currentIndoorHumiditySlider, 0, 1 );
    currentHumidityLayout->addWidget( label21, 0, 2 );
    currentHumidityLayout->addWidget( currentOutdoorHumiditySlider, 0, 3 );
    currentHumidityLayout->addWidget( currentIndoorHumidityLCDNumber, 1, 1 );
    currentHumidityLayout->addWidget( currentOutdoorHumidityLCDNumber, 1, 3 );
    currentHumidityGroup->setLayout(currentHumidityLayout);

    QGridLayout *currentForecastLayout = new QGridLayout();
    currentForecastLayout->addWidget( currentForecastlLabel, 0, 0, 2, 1 );
    currentForecastLayout->addWidget( currentTendencyLabel, 0, 1, 2, 1  );
    currentForecastLayout->addWidget( label30, 0, 2 );
    currentForecastLayout->addWidget( currentPressureLabel, 1, 2 );
    currentForecastGroup->setLayout( currentForecastLayout );

    QGridLayout *currentWindLayout = new QGridLayout();
    currentWindLayout->addWidget( currentWindDirectionDial, 0, 0, 2, 1 );
    currentWindLayout->addWidget( currentWindDirectionLabel, 0, 1 );
    currentWindLayout->addWidget( currentWindSpeedLabel, 1, 1 );
    currentWindGroup->setLayout( currentWindLayout );

    QGridLayout *dataReadingLayout = new QGridLayout();
    dataReadingLayout->addWidget( label40, 0, 0 );
    dataReadingLayout->addWidget( firstDataReadingDateLabel, 0, 1 );
    dataReadingLayout->addWidget( label41, 1, 0 );
    dataReadingLayout->addWidget( lastDataReadingDateLabel, 1, 1 );
    dataReadingLayout->addWidget( label42, 2, 0 );
    dataReadingLayout->addWidget( nextDataReadingDateLabel, 2, 1 );
    dataReadingLayout->addWidget( connectionStatusLabel, 3, 0 );
    dataReadingLayout->addWidget( dataReadingButton, 3, 1 );
    dataReadingGroup->setLayout( dataReadingLayout );

    QGridLayout *currentLayout = new QGridLayout();
    currentLayout->addWidget( currentTempGroup, 0, 0 );
    currentLayout->addWidget( currentRainGroup, 0, 1 );
    currentLayout->addWidget( currentHumidityGroup, 0, 2 );
    currentLayout->addWidget( currentForecastGroup, 1, 0 );
    currentLayout->addWidget( currentWindGroup, 1, 1 );
    currentLayout->addWidget( dataReadingGroup, 1, 2 );

    this->setLayout( currentLayout );
}

/**
 * Update all displayed data
 */
void CurrentDataTab::updateSensorsValues( void )
{
    const char *directions[]= { "N","NNE","NE","ENE","E","ESE","SE","SSE",
                                "S","SSW","SW","WSW","W","WNW","NW","NNW" };

    const QString tendencyValues[] = { ":/Images/Steady.png", ":/Images/Rising.png", ":/Images/Falling.png" };
    const QString forecastValues[] = { ":/Images/Rainy.png", ":/Images/Cloudy.png", ":/Images/Sunny.png" };

    currentIndoorTemperatureSlider->setValue( theCurrentData->getIndoorTemperature() );
    currentOutdoorTemperatureSlider->setValue( theCurrentData->getOutdoorTemperature() );
    currentWindChillTemperatureSlider->setValue( theCurrentData->getWindChillTemperature() );
    currentDewPointTemperatureSlider->setValue( theCurrentData->getDewPointTemperature() );

    currentIndoorTemperatureLCDNumber->display( theCurrentData->getIndoorTemperature() );
    currentOutdoorTemperatureLCDNumber->display( theCurrentData->getOutdoorTemperature() );
    currentWindChillTemperatureLCDNumber->display( theCurrentData->getWindChillTemperature() );
    currentDewPointTemperatureLCDNumber->display( theCurrentData->getDewPointTemperature() );

    currentRain1HourProgressBar->setValue( theCurrentData->getRain1Hour() );
    currentRain1DayProgressBar->setValue( theCurrentData->getRain1Day() );
    currentRainTotalProgressBar->setValue( theCurrentData->getRainTotal() );

    currentRain1HourLabel->setNum( theCurrentData->getRain1Hour() );
    currentRain1DayLabel->setNum( theCurrentData->getRain1Day() );
    currentRainTotalLabel->setNum( theCurrentData->getRainTotal() );

    currentIndoorHumiditySlider->setValue( theCurrentData->getIndoorHumidity() );
    currentOutdoorHumiditySlider->setValue( theCurrentData->getOutdoorHumidity() );
    currentIndoorHumidityLCDNumber->display( theCurrentData->getIndoorHumidity() );
    currentOutdoorHumidityLCDNumber->display( theCurrentData->getOutdoorHumidity() );

    int weatherT = theCurrentData->getWeatherTendency();
    if ( ( weatherT == 0) || ( weatherT == 1 ) || ( weatherT == 2 ) )
    {
        QPixmap imageTendency( tendencyValues[ weatherT ] );
        currentTendencyLabel->setPixmap( imageTendency );
    }
    else
    {
        qDebug() << "ERROR:T:" << weatherT;
    }

    int weatherF = theCurrentData->getWeatherForecast();
    if ( ( weatherF == 0) || ( weatherF == 1 ) || ( weatherF == 2 ) )
    {
        QPixmap imageForecast( forecastValues[ weatherF ] );
        currentForecastlLabel->setPixmap( imageForecast );
    }
    else
    {
        qDebug() << "ERROR:F:" << weatherF;
    }

    QString a;
    a.setNum( theCurrentData->getPressureRelative() );
    currentPressureLabel->setText( a + " hPa" );

    currentWindDirectionLabel->setText( directions[( int )( theCurrentData->getWindDirection() / 22.5 )] );
    currentWindSpeedLabel->setNum( theCurrentData->getWindSpeed() );
    currentWindSpeedLabel->setText( currentWindSpeedLabel->text() + " Km/h" );
    currentWindDirectionDial->setValue( theCurrentData->getWindDirection() );

    QLocale francais("fr_FR");


    firstDataReadingDateLabel->setText( francais.toString( theStation->getTheDataLogged()->getFirstDataReadingDate()->date(),  "dddd dd MMMM yyyy" ) +
                                        theStation->getTheDataLogged()->getFirstDataReadingDate()->time().toString( trUtf8( " à hh:mm:ss" )));

    lastDataReadingDateLabel->setText( francais.toString(theStation->getTheDataLogged()->getLastDataReadingDate()->date(), "dddd dd MMMM yyyy" ) +
                                       theStation->getTheDataLogged()->getLastDataReadingDate()->time().toString( trUtf8( " à hh:mm:ss" )));

    theStation->getTheDataLogged()->calculateNextDataReadingDate();

    nextDataReadingDateLabel->setText( francais.toString(theStation->getTheDataLogged()->getNextDataReadingDate()->date(), "dddd dd MMMM yyyy" ) +
                                       theStation->getTheDataLogged()->getNextDataReadingDate()->time().toString( trUtf8( " à hh:mm:ss" )));

    if( theStation->getTheConnection()->isConnected() )
    {
        connectionStatusLabel->setText( QObject::trUtf8( "<font color='green'>Connecté</font>" ) );
    }
    else
    {
        connectionStatusLabel->setText( QObject::trUtf8( "<font color='red'>Déconnecté</font>" ) );
    }

    if( theStation->getTheDataLogged()->getCurrentLogRead() != 0 )
    {
        QString b;
        b.setNum( theStation->getTheDataLogged()->getCurrentLogRead() );
        dataReadingButton->setText( trUtf8( "Relevé en cours - " ) + b );
    }
    else
    {
        dataReadingButton->setText( trUtf8( "Relevé" ) );
    }
}

/**
 * When "DataReading" button is clicked, it performs a new reading
 */
void CurrentDataTab::dataReadingButtonClicked()
{
    dataReadingButton->setText( trUtf8( "Relevé en cours - " ) );

    theCurrentData->startDataReading();
}
