/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CURRENTDATATAB_H
#define CURRENTDATATAB_H

#include <QFrame>
#include <QtGui>
#include <qwt_compass.h>
#include "Ws2355.h"
#include "CurrentData.h"
#include "Configuration.h"
#include "RotatedQLabel.h"

class CurrentDataTab : public QFrame
{
    Q_OBJECT
public:
    explicit CurrentDataTab( QWidget *parent = 0, Ws2355 *station = NULL, Configuration *config = NULL );
    ~CurrentDataTab();

    void            updateSensorsValues( void );

signals:

public slots:
    void            setDataReadingButtonText( QString );
    void            changeEvent(QEvent *event);

private slots:
    void            dataReadingButtonClicked();

private:
    void            createTab( void );

    QSlider         *currentIndoorTemperatureSlider;
    QSlider         *currentOutdoorTemperatureSlider;
    QSlider         *currentWindChillTemperatureSlider;
    QSlider         *currentDewPointTemperatureSlider;

    QLCDNumber      *currentIndoorTemperatureLCDNumber;
    QLCDNumber      *currentOutdoorTemperatureLCDNumber;
    QLCDNumber      *currentWindChillTemperatureLCDNumber;
    QLCDNumber      *currentDewPointTemperatureLCDNumber;

    QProgressBar    *currentRain1HourProgressBar;
    QProgressBar    *currentRain1DayProgressBar;
    QProgressBar    *currentRainTotalProgressBar;

    QLabel          *currentRain1HourLabel;
    QLabel          *currentRain1DayLabel;
    QLabel          *currentRainTotalLabel;

    QSlider         *currentIndoorHumiditySlider;
    QSlider         *currentOutdoorHumiditySlider;
    QLCDNumber      *currentIndoorHumidityLCDNumber;
    QLCDNumber      *currentOutdoorHumidityLCDNumber;

    QLabel          *currentPressureLabel;
    QLabel          *currentTendencyLabel;
    QLabel          *currentForecastlLabel;

    QLabel          *currentWindDirectionLabel;
    QwtCompass      *currentWindDirectionDial;
    QLabel          *currentWindSpeedLabel;

    QLabel          *firstDataReadingDateLabel;
    QLabel          *lastDataReadingDateLabel;
    QLabel          *nextDataReadingDateLabel;
    QLabel          *connectionStatusLabel;

    QPushButton     *dataReadingButton;

    Ws2355          *theStation;
    CurrentData     *theCurrentData;
    Configuration   *theGlobalConfiguration;

    QGroupBox       *currentTempGroup;
    RotatedQLabel   *label1;
    RotatedQLabel   *label2;
    RotatedQLabel   *label3;
    RotatedQLabel   *label4;
    QGroupBox       *currentRainGroup;
    QLabel          *label10;
    QLabel          *label11;
    QLabel          *label12;
    QGroupBox       *currentHumidityGroup;
    QGroupBox       *currentForecastGroup;
    QLabel          *label30;
    QGroupBox       *currentWindGroup;
    QGroupBox       *dataReadingGroup;
    QLabel          *label40;
    QLabel          *label41;
    QLabel          *label42;
    RotatedQLabel   *label20;
    RotatedQLabel   *label21;

};

#endif // CURRENTDATATAB_H
