/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DAILYDATATAB_H
#define DAILYDATATAB_H

#include <QtGui>
#include <QFrame>
#include "Ws2355.h"
#include "TemperaturesGraph.h"
#include "RainGraph.h"
#include "HumidityGraph.h"
#include "PressureGraph.h"
#include "WindGraph.h"
#include "Configuration.h"

class DailyDataTab : public QFrame
{
    Q_OBJECT
public:
    explicit DailyDataTab( QWidget *parent = 0, Ws2355 *station = NULL, Configuration *config = NULL );
    ~DailyDataTab();

signals:

private slots:
    void                selectedDateChanged();
public slots:
    void                changeEvent(QEvent *event);

private:
    void                createTab( void );
    void                updateGraphics( QDate date );

    Ws2355              *theStation;
    Configuration       *theGlobalConfiguration;

    TemperaturesGraph   *daylyTemperaturesGraph;
    RainGraph           *daylyRainGraph;
    HumidityGraph       *daylyHumidityGraph;
    PressureGraph       *daylyPressureGraph;
    WindGraph           *daylyWindGraph;

    QCalendarWidget     *dailyDataDate;

    QGroupBox           *temperaturesGroup;
    QGroupBox           *rainGroup;
    QGroupBox           *humidityGroup;
    QGroupBox           *pressureGroup;
    QGroupBox           *windGroup;
    QGroupBox           *dateGroup;

};

#endif // DAILYDATATAB_H
