/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "DataLogged.h"
#include <QDebug>

DataLogged::DataLogged( Configuration *config )
    :theGlobalConfiguration( config )
{
    currentLogRead = 0;

    updateDataReadingDates();

    calculateNextDataReadingDate();

}

void DataLogged::calculateNextDataReadingDate( void )
{
    nextDataReadingDate = lastDataReadingDate.addSecs(  60 * 60 * theGlobalConfiguration->getfrequencyOfReading() );
}

void DataLogged::updateDataReadingDates( void )
{
    // Open the current data file
    QString strFile( theGlobalConfiguration->getFullDataFileName() );

    int tmp1, tmp2;
    struct tm timeLastLog_tm;

    // In read only mode
    FILE *currentDataFileDescriptor = fopen( strFile.toAscii(), "ab+" );

    if( currentDataFileDescriptor == NULL )
    {
        qDebug() << QObject::trUtf8( "Impossible d'ouvrir le fichier currentDataFile.txt\n" );
        return;
    }

    // Go to the beginning of file
    fseek( currentDataFileDescriptor, 0L, SEEK_CUR );

    // Read the timestamp of the first line
    if(  fscanf( currentDataFileDescriptor,"%4d%2d%2d%2d%2d", &tmp1, &tmp2,
                 &timeLastLog_tm.tm_mday, &timeLastLog_tm.tm_hour,
                 &timeLastLog_tm.tm_min) == 5 )
    {
        timeLastLog_tm.tm_year = tmp1 - 1900;
        timeLastLog_tm.tm_mon = tmp2 - 1;
        timeLastLog_tm.tm_sec = 0;
        timeLastLog_tm.tm_isdst = -1;
    }

    // set the date and the time of the first reading
    QDate d;
    d.setDate( timeLastLog_tm.tm_year + 1900, timeLastLog_tm.tm_mon + 1, timeLastLog_tm.tm_mday) ;
    QTime t;
    t.setHMS( timeLastLog_tm.tm_hour, timeLastLog_tm.tm_min, timeLastLog_tm.tm_sec, 0 );
    firstDataReadingDate.setTime( t );
    firstDataReadingDate.setDate( d );

    // Go to the end of file
    fseek( currentDataFileDescriptor, 1L, SEEK_END );

    // Get the last line
    long counter = 60;
    char ch;
    do
    {
        counter++;
        if( fseek( currentDataFileDescriptor, -counter, SEEK_END ) < 0 )
            break;
        ch = getc( currentDataFileDescriptor );
    }
    while ( ch != '\n' && ch != '\r' );

    // Read the timestamp of the last line
    if(  fscanf( currentDataFileDescriptor,"%4d%2d%2d%2d%2d", &tmp1, &tmp2,
                 &timeLastLog_tm.tm_mday, &timeLastLog_tm.tm_hour,
                 &timeLastLog_tm.tm_min) == 5 )
    {
        timeLastLog_tm.tm_year = tmp1 - 1900;
        timeLastLog_tm.tm_mon = tmp2 - 1;
        timeLastLog_tm.tm_sec = 0;
        timeLastLog_tm.tm_isdst = -1;
    }
    // set the date and the time of the last reading
    d.setDate( timeLastLog_tm.tm_year + 1900, timeLastLog_tm.tm_mon + 1, timeLastLog_tm.tm_mday) ;
    t.setHMS( timeLastLog_tm.tm_hour, timeLastLog_tm.tm_min, timeLastLog_tm.tm_sec, 0 );
    lastDataReadingDate.setTime( t );
    lastDataReadingDate.setDate( d );


    fclose( currentDataFileDescriptor );
}

/**
 * read data from the data file and fill the structure "data" with data according to the date
 *
 * @param day : QDate
 * @param displayMode : BY_DAY, BY_MONTH, BY_YEAR
 */
void DataLogged::readData( QDate date, unsigned int displayMode )
{
    double rain[ 32 ],
            pressure[ 32 ], pressureMin[ 32 ], pressureMax[ 32 ],
            humidityIn[ 32 ], humidityInMin[ 32 ], humidityInMax[ 32 ],
            humidityOut[ 32 ], humidityOutMin[ 32 ], humidityOutMax[ 32 ],
            windSpeed[ 32 ], windSpeedMin[ 32 ], windSpeedMax[ 32 ],
            windDirection[ 32 ],
            inTemperature[ 32 ], inTemperatureMin[ 32 ], inTemperatureMax[ 32 ],
            outTemperature[ 32 ], outTemperatureMin[ 32 ], outTemperatureMax[ 32 ],
            dewPointTemperature[ 32 ], dewPointTemperatureMin[ 32 ], dewPointTemperatureMax[ 32 ];

    int nbOfReadsByDay[ 32 ];
    int nbOfReadsByMonth[ 32 ];

    // Initialize local data
    for( int i = 0; i < 32; i ++)
    {
        rain[ i ] = pressure[ i ] = pressureMin[ i ] = nbOfReadsByDay[ i ] = 0;
        humidityIn[ i ] = humidityInMin[ i ] = 0;
        humidityOut[ i ] = humidityOutMin[ i ] = 0;
        windSpeed[ i ] = windSpeedMin[ i ] = 0;
        windDirection[ i ] = 0;
        inTemperature[ i ] = inTemperatureMin[ i ] = 0;
        outTemperature[ i ] = outTemperatureMin[ 32 ] = 0;
        dewPointTemperature[ i ] = dewPointTemperatureMin[ i ] = 0;

        inTemperatureMax[ i ] = 100;
        outTemperatureMax[ i ] = 100;
        dewPointTemperatureMax[ i ] = 100;
        windSpeedMax[ i ] = 150;

        humidityInMax[ i ] = 100;
        humidityOutMax[ i ] = 100;
        pressureMax[ i ] = 1500;

        nbOfReadsByDay[ i ] = 0;
        nbOfReadsByMonth[ i ] = 0;

    }

    // Clear old values
    data.inTemperatureX.clear();
    data.inTemperatureY.clear();
    data.outTemperatureX.clear();
    data.outTemperatureY.clear();
    data.windChillTemperatureX.clear();
    data.windChillTemperatureY.clear();
    data.dewPointTemperatureX.clear();
    data.dewPointTemperatureY.clear();
    data.rainX.clear();
    data.rainY.clear();
    data.inHumidityX.clear();
    data.inHumidityY.clear();
    data.outHumidityX.clear();
    data.outHumidityY.clear();
    data.pressureX.clear();
    data.pressureY.clear();
    data.windSpeedX.clear();
    data.windSpeedY.clear();
    data.windDirectionX.clear();
    data.windDirectionY.clear();


    // Open the current data file
    QString strFile( theGlobalConfiguration->getFullDataFileName() );
    QFile file(theGlobalConfiguration->getFullDataFileName());

    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in(&file);
    while (!in.atEnd())
    {
        QString line = in.readLine();
        QStringList list1 = line.split( " ", QString::SkipEmptyParts );

        QDate lineDate( list1[0].mid( 0, 4 ).toInt(), list1[ 0 ].mid( 4, 2 ).toInt(), list1[ 0 ].mid( 6, 2 ).toInt() );

        switch ( displayMode )
        {
            case BY_DAY :
            {
                if( lineDate == date )
                {
                    double hour = list1[ 0 ].mid( 8, 2 ).toDouble();

                    data.inTemperatureX.append( hour );
                    data.inTemperatureY.append( list1[ 1 ].toFloat() );
                    data.outTemperatureX.append( hour );
                    data.outTemperatureY.append( list1[ 2 ].toFloat() );
                    data.windChillTemperatureX.append( hour );
                    data.windChillTemperatureY.append( list1[ 9 ].toFloat());
                    data.dewPointTemperatureX.append( hour );
                    data.dewPointTemperatureY.append( list1[ 3 ].toFloat() );

                    // Calculate the rain hour by hour
                    data.rainX.append( hour );
                    data.rainY.append( list1[ 10 ].toFloat() - rain[ 0 ] );
                    rain[ 0 ] = list1[ 10 ].toFloat();

                    data.inHumidityX.append( hour );
                    data.inHumidityY.append( list1[ 4 ].toFloat() );
                    data.outHumidityX.append( hour );
                    data.outHumidityY.append( list1[ 5 ].toFloat() );

                    data.pressureX.append( hour );
                    data.pressureY.append( list1[ 11 ].toFloat() );

                    data.windSpeedX.append( hour );
                    data.windSpeedY.append( list1[ 6 ].toFloat() );
                    data.windDirectionX.append( hour );
                    data.windDirectionY.append( list1[ 7 ].toFloat() );
                }
                else
                {
                    rain[ 0 ] = list1[ 10 ].toFloat();
                }
                break;
            }
            case BY_MONTH :
            {
                if( ( lineDate.month() == date.month() ) && ( lineDate.year() == date.year() ))
                {
                    int day = list1[ 0 ].mid( 6, 2 ).toInt();

                    // Calculate the number of hours by day (for averages)
                    nbOfReadsByDay[ day ] ++;

                    // Memorize the minimum and the maximum of tempertures
                    if( inTemperatureMin[ day ] > list1[ 1 ].toDouble() )
                        inTemperatureMin[ day ] = list1[ 1 ].toDouble();
                    if( inTemperatureMax[ day ] < list1[ 1 ].toDouble() )
                        inTemperatureMax[ day ] = list1[ 1 ].toDouble();
                    inTemperature[ day ] += list1[ 1 ].toDouble();
                    if( outTemperatureMin[ day ] > list1[ 2 ].toDouble() )
                        outTemperatureMin[ day ] = list1[ 2 ].toDouble();
                    if( outTemperatureMax[ day ] < list1[ 2 ].toDouble() )
                        outTemperatureMax[ day ] = list1[ 2 ].toDouble();
                    outTemperature[ day ] += list1[ 2 ].toDouble();
                    if( dewPointTemperatureMin[ day ] > list1[ 3 ].toDouble() )
                        dewPointTemperatureMin[ day ] = list1[ 3 ].toDouble();
                    if( dewPointTemperatureMax[ day ] < list1[ 3 ].toDouble() )
                        dewPointTemperatureMax[ day ] = list1[ 3 ].toDouble();
                    dewPointTemperature[ day ] += list1[ 3 ].toDouble();

                    // Calculate the rain day by day
                    if( list1[ 10 ].toFloat() > rain[ 0 ] )
                        rain[ day ] += ( list1[ 10 ].toFloat() - rain[ 0 ] );
                    rain[ 0 ] = list1[ 10 ].toFloat();

                    // Memorize the minimum and the maximum of humidity
                    if( humidityInMin[ day ] > list1[ 4 ].toDouble() )
                        humidityInMin[ day ] = list1[ 4 ].toDouble();
                    if( humidityInMax[ day ] < list1[ 4 ].toDouble() )
                        humidityInMax[ day ] = list1[ 4 ].toDouble();
                    humidityIn[ day ] += list1[ 4 ].toDouble();
                    if( humidityOutMin[ day ] > list1[ 5 ].toDouble() )
                        humidityOutMin[ day ] = list1[ 5 ].toDouble();
                    if( humidityOutMax[ day ] < list1[ 5 ].toDouble() )
                        humidityOutMax[ day ] = list1[ 5 ].toDouble();
                    humidityOut[ day ] += list1[ 5 ].toDouble();

                    // Memorize the minimum and the maximum of pressure
                    if( pressureMin[ day ] > list1[ 11 ].toDouble() )
                        pressureMin[ day ] = list1[ 11 ].toDouble();
                    if( pressureMax[ day ] < list1[ 11 ].toDouble() )
                        pressureMax[ day ] = list1[ 11 ].toDouble();
                    pressure[ day ] += list1[ 11 ].toDouble();

                    // Memorize the minimum and the maximum of windSpeed
                    if( windSpeedMin[ day ] > list1[ 6 ].toDouble() )
                        windSpeedMin[ day ] = list1[ 6 ].toDouble();
                    if( windSpeedMax[ day ] < list1[ 6 ].toDouble() )
                        windSpeedMax[ day ] = list1[ 6 ].toDouble();
                    windSpeed[ day ] += list1[ 6 ].toDouble();

                    windDirection[ day ] += list1[ 7 ].toDouble();
                }
                else
                {
                    rain[ 0 ] = list1[ 10 ].toFloat();
                }
                break;
            }
            case BY_YEAR :
            {
                if( lineDate.year() == date.year() )
                {
                    int month = list1[ 0 ].mid( 4, 2 ).toInt();

                    // Calculate the number of day by month (for averages)
                    nbOfReadsByMonth[ month ] ++;

                    // Memorize the minimum and the maximum of temperatures
                    if( inTemperatureMin[ month ] > list1[ 1 ].toDouble() )
                        inTemperatureMin[ month ] = list1[ 1 ].toDouble();
                    if( inTemperatureMax[ month ] < list1[ 1 ].toDouble() )
                        inTemperatureMax[ month ] = list1[ 1 ].toDouble();
                    inTemperature[ month ] += list1[ 1 ].toDouble();
                    if( outTemperatureMin[ month ] > list1[ 2 ].toDouble() )
                        outTemperatureMin[ month ] = list1[ 2 ].toDouble();
                    if( outTemperatureMax[ month ] < list1[ 2 ].toDouble() )
                        outTemperatureMax[ month ] = list1[ 2 ].toDouble();
                    outTemperature[ month ] += list1[ 2 ].toDouble();
                    if( dewPointTemperatureMin[ month ] > list1[ 3 ].toDouble() )
                        dewPointTemperatureMin[ month ] = list1[ 3 ].toDouble();
                    if( dewPointTemperatureMax[ month ] < list1[ 3 ].toDouble() )
                        dewPointTemperatureMax[ month ] = list1[ 3 ].toDouble();
                    dewPointTemperature[ month ] += list1[ 3 ].toDouble();

                    // Calculate the rain day by day
                    if( list1[ 10 ].toFloat() > rain[ 0 ] )
                        rain[ month ] += ( list1[ 10 ].toFloat() - rain[ 0 ] );
                    rain[ 0 ] = list1[ 10 ].toFloat();

                    // Memorize the minimum and the maximum of humidity
                    if( humidityInMin[ month ] > list1[ 4 ].toDouble() )
                        humidityInMin[ month ] = list1[ 4 ].toDouble();
                    if( humidityInMax[ month ] < list1[ 4 ].toDouble() )
                        humidityInMax[ month ] = list1[ 4 ].toDouble();
                    humidityIn[ month ] += list1[ 4 ].toDouble();

                    if( humidityOutMin[ month ] > list1[ 5 ].toDouble() )
                        humidityOutMin[ month ] = list1[ 5 ].toDouble();
                    if( humidityOutMax[ month ] < list1[ 5 ].toDouble() )
                        humidityOutMax[ month ] = list1[ 5 ].toDouble();
                    humidityOut[ month ] += list1[ 5 ].toDouble();

                    // Memorize the minimum and the maximum of pressure
                    if( pressureMin[ month ] > list1[ 11 ].toDouble() )
                        pressureMin[ month ] = list1[ 11 ].toDouble();
                    if( pressureMax[ month ] < list1[ 11 ].toDouble() )
                        pressureMax[ month ] = list1[ 11 ].toDouble();
                    pressure[ month ] += list1[ 11 ].toDouble();

                    // Memorize the minimum and the maximum of windSpeed
                    if( windSpeedMin[ month ] > list1[ 6 ].toDouble() )
                        windSpeedMin[ month ] = list1[ 6 ].toDouble();
                    if( windSpeedMax[ month ] < list1[ 6 ].toDouble() )
                        windSpeedMax[ month ] = list1[ 6 ].toDouble();
                    windSpeed[ month ] += list1[ 6 ].toDouble();

                    windDirection[ month ] += list1[ 7 ].toDouble();
                }
                else
                {
                    rain[ 0 ] = list1[ 10 ].toFloat();
                }
                break;
            }
            default :
            {
            }
        }
    }

    if( displayMode == BY_MONTH )
    {
        for( int i = 1; i <= 31; i ++ )
        {
            data.rainX.append(( double ) i );
            data.pressureX.append(( double ) i );
            data.inHumidityX.append(( double ) i );
            data.outHumidityX.append(( double ) i );
            data.windSpeedX.append(( double ) i );
            data.windDirectionX.append(( double ) i );
            data.inTemperatureX.append(( double ) i );
            data.outTemperatureX.append(( double ) i );
            data.dewPointTemperatureX.append(( double ) i );

            data.rainY.append( rain[ i ] ) ;
            data.windDirectionY.append( windDirection[ i ] / nbOfReadsByDay[ i ] ) ;

            if( nbOfReadsByDay[ i ] != 0 )
            {
                data.pressureY.append( pressure[ i ] / nbOfReadsByDay[ i ] );
                data.inHumidityY.append( humidityIn[ i ] / nbOfReadsByDay[ i ] );
                data.outHumidityY.append( humidityOut[ i ] / nbOfReadsByDay[ i ] );
                data.windSpeedY.append( windSpeed[ i ] / nbOfReadsByDay[ i ] );
                data.inTemperatureY.append( inTemperature[ i ] / nbOfReadsByDay[ i ] );
                data.outTemperatureY.append( outTemperature[ i ] / nbOfReadsByDay[ i ] );
                data.dewPointTemperatureY.append( dewPointTemperature[ i ] / nbOfReadsByDay[ i ] );
            }
            else
            {
                data.pressureY.append( 0 );
                data.inHumidityY.append( 0 );
                data.outHumidityY.append( 0 );
                data.windSpeedY.append( 0 );
                data.inTemperatureY.append( 0 );
                data.outTemperatureY.append( 0 );
                data.dewPointTemperatureY.append( 0 );

            }
        }
    }
    if( displayMode == BY_YEAR )
    {
        for( int i = 1; i <= 12; i ++ )
        {
            data.rainX.append(( double ) i );
            data.pressureX.append(( double ) i );
            data.inHumidityX.append(( double ) i );
            data.outHumidityX.append(( double ) i );
            data.windSpeedX.append(( double ) i );
            data.windDirectionX.append(( double ) i );
            data.inTemperatureX.append(( double ) i );
            data.outTemperatureX.append(( double ) i );
            data.dewPointTemperatureX.append(( double ) i );

            data.rainY.append( rain[ i ] ) ;
            data.windDirectionY.append( windDirection[ i ] / nbOfReadsByMonth[ i ] ) ;

            if( nbOfReadsByMonth[ i ] != 0 )
            {
                data.pressureY.append( pressure[ i ] / nbOfReadsByMonth[ i ] );
                data.inHumidityY.append( humidityIn[ i ] / nbOfReadsByMonth[ i ] );
                data.outHumidityY.append( humidityOut[ i ] / nbOfReadsByMonth[ i ] );
                data.windSpeedY.append( windSpeed[ i ] / nbOfReadsByMonth[ i ] );
                data.inTemperatureY.append( inTemperature[ i ] / nbOfReadsByMonth[ i ] );
                data.outTemperatureY.append( outTemperature[ i ] / nbOfReadsByMonth[ i ] );
                data.dewPointTemperatureY.append( dewPointTemperature[ i ] / nbOfReadsByMonth[ i ] );
            }
            else
            {
                data.pressureY.append( 0 );
                data.inHumidityY.append( 0 );
                data.outHumidityY.append( 0 );
                data.windSpeedY.append( 0 );
                data.inTemperatureY.append( 0 );
                data.outTemperatureY.append( 0 );
                data.dewPointTemperatureY.append( 0 );
            }
        }
    }
}
