/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DATAREADING_H
#define DATAREADING_H

#include <QDateTime>
#include <QLabel>
#include "Configuration.h"

// define the station configuration structure
typedef struct
{
    QVector<double> inTemperatureX;
    QVector<double> inTemperatureY;
    QVector<double> outTemperatureX;
    QVector<double> outTemperatureY;
    QVector<double> windChillTemperatureX;
    QVector<double> windChillTemperatureY;
    QVector<double> dewPointTemperatureX;
    QVector<double> dewPointTemperatureY;

    QVector<double> rainX;
    QVector<double> rainY;

    QVector<double> inHumidityX;
    QVector<double> inHumidityY;
    QVector<double> outHumidityX;
    QVector<double> outHumidityY;

    QVector<double> pressureX;
    QVector<double> pressureY;

    QVector<double> windSpeedX;
    QVector<double> windSpeedY;
    QVector<double> windDirectionX;
    QVector<double> windDirectionY;

} dataLoggedType;

/**
 * Used by readData()
 */
#define BY_DAY      0
#define BY_MONTH    1
#define BY_YEAR     2

class DataLogged
{
public:
    DataLogged( Configuration *config);

    QDateTime       *getFirstDataReadingDate( void ) { return &firstDataReadingDate; }
    QDateTime       *getLastDataReadingDate( void ) { return &lastDataReadingDate; }
    QDateTime       *getNextDataReadingDate( void ) { return &nextDataReadingDate; }

    unsigned int    getCurrentLogRead( void ) { return currentLogRead; }
    void            setCurrentLogRead( unsigned int log ) { currentLogRead = log; }

    void            setLastDataReadingDate( QDateTime val ) { lastDataReadingDate = val; }

    void            calculateNextDataReadingDate( void );
    void            updateDataReadingDates( void );
    dataLoggedType  *getData( void ) { return &data; }
    void            readData( QDate, unsigned int displayMode );

private:
    dataLoggedType  data;
    Configuration   *theGlobalConfiguration;
    QDateTime       firstDataReadingDate;
    QDateTime       lastDataReadingDate;
    QDateTime       nextDataReadingDate;

    unsigned int    currentLogRead;
};

#endif // DATAREADING_H
