/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <qwt_text.h>
#include <qwt_legend.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include "HumidityGraph.h"

HumidityGraph::HumidityGraph( double min, double max, double step )
{
    xMin = min;
    xMax = max;
    xStep = step;

    humidityPlot = new QwtPlot();
    humidityLeftAxisTitle = new QwtText( QObject::trUtf8( "H u m i d i t é   ( % )" ));
    humidityLeftAxisTitle->setFont( QFont("Arial", 9, QFont::Bold ));
    humidityPlot->setAxisTitle( QwtPlot::yLeft, *humidityLeftAxisTitle );
    humidityPlot->setAxisScale( QwtPlot::xBottom, min, max, step );
    humidityPlot->setAxisMaxMinor ( QwtPlot::xBottom,1 );
    humidityPlot->setAxisScale( QwtPlot::yLeft, 0, 100, 10 );
    humidityPlot->setAxisMaxMinor ( QwtPlot::yLeft,1 );
    humidityPlot->setMinimumSize( 350,250 );
    humidityPlot->setAxisFont( QwtPlot::yLeft, QFont("Arial", 8, QFont::Normal));
    humidityPlot->setAxisFont( QwtPlot::xBottom, QFont("Arial", 7, QFont::Normal));

    // Display the legend
    QwtLegend *humidityLegend = new QwtLegend;
    humidityLegend->setFrameStyle( QFrame::Box|QFrame::Sunken );
    humidityPlot->insertLegend( humidityLegend, QwtPlot::BottomLegend );

    // One curve creation
    inHumidityCurve = new QwtPlotCurve( QObject::trUtf8( "Intérieure" ));
    inHumidityCurve->setPen( QPen( Qt::red, 2 ));
    outHumidityCurve = new QwtPlotCurve( QObject::trUtf8( "Extérieure" ));
    outHumidityCurve->setPen( QPen( Qt::blue, 2 ));

    inHumidityCurve->attach( humidityPlot );
    outHumidityCurve->attach( humidityPlot );

}

HumidityGraph::~HumidityGraph()
{
    delete humidityPlot;
    delete inHumidityCurve;
    delete outHumidityCurve;
    delete humidityLeftAxisTitle;
}

void HumidityGraph::setXAxisMax( double max)
{
    humidityPlot->setAxisScale( QwtPlot::xBottom, xMin, max, xStep );
    humidityPlot->setAxisTitle( QwtPlot::yLeft, *humidityLeftAxisTitle );
    humidityPlot->replot();
}

void HumidityGraph::setText( void )
{
    humidityLeftAxisTitle->setText( QObject::trUtf8( "H u m i d i t é   ( % )" ));
    inHumidityCurve->setTitle( QObject::trUtf8( "Intérieure" ));
    outHumidityCurve->setTitle( QObject::trUtf8( "Extérieure" ));
}

/**
 * Update the two curves with the new data
 *
 * @param x1 : QVector<double> - x value for In humidity curve
 * @param y1 : QVector<double> - y value for In humidity curve
 * @param x2 : QVector<double> - x value for Out humidity curve
 * @param y2 : QVector<double> - y value for Out humidity curve
 */
void HumidityGraph::update( QVector<double> x1, QVector<double> y1, QVector<double> x2, QVector<double> y2 )
{
    inHumidityCurve->setSamples( x1, y1 );
    outHumidityCurve->setSamples( x2, y2 );

    humidityPlot->replot();
}
