/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <QSlider>
#include <QPixmap>
#include "math.h"

#include "MainWindow.h"
#include "ui_MainWindow.h"
#include "CurrentDataTab.h"
#include "DailyDataTab.h"
#include "MonthlyDataTab.h"
#include "YearlyDataTab.h"
#include "ConfigurationTab.h"
#include "Configuration.h"
#include <WsMeteo.h>

MainWindow::MainWindow( QWidget *parent ) :
    QMainWindow( parent ),
    ui( new Ui::MainWindow )
{
    ui->setupUi(this);

    // The configuration of the program and the station
    theConfiguration = new Configuration();

    // Create a station
    theStation = new Ws2355( theConfiguration );

    // Set up widgets
    mainTabWidget = new QTabWidget;

    // Tabs creation
    currentWidget = new CurrentDataTab( this, theStation, theConfiguration );
    dailyWidget = new DailyDataTab( this, theStation, theConfiguration );
    monthlyWidget = new MonthlyDataTab( this, theStation, theConfiguration );
    yearlyWidget = new YearlyDataTab( this, theStation, theConfiguration );
    configurationWidget = new ConfigurationTab( this, theStation, theConfiguration );

    mainTabWidget->addTab( currentWidget, tr( "Valeurs courantes" ));
    mainTabWidget->addTab( dailyWidget, trUtf8( "Jour" ));
    mainTabWidget->addTab( monthlyWidget, tr( "Mois" ));
    mainTabWidget->addTab( yearlyWidget, trUtf8( "Année" ));
    mainTabWidget->addTab( configurationWidget, tr( "Configuration" ));

    QGridLayout *mainLayout = new QGridLayout;
    mainLayout->addWidget( mainTabWidget, 0, 0 );
    centralWidget()->setLayout( mainLayout );
    setCentralWidget( mainTabWidget );

    setWindowTitle( GUIStrings::WSMETEO_PROGRAM_NAME + " - V" + GUIStrings::WSMETEO_PROGRAM_VERSION );

    QTimer *timer = new QTimer();
    connect( timer, SIGNAL( timeout() ), this, SLOT( periodicTask() ));
    timer->start( 2000 );
}

/**
 * Destructor
 */
MainWindow::~MainWindow()
{
    delete theStation;
    delete theConfiguration;

    delete currentWidget;
    delete dailyWidget;
    delete monthlyWidget;
    delete yearlyWidget;
    delete configurationWidget;

    delete mainTabWidget;

    delete ui;
}

/**
 * When the language change, update all text fields.
 * Capture QEvent::LanguageChange and forward all other events
 *
 * @param event
 */
void MainWindow::changeEvent(QEvent *event) {
    if (event->type() == QEvent::LanguageChange)
    {
        // Set all texts
        mainTabWidget->setTabText( 0, tr( "Valeurs courantes" ));
        mainTabWidget->setTabText( 1, trUtf8( "Jour" ));
        mainTabWidget->setTabText( 2, tr( "Mois" ));
        mainTabWidget->setTabText( 3, trUtf8( "Année" ));
        mainTabWidget->setTabText( 4, tr( "Configuration" ));
    }
    else
    {
        QWidget::changeEvent(event);
    }
}


void MainWindow::periodicTask()
{
    // Read the new values from the station
    theStation->readCurrentValues();

    // And update the current data
    currentWidget->updateSensorsValues();
    configurationWidget->updateValues();

    qDebug() << "Periodic task";
}
