/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MONTHLYDATATAB_H
#define MONTHLYDATATAB_H

#include <QFrame>
#include <QtGui>
#include "Ws2355.h"
#include "TemperaturesGraph.h"
#include "RainGraph.h"
#include "HumidityGraph.h"
#include "PressureGraph.h"
#include "WindGraph.h"
#include "Configuration.h"

class MonthlyDataTab : public QFrame
{
    Q_OBJECT
public:
    explicit MonthlyDataTab( QWidget *parent = 0, Ws2355 *station = NULL, Configuration *config = NULL );
    ~MonthlyDataTab();

signals:

public slots:

private slots:
    void                monthChanged(const QDate & date);

public slots:
    void                changeEvent(QEvent *event);

private:
    void                createTab( void );
    void                updateGraphics( QDate date );

    Ws2355              *theStation;
    Configuration       *theGlobalConfiguration;

    TemperaturesGraph   *monthlyTemperaturesGraph;
    RainGraph           *monthlyRainGraph;
    HumidityGraph       *monthlyHumidityGraph;
    PressureGraph       *monthlyPressureGraph;
    WindGraph           *monthlyWindGraph;
    QDateEdit           *month;

    QGroupBox           *temperaturesGroup;
    QGroupBox           *rainGroup;
    QGroupBox           *humidityGroup;
    QGroupBox           *pressureGroup;
    QGroupBox           *windGroup;

};

#endif // MONTHLYDATATAB_H
