/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <qwt_text.h>
#include <qwt_legend.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include "PressureGraph.h"

PressureGraph::PressureGraph( double min, double max, double step )
{
    xMin = min;
    xMax = max;
    xStep = step;

    pressurePlot = new QwtPlot();
    pressureLeftAxisTitle = new QwtText( QObject::trUtf8( "P r e s s i o n   ( h P a )" ));
    pressureLeftAxisTitle->setFont( QFont("Arial", 9, QFont::Bold ));
    pressurePlot->setAxisTitle( QwtPlot::yLeft, *pressureLeftAxisTitle );
    pressurePlot->setAxisScale( QwtPlot::xBottom, min, max, step );
    pressurePlot->setAxisMaxMinor ( QwtPlot::xBottom,1 );
    pressurePlot->setAxisScale( QwtPlot::yLeft, 960, 1040, 5 );
    pressurePlot->setAxisMaxMinor ( QwtPlot::yLeft,1 );
    pressurePlot->setMinimumSize( 350,250 );
    pressurePlot->setAxisFont( QwtPlot::yLeft, QFont("Arial", 8, QFont::Normal));
    pressurePlot->setAxisFont( QwtPlot::xBottom, QFont("Arial", 7, QFont::Normal));

    // Display the legend
    QwtLegend *pressureLegend = new QwtLegend;
    pressureLegend->setFrameStyle( QFrame::Box|QFrame::Sunken );
    pressurePlot->insertLegend( pressureLegend, QwtPlot::BottomLegend );

    // One curve creation
    pressureCurve = new QwtPlotCurve( QObject::trUtf8( "Pression" ));
    pressureCurve->setPen( QPen( Qt::red, 2 ));
    pressureCurve->attach( pressurePlot );

}

PressureGraph::~PressureGraph()
{
    delete pressurePlot;
    delete pressureCurve;
    delete pressureLeftAxisTitle;
}

void PressureGraph::setXAxisMax( double max)
{
    pressurePlot->setAxisScale( QwtPlot::xBottom, xMin, max, xStep );
    pressurePlot->replot();
}

void PressureGraph::setText( void )
{
    pressureLeftAxisTitle->setText( QObject::trUtf8( "P r e s s i o n   ( h P a )" ));
    pressurePlot->setAxisTitle( QwtPlot::yLeft, *pressureLeftAxisTitle );
    pressureCurve->setTitle( QObject::trUtf8( "Pression" ));
}

/**
 * Update the curve with the new data
 *
 * @param x1 : QVector<double> - x value for pressure curve
 * @param y1 : QVector<double> - y value for pressure curve
 */
void PressureGraph::update( QVector<double> x1, QVector<double> y1 )
{
    pressureCurve->setSamples( x1, y1 );

    pressurePlot->replot();
}
