/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <qwt_text.h>
#include <qwt_legend.h>
#include <qwt_plot.h>
//#include <qwt_plot_curve.h>
#include <qwt_plot_histogram.h>
#include "RainGraph.h"

RainGraph::RainGraph( double min, double max, double step )
{
    xMin = min;
    xMax = max;
    xStep = step;

    rainPlot = new QwtPlot();
    rainLeftAxisTitle = new QwtText( QObject::trUtf8( "P r é c i p i t a t i o n s   ( m m )" ));
    rainLeftAxisTitle->setFont( QFont("Arial", 9, QFont::Bold ));
    rainPlot->setAxisTitle( QwtPlot::yLeft, *rainLeftAxisTitle );
    rainPlot->setAxisScale( QwtPlot::xBottom, min, max, step );
    rainPlot->setAxisMaxMinor ( QwtPlot::xBottom,1 );
    rainPlot->setAxisScale( QwtPlot::yLeft, 0, 30, 2 );
    rainPlot->setAxisMaxMinor ( QwtPlot::yLeft, 0 );
    rainPlot->setMinimumSize( 350,250 );
    rainPlot->setAxisFont( QwtPlot::yLeft, QFont("Arial", 8, QFont::Normal));
    rainPlot->setAxisFont( QwtPlot::xBottom, QFont("Arial", 7, QFont::Normal));

    rainPlot->setAxisAutoScale( QwtPlot::yLeft );

    // Display the legend
    QwtLegend *rainLegend = new QwtLegend;
    rainLegend->setFrameStyle( QFrame::Box|QFrame::Sunken );
    rainPlot->insertLegend( rainLegend, QwtPlot::BottomLegend );

    // One curve creation
    rainCurve = new QwtPlotHistogram( QObject::trUtf8( "Précipitations" ));
    rainCurve->setPen( QPen( Qt::red, 2 ) );
    rainCurve->setStyle( QwtPlotHistogram::Columns );

    rainCurve->attach( rainPlot );
}

RainGraph::~RainGraph()
{
    delete rainPlot;
    delete rainCurve;
    delete rainLeftAxisTitle;
}

void RainGraph::setXAxisMax( double max)
{
    rainPlot->setAxisScale( QwtPlot::xBottom, xMin, max, xStep );
    rainPlot->replot();
}

void RainGraph::setText( void )
{
    rainLeftAxisTitle->setText( QObject::trUtf8( "P r é c i p i t a t i o n s   ( m m )" ));
    rainPlot->setAxisTitle( QwtPlot::yLeft, *rainLeftAxisTitle );
    rainCurve->setTitle( QObject::trUtf8( "Précipitations" ));
}

/**
 * Update the curve with the new data
 *
 * @param x1 : QVector<double> - x value for Rain curve
 * @param y1 : QVector<double> - y value for Rain curve
 */
void RainGraph::update( QVector<double> x1, QVector<double> y1 )
{
    QVector<QwtIntervalSample> samples( x1.count() );
    for ( int i = 0; i < x1.count(); i++ )
    {
        QwtInterval interval( double( i ), i + 1.0);
        interval.setBorderFlags( QwtInterval::ExcludeMaximum );

        samples[ i ] = QwtIntervalSample( y1[ i ], interval );
    }

    rainCurve->setData( new QwtIntervalSeriesData( samples ));
    rainPlot->replot();
}
