/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <qwt_text.h>
#include <qwt_legend.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include "TemperaturesGraph.h"
#include "WsMeteo.h"
#include "Ws2355.h"

TemperaturesGraph::TemperaturesGraph(double min, double max, double step , int unit)
{
    xMin = min;
    xMax = max;
    xStep = step;

    temperaturesPlot = new QwtPlot();
    temperaturesLeftAxisTitle = new QwtText( QObject::trUtf8( "T e m p é r a t u r e s   ( ° C )" ));
    temperaturesLeftAxisTitle->setFont( QFont("Arial", 9, QFont::Bold ));

    temperaturesPlot->setAxisTitle( QwtPlot::yLeft, *temperaturesLeftAxisTitle );
    temperaturesPlot->setAxisScale( QwtPlot::xBottom, xMin, xMax, xStep );
    temperaturesPlot->setAxisMaxMinor ( QwtPlot::xBottom,1 );
    if( unit == FAHRENHEIT )
        temperaturesPlot->setAxisScale( QwtPlot::yLeft, ( WSMETEO_TEMPERATURE_MIN * 9 / 5 ) + 32 ,( WSMETEO_TEMPERATURE_MAX  * 9 / 5 ) + 32, 5 );
    else
        temperaturesPlot->setAxisScale( QwtPlot::yLeft, WSMETEO_TEMPERATURE_MIN , WSMETEO_TEMPERATURE_MAX, 5 );

    temperaturesPlot->setAxisMaxMinor ( QwtPlot::yLeft,1 );
    temperaturesPlot->setMinimumSize( 350,250 );
    temperaturesPlot->setAxisFont( QwtPlot::yLeft, QFont("Arial", 8, QFont::Normal ));
    temperaturesPlot->setAxisFont( QwtPlot::xBottom, QFont("Arial", 7, QFont::Normal ));

    // Display the legend
    QwtLegend *temperaturesLegend = new QwtLegend;
    temperaturesLegend->setFrameStyle( QFrame::Box|QFrame::Sunken );
    temperaturesPlot->insertLegend( temperaturesLegend, QwtPlot::BottomLegend );

    // Four curves creation
    inTempCurve = new QwtPlotCurve( QObject::trUtf8( "Intérieure" ));
    inTempCurve->setPen( QPen( Qt::red, 2 ));
    inTempCurve->attach( temperaturesPlot );
    outTempCurve = new QwtPlotCurve( QObject::trUtf8( "Extérieure" ));
    outTempCurve->setPen( QPen( Qt::blue, 2 ));
    outTempCurve->attach( temperaturesPlot );
    dewPointTempCurve = new QwtPlotCurve( QObject::trUtf8( "Point de rosée" ));
    dewPointTempCurve->setPen( QPen( Qt::yellow, 2 ));
    dewPointTempCurve->attach( temperaturesPlot );
}

TemperaturesGraph::~TemperaturesGraph()
{
    delete temperaturesPlot;
    delete inTempCurve;
    delete outTempCurve;
    delete dewPointTempCurve;
    delete temperaturesLeftAxisTitle;
}

void TemperaturesGraph::setXAxisMax( double max)
{
    temperaturesPlot->setAxisScale( QwtPlot::xBottom, xMin, max, xStep );
    temperaturesPlot->replot();
}

void TemperaturesGraph::setText( void )
{
    temperaturesLeftAxisTitle->setText( QObject::trUtf8( "T e m p é r a t u r e s   ( ° C )" ));
    temperaturesPlot->setAxisTitle( QwtPlot::yLeft, *temperaturesLeftAxisTitle );
    inTempCurve->setTitle( QObject::trUtf8( "Intérieure" ));
    outTempCurve->setTitle( QObject::trUtf8( "Extérieure" ));
    dewPointTempCurve->setTitle( QObject::trUtf8( "Point de rosée" ));
}


/**
 * Update the four curves with the new data
 *
 * @param x1 : QVector<double> - x value for In temperature curve
 * @param y1 : QVector<double> - y value for In temperature curve
 * @param x2 : QVector<double> - x value for Out temperature curve
 * @param y2 : QVector<double> - y value for Out temperature curve
 * @param x3 : QVector<double> - x value for Dew point temperature curve
 * @param y3 : QVector<double> - y value for Dew point temperature curve
 */
void TemperaturesGraph::update( QVector<double> x1, QVector<double> y1, QVector<double> x2, QVector<double> y2,
                                QVector<double> x3, QVector<double> y3 )
{
    inTempCurve->setSamples( x1, y1 );
    outTempCurve->setSamples( x2, y2 );
    dewPointTempCurve->setSamples( x3, y3 );

    temperaturesPlot->replot();
}
