/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <qwt_text.h>
#include <qwt_legend.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include "WindGraph.h"

WindGraph::WindGraph( double min, double max, double step )
{
    xMin = min;
    xMax = max;
    xStep = step;

    windPlot = new QwtPlot();
    windLeftAxisTitle = new QwtText( QObject::trUtf8( "D i r e c t i o n   ( ° )" ));
    windLeftAxisTitle->setFont( QFont("Arial", 9, QFont::Bold ));
    rightAxisTitle = new QwtText( QObject::trUtf8( "V i t e s s e   ( k m / h )" ));
    rightAxisTitle->setFont( QFont("Arial", 9, QFont::Bold ));
    windPlot->setAxisTitle( QwtPlot::yLeft, *windLeftAxisTitle );
    windPlot->setAxisTitle( QwtPlot::yRight, *rightAxisTitle );
    windPlot->setAxisScale( QwtPlot::xBottom, min, max, step );
    windPlot->setAxisScale( QwtPlot::yLeft, 0, 360, 30 );
    windPlot->setAxisScale( QwtPlot::yRight, 0, 50, 10 );
    windPlot->setAxisMaxMinor ( QwtPlot::xBottom, 1 );
    windPlot->setAxisMaxMinor ( QwtPlot::yLeft, 0 );
    windPlot->setAxisMaxMinor ( QwtPlot::yRight, 1 );
    windPlot->setAxisFont( QwtPlot::yRight , QFont("Arial", 8, QFont::Normal));
    windPlot->setAxisFont( QwtPlot::yLeft, QFont("Arial", 8, QFont::Normal));
    windPlot->setAxisFont( QwtPlot::xBottom, QFont("Arial", 7, QFont::Normal));
    windPlot->setMinimumSize( 350,250 );
    windPlot->enableAxis(  QwtPlot::yRight, true );

    // Display the legend
    QwtLegend *windLegend = new QwtLegend;
    windLegend->setFrameStyle( QFrame::Box|QFrame::Sunken );
    windPlot->insertLegend( windLegend, QwtPlot::BottomLegend );

    // One curve creation
    windDirectionCurve = new QwtPlotCurve( QObject::trUtf8( "Direction" ));
    windDirectionCurve->setPen( QPen( Qt::red, 2 ));
    windSpeedCurve = new QwtPlotCurve( QObject::trUtf8( "Vitesse" ));
    windSpeedCurve->setPen( QPen( Qt::blue, 2 ));

    windDirectionCurve->attach( windPlot );
    windSpeedCurve->attach( windPlot );

}

WindGraph::~WindGraph()
{
    delete windPlot;
    delete windDirectionCurve;
    delete windSpeedCurve;
    delete windLeftAxisTitle;
    delete rightAxisTitle;
}

void WindGraph::setText( void )
{
    windLeftAxisTitle->setText( QObject::trUtf8( "D i r e c t i o n   ( ° )" ));
    rightAxisTitle->setText( QObject::trUtf8( "V i t e s s e   ( k m / h )" ));
    windPlot->setAxisTitle( QwtPlot::yLeft, *windLeftAxisTitle );
    windPlot->setAxisTitle( QwtPlot::yRight, *rightAxisTitle );
    windDirectionCurve->setTitle( QObject::trUtf8( "Direction" ));
    windSpeedCurve->setTitle( QObject::trUtf8( "Vitesse" ));
}

void WindGraph::setXAxisMax( double max)
{
    windPlot->setAxisScale( QwtPlot::xBottom, xMin, max, xStep );
    windPlot->replot();
}


/**
 * Update the two curves with the new data
 *
 * @param x1 : QVector<double> - x value for Wind direction curve
 * @param y1 : QVector<double> - y value for Wind direction curve
 * @param x2 : QVector<double> - x value for Wind speed curve
 * @param y2 : QVector<double> - y value for Wind speed curve
 */
void WindGraph::update( QVector<double> x1, QVector<double> y1, QVector<double> x2, QVector<double> y2 )
{
    windDirectionCurve->setSamples( x1, y1 );
    windSpeedCurve->setSamples( x2, y2 );

    windPlot->replot();
}
