/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <errno.h>
#include <QSettings>
#include <QTimer>
#include <QTime>
#include <QDebug>
#include "Ws2355.h"
#include <unistd.h>
#include <QCoreApplication>
#include <termios.h>
#include <math.h>
#include "Ws2355Thread.h"
#include "Connection.h"
#include "DataLogged.h"
#include "Ws2355Thread.h"

Ws2355::Ws2355( Configuration *globalConfiguration )
{
    theGlobalConfiguration = globalConfiguration;
    theDataLogged = new DataLogged( globalConfiguration );
    theCurrentData = new CurrentData();
    theConnection = new Connection();

    newThread = new Ws2355Thread( this, theGlobalConfiguration, theCurrentData, theConnection );

    readSettings();
}

Ws2355::~Ws2355()
{
    writeSettings();

    delete theDataLogged;
    delete theConnection;
    delete theCurrentData;
    delete newThread;
}


void Ws2355::readSettings( void )
{
    QSettings settings("Gil", "WsMeteo");

    theStationConfiguration.inTemperatureMinConf = settings.value( "inTemperatureMinConf" ).toDouble();
    theStationConfiguration.inTemperatureMinAlarmConf = settings.value( "inTemperatureMinAlarmConf" ).toBool();
    theStationConfiguration.inTemperatureMaxConf = settings.value( "inTemperatureMaxConf" ).toDouble();
    theStationConfiguration.inTemperatureMaxAlarmConf = settings.value( "inTemperatureMaxAlarmConf" ).toBool();
    theStationConfiguration.outTemperatureMinConf = settings.value( "outTemperatureMinConf" ).toDouble();
    theStationConfiguration.outTemperatureMinAlarmConf = settings.value( "outTemperatureMinAlarmConf" ).toBool();
    theStationConfiguration.outTemperatureMaxConf = settings.value( "outTemperatureMaxConf" ).toDouble();
    theStationConfiguration.outTemperatureMaxAlarmConf = settings.value( "outTemperatureMaxAlarmConf" ).toBool();
    theStationConfiguration.windChillTemperatureMinConf = settings.value( "windChillTemperatureMinConf" ).toDouble();
    theStationConfiguration.windChillTemperatureMinAlarmConf = settings.value( "windChillTemperatureMinAlarmConf" ).toBool();
    theStationConfiguration.windChillTemperatureMaxConf = settings.value( "windChillTemperatureMaxConf" ).toDouble();
    theStationConfiguration.windChillTemperatureMaxAlarmConf = settings.value( "windChillTemperatureMaxAlarmConf" ).toBool();
    theStationConfiguration.dewPointTemperatureMinConf = settings.value( "dewPointTemperatureMinConf" ).toDouble();
    theStationConfiguration.dewPointTemperatureMinAlarmConf = settings.value( "dewPointTemperatureMinAlarmConf" ).toBool();
    theStationConfiguration.dewPointTemperatureMaxConf = settings.value( "dewPointTemperatureMaxConf" ).toDouble();
    theStationConfiguration.dewPointTemperatureMaxAlarmConf = settings.value( "dewPointTemperatureMaxAlarmConf" ).toBool();

    theStationConfiguration.rain1HConf = settings.value( "rain1HConf" ).toDouble();
    theStationConfiguration.rain1HAlarmConf = settings.value( "rain1HAlarmConf" ).toBool();
    theStationConfiguration.rain1DayConf = settings.value( "rain1DayConf" ).toDouble();
    theStationConfiguration.rain1DayAlarmConf = settings.value( "rain1DayAlarmConf" ).toBool();

    theStationConfiguration.inHumidityMinConf = settings.value( "inHumidityMinConf" ).toDouble();
    theStationConfiguration.inHumidityMinAlarmConf = settings.value( "inHumidityMinAlarmConf" ).toBool();
    theStationConfiguration.inHumidityMaxConf = settings.value( "inHumidityMaxConf" ).toDouble();
    theStationConfiguration.inHumidityMaxAlarmConf = settings.value( "inHumidityMaxAlarmConf" ).toBool();

    theStationConfiguration.outHumidityMinConf = settings.value( "outHumidityMinConf" ).toDouble();
    theStationConfiguration.outHumidityMinAlarmConf = settings.value( "outHumidityMinAlarmConf" ).toBool();
    theStationConfiguration.outHumidityMaxConf = settings.value( "outHumidityMaxConf" ).toDouble();
    theStationConfiguration.outHumidityMaxAlarmConf = settings.value( "outHumidityMaxAlarmConf" ).toBool();

    theStationConfiguration.pressureMinConf = settings.value( "pressureMinConf" ).toDouble();
    theStationConfiguration.pressureMinAlarmConf = settings.value( "pressureMinAlarmConf" ).toBool();
    theStationConfiguration.pressureMaxConf = settings.value( "pressureMaxConf" ).toDouble();
    theStationConfiguration.pressureMaxAlarmConf = settings.value( "pressureMaxAlarmConf" ).toBool();

    theStationConfiguration.windSpeedMinConf = settings.value( "windSpeedMinConf" ).toDouble();
    theStationConfiguration.windSpeedMinAlarmConf = settings.value( "windSpeedMinAlarmConf" ).toBool();
    theStationConfiguration.windSpeedMaxConf = settings.value( "windSpeedMaxConf" ).toDouble();
    theStationConfiguration.windSpeedMaxAlarmConf = settings.value( "windSpeedMaxAlarmConf" ).toBool();

    theStationConfiguration.windDirectionConf = settings.value( "windDirectionConf" ).toDouble();
    theStationConfiguration.windDirectionAlarmConf = settings.value( "windDirectionAlarmConf" ).toBool();
}

void Ws2355::writeSettings( void )
{
    QSettings settings("Gil", "WsMeteo");

    settings.setValue( "inTemperatureMinConf", theStationConfiguration.inTemperatureMinConf );
    settings.setValue( "inTemperatureMinAlarmConf", theStationConfiguration.inTemperatureMinAlarmConf );
    settings.setValue( "inTemperatureMaxConf", theStationConfiguration.inTemperatureMaxConf );
    settings.setValue( "inTemperatureMaxAlarmConf", theStationConfiguration.inTemperatureMaxAlarmConf );
    settings.setValue( "outTemperatureMinConf", theStationConfiguration.outTemperatureMinConf );
    settings.setValue( "outTemperatureMinAlarmConf", theStationConfiguration.outTemperatureMinAlarmConf );
    settings.setValue( "outTemperatureMaxConf", theStationConfiguration.outTemperatureMaxConf );
    settings.setValue( "outTemperatureMaxAlarmConf", theStationConfiguration.outTemperatureMaxAlarmConf );
    settings.setValue( "windChillTemperatureMinConf", theStationConfiguration.windChillTemperatureMinConf );
    settings.setValue( "windChillTemperatureMinAlarmConf", theStationConfiguration.windChillTemperatureMinAlarmConf );
    settings.setValue( "windChillTemperatureMaxConf", theStationConfiguration.windChillTemperatureMaxConf );
    settings.setValue( "windChillTemperatureMaxAlarmConf", theStationConfiguration.windChillTemperatureMaxAlarmConf );
    settings.setValue( "dewPointTemperatureMinConf", theStationConfiguration.dewPointTemperatureMinConf );
    settings.setValue( "dewPointTemperatureMinAlarmConf", theStationConfiguration.dewPointTemperatureMinAlarmConf );
    settings.setValue( "dewPointTemperatureMaxConf", theStationConfiguration.dewPointTemperatureMaxConf );
    settings.setValue( "dewPointTemperatureMaxAlarmConf", theStationConfiguration.dewPointTemperatureMaxAlarmConf );

    settings.setValue( "rain1HConf", theStationConfiguration.rain1HConf );
    settings.setValue( "rain1HAlarmConf", theStationConfiguration.rain1HAlarmConf );
    settings.setValue( "rain1DayConf", theStationConfiguration.rain1DayConf );
    settings.setValue( "rain1DayAlarmConf", theStationConfiguration.rain1DayAlarmConf );

    settings.setValue( "inHumidityMinConf", theStationConfiguration.inHumidityMinConf );
    settings.setValue( "inHumidityMinAlarmConf", theStationConfiguration.inHumidityMinAlarmConf );
    settings.setValue( "inHumidityMaxConf", theStationConfiguration.inHumidityMaxConf );
    settings.setValue( "inHumidityMaxAlarmConf", theStationConfiguration.inHumidityMaxAlarmConf );

    settings.setValue( "outHumidityMinConf", theStationConfiguration.outHumidityMinConf );
    settings.setValue( "outHumidityMinAlarmConf", theStationConfiguration.outHumidityMinAlarmConf );
    settings.setValue( "outHumidityMaxConf", theStationConfiguration.outHumidityMaxConf );
    settings.setValue( "outHumidityMaxAlarmConf", theStationConfiguration.outHumidityMaxAlarmConf );

    settings.setValue( "pressureMinConf", theStationConfiguration.pressureMinConf );
    settings.setValue( "pressureMinAlarmConf", theStationConfiguration.pressureMinAlarmConf );
    settings.setValue( "pressureMaxConf", theStationConfiguration.pressureMaxConf );
    settings.setValue( "pressureMaxAlarmConf", theStationConfiguration.pressureMaxAlarmConf );

    settings.setValue( "windSpeedMinConf", theStationConfiguration.windSpeedMinConf );
    settings.setValue( "windSpeedMinAlarmConf", theStationConfiguration.windSpeedMinAlarmConf );
    settings.setValue( "windSpeedMaxConf", theStationConfiguration.windSpeedMaxConf );
    settings.setValue( "windSpeedMaxAlarmConf", theStationConfiguration.windSpeedMaxAlarmConf );

    settings.setValue( "windDirectionConf", theStationConfiguration.windDirectionConf );
    settings.setValue( "windDirectionAlarmConf", theStationConfiguration.windDirectionAlarmConf );
}







/**
 * Read the current values
 */
void Ws2355::readCurrentValues( void )
{

}


/**
 * Read indoor temperature, current temperature only
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readTemperatureIndoor( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( CURRENT_INDOOR_TEMPERATURE, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}

/**
 * Read indoor max temperature of the alarm
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readIndoorTemperatureMax( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( HIGH_ALARM_INDOOR_TEMPERATURE, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}

/**
 * Read indoor min temperature of the alarm
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readIndoorTemperatureMin( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( LOW_ALARM_INDOOR_TEMPERATURE, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}

/**
 * Read indoor temperature max alarm
 * *
 * @return bool : Indoor temperature max alarm
 */
bool Ws2355::readIndoorTemperatureMaxAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( INDOOR_TEMPERATURE_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & INDOOR_TEMPERATURE_MAX_ALARM_BIT ) == INDOOR_TEMPERATURE_MAX_ALARM_BIT );
}

/**
 * Read indoor temperature min alarm
 * *
 * @return bool : Indoor temperature min alarm
 */
bool Ws2355::readIndoorTemperatureMinAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( INDOOR_TEMPERATURE_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & INDOOR_TEMPERATURE_MIN_ALARM_BIT ) == INDOOR_TEMPERATURE_MIN_ALARM_BIT );
}

/**
 * Read indoor min/max temperatures with timestamps
 *
 * @param temperatureUnit ( IN Parameter ) controlling convertion to deg F
 *
 * @param tempMin ( OUT Parameter ) Temperatures tempMin
 * @param tempMax ( OUT Parameter ) Temperatures tempMax
 *                (deg C if temperatureUnit is CELCIUS )
 *                (deg F if temperatureUnit is FAHRENHEIT )
 * @param tempMin ( OUT Parameter )
 * @param tempMax ( OUT Parameter )
 *         Timestamps for tempMin and tempMax in pointers to
 *                timestamp structures for timeMin and timeMax
 *
 */
void Ws2355::temperatureIndoorMinMax( int temperatureUnit,
                               double *tempMin,
                               double *tempMax,
                               QDateTime *timeMin,
                               QDateTime *timeMax)
{
        unsigned char data[ 20 ];
        unsigned char command[ 25 ];	//room for write data also

        if(  readSafe( MINIMUM_INDOOR_TEMPERATURE, 15, data, command ) != 15 )
                readErrorExit();

        *tempMin = (( data[ 1 ] >> 4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
                    ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0;

        *tempMax = (( data[ 4 ] & 0xF ) * 10 + ( data[ 3 ] >> 4 ) + ( data[ 3 ] & 0xF ) / 10.0 +
                    ( data[ 2 ] >> 4 ) / 100.0 ) - 30.0;

        // Fahrenheit conversion
        if(  temperatureUnit == FAHRENHEIT )
        {
                *tempMin = *tempMin * 9 / 5 + 32;
                *tempMax = *tempMax * 9 / 5 + 32;
        }

        QTime t(((data[ 6 ] & 0xF) * 10) + (data[ 5 ] >> 4),
                ((data[ 5 ] & 0xF) * 10) + (data[ 4 ] >> 4));
        timeMin->setTime( t );

        QDate d(2000 + ((data[ 9 ] & 0xF) * 10) + (data[ 8 ] >> 4),
                       ((data[ 8 ] & 0xF) * 10) + (data[ 7 ] >> 4),
                       ((data[ 7 ] & 0xF) * 10) + (data[ 6 ] >> 4));

        timeMin->setDate( d );

        QTime t1( ((data[11] & 0xF) * 10) + (data[10] >> 4),
                  ((data[10] & 0xF) * 10) + (data[ 9 ] >> 4));
        timeMax->setTime( t1 );

        QDate d1( 2000 + ((data[14] & 0xF) * 10) + (data[13] >> 4),
                         ((data[13] & 0xF) * 10) + (data[12] >> 4),
                         ((data[12] & 0xF) * 10) + (data[11] >> 4));
        timeMax->setDate( d1 );

        return;
}

/**
 * Reset indoor min/max temperatures with timestamps
 *
 * @param minmax ( IN Parameter ) char (8 bit integer) that controls if minimum,
 *                 maximum or both are reset
 * @returns 1 if success
 */
int Ws2355::temperatureIndoorReset( char minmax )
{
        unsigned char data_read[ 20 ];
        unsigned char data_value[ 20 ];
        unsigned char data_time[ 20 ];
        unsigned char command[ 25 ];	//room for write data also

        // First read current temperature into data_value
        if( readSafe( CURRENT_INDOOR_TEMPERATURE, 2, data_read, command ) != 2 )
            readErrorExit();

        data_value[ 0 ] = data_read[ 0 ] &  0xF;
        data_value[ 1 ] = data_read[ 0 ] >> 4;
        data_value[ 2 ] = data_read[ 1 ] &  0xF;
        data_value[ 3 ] = data_read[ 1 ] >> 4;

        // Get current time from station
        if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
            readErrorExit();

        data_time[ 0 ] = data_read[ 0 ] &  0xF;
        data_time[ 1 ] = data_read[ 0 ] >> 4;
        data_time[ 2 ] = data_read[ 1 ] &  0xF;
        data_time[ 3 ] = data_read[ 1 ] >> 4;
        data_time[ 4 ] = data_read[ 2 ] >> 4;
        data_time[ 5 ] = data_read[ 3 ] &  0xF;
        data_time[ 6 ] = data_read[ 3 ] >> 4;
        data_time[ 7 ] = data_read[ 4 ] &  0xF;
        data_time[ 8 ] = data_read[ 4 ] >> 4;
        data_time[ 9 ] = data_read[ 5 ] &  0xF;

        if( minmax & RESET_MIN ) // minimum
        {
            // Set min value to current value
            if( writeSafe( MINIMUM_INDOOR_TEMPERATURE, 4, WRITENIB, data_value, command ) != 4 )
                    writeErrorExit();

            // Set min value timestamp to current time
            if( writeSafe( TIME_MINIMUM_INDOOR_TEMPERATURE, 10, WRITENIB, data_time, command ) != 10 )
                    writeErrorExit();
        }

        if( minmax & RESET_MAX ) // maximum
        {
            // Set max value to current value
            if( writeSafe( MAXIMUM_INDOOR_TEMPERATURE, 4, WRITENIB, data_value, command ) != 4 )
                writeErrorExit();

            // Set max value timestamp to current time
            if( writeSafe( TIME_MAXIMUM_INDOOR_TEMPERATURE, 10, WRITENIB, data_time, command ) != 10 )
                writeErrorExit();
        }

        return 1;
}


/**
 * Read indoor temperature, current temperature only
 *
 * @param temperatureUnit  controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readTemperatureOutdoor( int temperatureUnit )
{
        unsigned char data[ 20 ];
        unsigned char command[ 25 ];	//room for write data also

        if( readSafe( CURRENT_OUTDOOR_TEMPERATURE, 2, data, command ) != 2 )
            readErrorExit();

        if( temperatureUnit == FAHRENHEIT )
            return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                      ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) -
                          30.0 ) * 9 / 5 + 32 );
        else
            return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                      ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0) - 30.0 ));
}

/**
 * Read outdoor max temperature of the alarm
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readOutdoorTemperatureMax( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( HIGH_ALARM_OUTDOOR_TEMPERATURE, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}

/**
 * Read outdoor temperature max alarm
 * *
 * @return bool : Outdoor temperature max alarm
 */
bool Ws2355::readOutdoorTemperatureMaxAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( OUTDOOR_TEMPERATURE_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & OUTDOOR_TEMPERATURE_MAX_ALARM_BIT ) == OUTDOOR_TEMPERATURE_MAX_ALARM_BIT );
}

/**
 * Read outdoor min temperature of the alarm
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readOutdoorTemperatureMin( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( LOW_ALARM_OUTDOOR_TEMPERATURE, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}

/**
 * Read outdoor temperature min alarm
 * *
 * @return bool : Outdoor temperature min alarm
 */
bool Ws2355::readOutdoorTemperatureMinAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( OUTDOOR_TEMPERATURE_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & OUTDOOR_TEMPERATURE_MIN_ALARM_BIT ) == OUTDOOR_TEMPERATURE_MIN_ALARM_BIT );
}

/**
 * Read outdoor min/max temperatures with timestamps
 *
 * @param temperatureUnit ( IN Parameter ) controlling convertion to deg F
 *
 * @param tempMin ( OUT Parameter ) Temperatures tempMin
 * @param tempMax ( OUT Parameter ) Temperatures tempMax
 *                (deg C if temperatureUnit is CELCIUS )
 *                (deg F if temperatureUnit is FAHRENHEIT )
 * @param tempMin ( OUT Parameter )
 * @param tempMax ( OUT Parameter )
 *         Timestamps for tempMin and tempMax in pointers to
 *                timestamp structures for timeMin and timeMax
 */
void Ws2355::temperatureOutdoorMinMax( int temperatureUnit,
                                       double *tempMin,
                                       double *tempMax,
                                       QDateTime *timeMin,
                                       QDateTime *timeMax)
{
        unsigned char data[ 20 ];
        unsigned char command[ 25 ];	//room for write data also

        if( readSafe( MINIMUM_OUTDOOR_TEMPERATURE, 15, data, command ) != 15 )
                readErrorExit();

        *tempMin = (( data[ 1 ] >> 4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
                    ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0;

        *tempMax = (( data[ 4 ] & 0xF ) * 10 + ( data[ 3 ] >> 4 ) + ( data[ 3 ] & 0xF ) / 10.0 +
                    ( data[ 2 ] >> 4 ) / 100.0 ) - 30.0;

        if( temperatureUnit == FAHRENHEIT )
        {
                *tempMin = *tempMin * 9 / 5 + 32;
                *tempMax = *tempMax * 9 / 5 + 32;
        }

        QTime t((( data[ 6 ] & 0xF ) * 10 ) + ( data[ 5 ] >> 4 ),
                (( data[ 5 ] & 0xF ) * 10 ) + ( data[ 4 ] >> 4 ));
        timeMin->setTime( t );
        QDate d( 2000 + (( data[ 9 ] & 0xF ) * 10 ) + ( data[ 8 ] >> 4 ),
                        (( data[ 8 ] & 0xF ) * 10 ) + ( data[ 7 ] >> 4 ),
                        (( data[ 7 ] & 0xF ) * 10 ) + ( data[ 6 ] >> 4 ));
        timeMin->setDate( d );

        QTime t1((( data[ 11 ] & 0xF ) * 10 ) + ( data[ 10 ] >> 4 ),
                 (( data[ 10 ] & 0xF ) * 10 ) + ( data[  9 ] >> 4 ));

        timeMax->setTime( t1 );
        QDate d1( 2000 + (( data[ 14 ] & 0xF ) * 10 ) + ( data[ 13 ] >> 4 ),
                         (( data[ 13 ] & 0xF ) * 10 ) + ( data[ 12 ] >> 4 ),
                         (( data[ 12 ] & 0xF ) * 10 ) + ( data[ 11 ] >> 4 ));
        timeMax->setDate( d1 );

        return;
}


/**
 * Reset outdoor min/max temperatures with timestamps
 *
 * @param minmax ( IN Parameter ) char (8 bit integer) that controls if minimum,
 *                 maximum or both are reset
 *
 * @return 1 if success
 *
 */
int Ws2355::temperatureOutdoorReset( char minmax )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    // First read current temperature into data_value
    if( readSafe( CURRENT_OUTDOOR_TEMPERATURE, 2, data_read, command ) != 2 )
        readErrorExit();

    data_value[ 0 ] = data_read[ 0 ] &  0xF;
    data_value[ 1 ] = data_read[ 0 ] >> 4;
    data_value[ 2 ] = data_read[ 1 ] &  0xF;
    data_value[ 3 ] = data_read[ 1 ] >> 4;

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    if( minmax & RESET_MIN ) // minimum
    {
        // Set min value to current value
        if( writeSafe( MINIMUM_OUTDOOR_TEMPERATURE, 4, WRITENIB, data_value, command ) != 4 )
            writeErrorExit();

        // Set min value timestamp to current time
        if( writeSafe( TIME_MINIMUM_OUTDOOR_TEMPERATURE, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    if( minmax & RESET_MAX ) // maximum
    {
        // Set max value to current value
        if( writeSafe( MAXIMUM_OUTDOOR_TEMPERATURE, 4, WRITENIB, data_value, command ) != 4 )
            writeErrorExit();

        // Set max value timestamp to current time
        if( writeSafe( TIME_MAXIMUM_OUTDOOR_TEMPERATURE, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    return 1;
}


/**
 * Read dewpoint, current value only
 *
 * @param temperatureUnit ( IN Parameter ) controlling convertion to deg F
 *
 * @return  Dewpoint    ( deg C if temperatureUnit is CELCIUS )
 *                      ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readDewpoint( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( CURRENT_DEWPOINT, 2, data, command ) != 2 )
         readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) -
                      30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0) - 30.0 ));
}

/**
 * Read dewpoint max temperature of the alarm
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readDewpointMax( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( HIGH_ALARM_DEWPOINT, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}

/**
 * Read dewpoint max alarm
 * *
 * @return bool : dewpoint max alarm
 */
bool Ws2355::readDewpointMaxAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( DEWPOINT_ALARM, 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & DEWPOINT_MAX_ALARM_BIT ) == DEWPOINT_MAX_ALARM_BIT );
}

/**
 * Read dewpoint min temperature of the alarm
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readDewpointMin( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( LOW_ALARM_DEWPOINT, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}


/**
 * Read dewpoint min alarm
 * *
 * @return bool : dewpoint main alarm
 */
bool Ws2355::readDewpointMinAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( DEWPOINT_ALARM, 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & DEWPOINT_MIN_ALARM_BIT ) == DEWPOINT_MIN_ALARM_BIT );
}

void Ws2355::writeDewPointMaxAlarm( bool val )
{
    //    unsigned char command[ 25 ];	//room for write data also
    //    unsigned char data[5];

    //    data[0] = DEWPOINT_MAX_ALARM_BIT;
    //    data[1] = 0;

    if( val == true )
    {
        //if( writeSafe( DEWPOINT_ALARM, 1, SETBIT, data, command ) != 1 )
        //    writeErrorExit();
    }
    else
    {
        //if( writeSafe( DEWPOINT_ALARM, 1, UNSETBIT, data, command ) != 1 )
        //    writeErrorExit();
    }

    theStationConfiguration.dewPointTemperatureMaxAlarmConf = val;

}


/**
 * Read outdoor min/max dewpoint with timestamps
 *
 * @param temperatureUnit ( IN Parameter ) controlling convertion to deg F
 *
 * @param dp_min ( OUT Parameter ) Dewpoints min
 * @param dp_max ( OUT Parameter ) Dewpoints max
 *                (deg C if temperatureUnit is 0),
 *                (deg F if temperatureUnit is 1)
 * @param timeMin ( OUT Parameter )  Timestamps for dp_min
 * @param timeMax ( OUT Parameter )  Timestamps for dp_max
 */
void Ws2355::dewpointMinMax( int temperatureUnit,
                             double *dp_min,
                             double *dp_max,
                             QDateTime *timeMin,
                             QDateTime *timeMax)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( MINIMUM_DEWPOINT, 15, data, command ) != 15 )
        readErrorExit();

    *dp_min = (( data[ 1 ] >> 4   ) * 10  + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
               ( data[ 0 ] &  0xF ) / 100.0) - 30.0;

    *dp_max = (( data[ 4 ] &  0xF ) * 10 + ( data[ 3 ] >> 4 ) + ( data[ 3 ] & 0xF ) / 10.0 +
               ( data[ 2 ] >> 4 ) / 100.0) - 30.0;

    if(  temperatureUnit == FAHRENHEIT )
    {
        *dp_min = *dp_min * 9 / 5 + 32;
        *dp_max = *dp_max * 9 / 5 + 32;
    }

    QTime t((( data[ 6 ] & 0xF ) * 10 ) + ( data[ 5 ] >> 4 ),
            (( data[ 5 ] & 0xF ) * 10 ) + ( data[ 4 ] >> 4 ));
    timeMin->setTime( t );

    QDate d( 2000 + (( data[ 9 ] & 0xF ) * 10 ) + ( data[ 8 ] >> 4 ),
                    (( data[ 8 ] & 0xF ) * 10 ) + ( data[ 7 ] >> 4 ),
                    (( data[ 7 ] & 0xF ) * 10 ) + ( data[ 6 ] >> 4 ));

    timeMin->setDate( d );

    QTime t1((( data[ 11 ] & 0xF ) * 10 ) + ( data[ 10 ] >> 4 ),
             (( data[ 10 ] & 0xF ) * 10 ) + ( data[ 9 ] >> 4 ));
    timeMax->setTime( t1 );

    QDate d1( 2000 + (( data[ 14 ] & 0xF ) * 10 ) + ( data[ 13 ] >> 4 ),
                     (( data[ 13 ] & 0xF ) * 10 ) + ( data[ 12 ] >> 4 ),
                     (( data[ 12 ] & 0xF ) * 10 ) + ( data[ 11 ] >> 4 ));
    timeMax->setDate( d1 );

    return;
}


/**
 * Reset min/max dewpoint with timestamps
 *
 * @param minmax that controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 *
 */
int Ws2355::dewpointReset( char minmax )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    // First read current dewpoint into data_value
    if( readSafe( CURRENT_DEWPOINT, 2, data_read, command ) != 2 )
        readErrorExit();

    data_value[ 0 ] = data_read[ 0 ] &  0xF;
    data_value[ 1 ] = data_read[ 0 ] >> 4;
    data_value[ 2 ] = data_read[ 1 ] &  0xF;
    data_value[ 3 ] = data_read[ 1 ] >> 4;

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    if( minmax & RESET_MIN ) // minimum
    {
        // Set min value to current value
        if( writeSafe( MINIMUM_DEWPOINT, 4, WRITENIB, data_value, command ) != 4 )
            writeErrorExit();

        // Set min value timestamp to current time
        if( writeSafe( TIME_MINIMUM_DEWPOINT, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    if( minmax & RESET_MAX ) // maximum
    {
        // Set max value to current value
        if( writeSafe( MAXIMUM_DEWPOINT, 4, WRITENIB, data_value, command ) != 4 )
            writeErrorExit();

        // Set max value timestamp to current time
        if( writeSafe( TIME_MAXIMUM_DEWPOINT, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    return 1;
}


/**
 * Read indoor relative humidity, current value only
 *
 * @return relative humidity in percent
 *
 */
int Ws2355::readHumidityIndoor( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( REL_HUMIDITY_INDOOR, 1, data, command ) != 1 )
        readErrorExit();

    return (( data[ 0 ] >> 4 ) * 10 + ( data[ 0 ] & 0xF ));
}

/**
 * Read indoor humidity max alarm
 * *
 * @return bool : indoor humidity max alarm
 */
bool Ws2355::readIndoorHumidityMaxAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( INDOOR_HUMIDITY_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & INDOOR_HUMIDITY_MAX_ALARM_BIT ) == INDOOR_HUMIDITY_MAX_ALARM_BIT );
}

/**
 * Read indoor humidity max of the alarm
 *
 * @return relative humidity in percent
 */
int Ws2355::readIndoorHumidityMax( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( HIGH_ALARM_HUMIDITY_INDOOR, 1, data, command ) != 1 )
        readErrorExit();

    return (( data[ 0 ] >> 4 ) * 10 + ( data[ 0 ] & 0xF ));
}

/**
 * Read indoor humidity min alarm
 * *
 * @return bool : indoor humidity min alarm
 */
bool Ws2355::readIndoorHumidityMinAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( INDOOR_HUMIDITY_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & INDOOR_HUMIDITY_MIN_ALARM_BIT ) == INDOOR_HUMIDITY_MIN_ALARM_BIT );
}

/**
 * Read indoor humidity min of the alarm
 *
 * @return relative humidity in percent
 */
int Ws2355::readIndoorHumidityMin( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( LOW_ALARM_HUMIDITY_INDOOR, 1, data, command ) != 1 )
        readErrorExit();

    return (( data[ 0 ] >> 4 ) * 10 + ( data[ 0 ] & 0xF ));
}

/**
 * Read both current indoor humidity and min/max values with timestamps
 *
 * @param humidityMin ( OUT Parameter ) Relative humidity in %
 * @param humidityMax ( OUT Parameter ) Relative humidity in %
 * @param timeMin ( OUT Parameter ) Timestamps for humidityMin
 * @param timeMax ( OUT Parameter ) Timestamps for humidityMax
 *
 * @return releative humidity current value in % (integer)
 *
 */
int Ws2355::humidityIndoorAll( int *humidityMin,
                               int *humidityMax,
                               QDateTime *timeMin,
                               QDateTime *timeMax )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( REL_HUMIDITY_INDOOR, 13, data, command ) != 13 )
        readErrorExit();

    *humidityMin = ( data[ 1 ] >> 4 ) * 10 + ( data[ 1 ] & 0xF );
    *humidityMax = ( data[ 2 ] >> 4 ) * 10 + ( data[ 2 ] & 0xF );

    QTime t(((data[ 4 ] >> 4) * 10) + (data[ 4 ] & 0xF),
            ((data[ 3 ] >> 4) * 10) + (data[ 3 ] & 0xF));
    timeMin->setTime( t );

    QDate d( 2000 + (( data[ 7 ] >> 4 ) * 10 ) + (data[ 7 ] & 0xF ),
                    (( data[ 6 ] >> 4 ) * 10 ) + (data[ 6 ] & 0xF ),
                    (( data[ 5 ] >> 4 ) * 10 ) + (data[ 5 ] & 0xF ));
    timeMin->setDate( d );

    QTime t1(((data[ 9 ] >> 4 ) * 10 ) + ( data[ 9 ] & 0xF ),
             ((data[ 8 ] >> 4 ) * 10 ) + ( data[ 8 ] & 0xF ));
    timeMax->setTime( t1 );

    QDate d1(2000 + (( data[ 12 ] >> 4 ) * 10 ) + ( data[ 12 ] & 0xF ),
                    (( data[ 11 ] >> 4 ) * 10 ) + ( data[ 11 ] & 0xF ),
                    (( data[ 10 ] >> 4 ) * 10 ) + ( data[ 10 ] & 0xF ));
    timeMax->setDate( d1 );

    return ((data[ 0 ] >> 4) * 10 + (data[ 0 ] & 0xF));
}


/**
 * Reset min/max indoor humidity with timestamps
 *
 * @param minmax controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 */
int Ws2355::humidityIndoorReset( char minmax )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    // First read current humidity into data_value
    if( readSafe( REL_HUMIDITY_INDOOR, 1, data_read, command ) != 1 )
        readErrorExit();

    data_value[ 0 ] = data_read[ 0 ] &  0xF;
    data_value[ 1 ] = data_read[ 0 ] >> 4;

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    if( minmax & RESET_MIN ) // minimum
    {
        // Set min value to current value
        if( writeSafe( MINIMUM_HUMIDITY_INDOOR, 2, WRITENIB, data_value, command ) != 2 )
            writeErrorExit();

        // Set min value timestamp to current time
        if( writeSafe( TIME_MINIMUM_HUMIDITY_INDOOR, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    if( minmax & RESET_MAX ) // maximum
    {
        // Set max value to current value
        if( writeSafe( MAXIMUM_HUMIDITY_INDOOR, 2, WRITENIB, data_value, command ) != 2 )
            writeErrorExit();

        // Set max value timestamp to current time
        if( writeSafe( TIME_MAXIMUM_HUMIDITY_INDOOR, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    return 1;
}

/**
 * Read relative humidity, current value only
 *
 * @return relative humidity in percent
 *
 */
int Ws2355::readHumidityOutdoor( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( REL_HUMIDITY_OUTDOOR, 1, data, command ) != 1 )
        readErrorExit();

    return (( data[ 0 ] >> 4 ) * 10 + ( data[ 0 ] & 0xF ));
}

/**
 * Read outdoor humidity max alarm
 *
 * @return relative humidity in percent
 */
bool Ws2355::readOutdoorHumidityMaxAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( OUTDOOR_HUMIDITY_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & OUTDOOR_HUMIDITY_MAX_ALARM_BIT ) == OUTDOOR_HUMIDITY_MAX_ALARM_BIT );
}

/**
 * Read outdoor humidity max of the alarm
 *
 * @return relative humidity in percent
 */
int Ws2355::readOutdoorHumidityMax( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( HIGH_ALARM_HUMIDITY_OUTDOOR, 1, data, command ) != 1 )
        readErrorExit();

    return (( data[ 0 ] >> 4 ) * 10 + ( data[ 0 ] & 0xF ));
}

/**
 * Read outdoor humidity min alarm
 * *
 * @return bool : outdoor humidity min alarm
 */
bool Ws2355::readOutdoorHumidityMinAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( OUTDOOR_HUMIDITY_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & OUTDOOR_HUMIDITY_MIN_ALARM_BIT ) == OUTDOOR_HUMIDITY_MIN_ALARM_BIT );
}

/**
 * Read outdoor humidity min of the alarm
 *
 * @return relative humidity in percent
 */
int Ws2355::readOutdoorHumidityMin( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( LOW_ALARM_HUMIDITY_OUTDOOR, 1, data, command ) != 1 )
        readErrorExit();

    return (( data[ 0 ] >> 4 ) * 10 + ( data[ 0 ] & 0xF ));
}


/**
 * Read both current outdoor humidity and min/max values with timestamps
 *
 * @param humidityMin ( OUT Parameter ) Relative humidity in %
 * @param humidityMax ( OUT Parameter ) Relative humidity in %
 * @param timeMin ( OUT Parameter ) Timestamps for humidityMin
 * @param timeMax ( OUT Parameter ) Timestamps for humidityMax
 *
 * @return releative humidity current value in % (integer)
 *
 */
int Ws2355::humidityOutdoorAll( int *humidityMin,
                                int *humidityMax,
                                QDateTime *timeMin,
                                QDateTime *timeMax)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( REL_HUMIDITY_OUTDOOR, 13, data, command ) != 13 )
        readErrorExit();

    *humidityMin = ( data[ 1 ] >> 4 ) * 10 + ( data[ 1 ] & 0xF );
    *humidityMax = ( data[ 2 ] >> 4 ) * 10 + ( data[ 2 ] & 0xF );

    QTime t((( data[ 4 ] >> 4 ) * 10 ) + ( data[ 4 ] & 0xF ),
            (( data[ 3 ] >> 4 ) * 10 ) + ( data[ 3 ] & 0xF ));
    timeMin->setTime( t );

    QDate d( 2000 + (( data[ 7 ] >> 4 ) * 10 ) + ( data[ 7 ] & 0xF ),
                    (( data[ 6 ] >> 4 ) * 10 ) + ( data[ 6 ] & 0xF ),
                    (( data[ 5 ] >> 4 ) * 10 ) + ( data[ 5 ] & 0xF ));
    timeMin->setDate( d );

    QTime t1((( data[ 9 ] >> 4 ) * 10 ) + ( data[ 9 ] & 0xF ),
             (( data[ 8 ] >> 4 ) * 10 ) + ( data[ 8 ] & 0xF ));
    timeMax->setTime( t1 );

    QDate d1( 2000 + (( data[ 12 ] >> 4 ) * 10 ) + ( data[ 12 ] & 0xF ),
                     (( data[ 11 ] >> 4 ) * 10 ) + ( data[ 11 ] & 0xF ),
                     (( data[ 10 ] >> 4 ) * 10 ) + ( data[ 10 ] & 0xF ));
    timeMax->setDate( d1 );

    return (( data[ 0 ] >> 4 ) * 10 + ( data[ 0 ] & 0xF ));
}


/**
 * Reset min/max outdoor humidity with timestamps
 *
 * @param minmax controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 *
 */
int Ws2355::humidityOutdoorReset( char minmax )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    // First read current humidity into data_value
    if( readSafe( REL_HUMIDITY_OUTDOOR, 1, data_read, command ) != 1 )
        readErrorExit();

    data_value[ 0 ] = data_read[ 0 ] &  0xF;
    data_value[ 1 ] = data_read[ 0 ] >> 4;

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    if( minmax & RESET_MIN ) // minimum
    {
        // Set min value to current value
        if(writeSafe( MINIMUM_HUMIDITY_OUTDOOR, 2, WRITENIB, data_value, command ) != 2 )
           writeErrorExit();

        // Set min value timestamp to current time
        if( writeSafe( TIME_MINIMUM_HUMIDITY_OUTDOOR, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    if( minmax & RESET_MAX ) // maximum
    {
        // Set max value to current value
        if( writeSafe( MAXIMUM_HUMIDITY_OUTDOOR, 2, WRITENIB, data_value, command ) != 2 )
            writeErrorExit();

        // Set max value timestamp to current time
        if( writeSafe( TIME_MAXIMUM_HUMIDITY_OUTDOOR, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    return 1;
}


/**
 * Read wind speed, wind direction and last 5 wind directions
 *
 * @param windSpeedUnit ( IN Parameter)controlling convertion to other units than m/s
 *
 * @param windDirection ( OUT Parameter) pointer to double in degrees
 *
 * @return Wind speed (double) in the unit given in the loaded config
 *
 */
double Ws2355::readWindCurrent( double  windSpeedUnit,
                            double *windDirection)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also
    int i;

    for( i = 0; i < MAXWINDRETRIES; i ++ )
    {
        if( readSafe( WIND_SPEED_AND_DIRECTION, 3, data, command ) != 3 ) //Wind
            readErrorExit();

        if(( data[ 0 ] != 0x00 ) ||                            //Invalid wind data
           (( data[ 1 ] == 0xFF ) && ((( data[ 2 ] & 0xF ) == 0 ) || (( data[ 2 ] & 0xF ) == 1 ))))
        {
            sleep( 10 ); //wait 10 seconds for new wind measurement
            continue;
        }
        else
        {
            break;
        }
    }

    //Calculate wind directions

    *windDirection = ( data[ 2 ] >> 4 ) * 22.5;

    //Calculate raw wind speed 	- convert from m/s to whatever
    return(((( data[ 2 ] & 0xF ) << 8 ) + ( data[ 1 ])) / 10.0 *  windSpeedUnit );
}


/**
 * Read wind direction alarm
 * *
 * @return bool : wind direction alarm
 */
bool Ws2355::readWindDirectionAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( WIND_DIRECTION_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & WIND_DIR_ALARM_BIT ) == WIND_DIR_ALARM_BIT );
}


/**
 * Read wind direction of the alarm
 * *
 * @return double : wind direction
 */
double Ws2355::readWindDirection( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( WIND_DIRECTION_ALARM_SETTING , 1, data, command ) != 1 )
         readErrorExit();

    return ( data[ 0 ] * 22.5 );
}

/**
 * Read wind speed max alarm
 * *
 * @return bool : wind speed max alarm
 */
bool Ws2355::readWindSpeedMaxAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( WIND_SPEED_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & WIND_SPEED_MAX_ALARM_BIT ) == WIND_SPEED_MAX_ALARM_BIT );
}

/**
 * Read max wind speed of the alarm
 *
 * @param windSpeedUnit ( IN Parameter)controlling convertion to other units than m/s
 *
 * @return Wind speed (double) in the unit given in the loaded config
 *
 */
double Ws2355::readWindSpeedMax( double  windSpeedUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( HIGH_WIND_ALARM_SETTING, 2, data, command ) != 2 ) //Wind
        readErrorExit();

    //Calculate raw wind speed 	- convert from m/s to whatever
    return((( data[ 1 ] << 8 ) + ( data[ 0 ])) / 10 * windSpeedUnit );
}

/**
 * Read wind speed min alarm
 * *
 * @return bool : wind speed min alarm
 */
bool Ws2355::readWindSpeedMinAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( WIND_SPEED_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & WIND_SPEED_MIN_ALARM_BIT ) == WIND_SPEED_MIN_ALARM_BIT );
}

/**
 * Read min wind speed of the alarm
 *
 * @param windSpeedUnit ( IN Parameter)controlling convertion to other units than m/s
 *
 * @return Wind speed (double) in the unit given in the loaded config
 *
 */
double Ws2355::readWindSpeedMin( double  windSpeedUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( LOW_WIND_ALARM_SETTING, 2, data, command ) != 2 ) //Wind
        readErrorExit();

    //Calculate raw wind speed 	- convert from m/s to whatever
    return((( data[ 1 ] << 8 ) + ( data[ 0 ])) / 10 * windSpeedUnit );
}

/**
 * Read wind speed, wind direction and last 5 wind directions
 *
 * @param windSpeedUnit controlling convertion to other units than m/s
 *
 * @param winddir_index ( OUT Parameter ) Current wind direction
 *             expressed as ticks from North where North=0.
 *             Used to convert to direction string
 * @param winddir ( OUT Parameter )Array of doubles containing current winddirection
 *              in winddir[ 0 ] and the last 5 in the following
 *              positions all given in degrees
 *
 * @return Wind speed (double) in the unit given in the loaded config
 *
 */
double Ws2355::windAll( double windSpeedUnit,
                        int *winddir_index,
                        double *winddir)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also
    int i;

    for( i = 0; i < MAXWINDRETRIES; i ++ )
    {
        if( readSafe( WIND_SPEED_AND_DIRECTION, 6, data, command ) != 6 ) //Wind
            readErrorExit();

        if((data[ 0 ] != 0x00 ) ||                             //Invalid wind data
           ((data[ 1 ] == 0xFF ) && ((( data[ 2 ] & 0xF ) == 0 ) || ((data[ 2 ] & 0xF ) == 1 ))))
        {
            sleep( 10 ); //wait 10 seconds for new wind measurement
            continue;
        }
        else
        {
            break;
        }
    }

    //Calculate wind directions

    *winddir_index = (data[ 2 ] >> 4 );
    winddir[ 0 ] = (data[ 2 ] >> 4 ) * 22.5;
    winddir[ 1 ] = (data[ 3 ] &  0xF ) * 22.5;
    winddir[ 2 ] = (data[ 3 ] >> 4 ) * 22.5;
    winddir[ 3 ] = (data[ 4 ] &  0xF ) * 22.5;
    winddir[ 4 ] = (data[ 4 ] >> 4 ) * 22.5;
    winddir[ 5 ] = (data[ 5 ] &  0xF ) * 22.5;

    //Calculate raw wind speed - convert from m/s to whatever
    return ( ((( data[ 2 ] & 0xF ) << 8 ) + ( data[ 1 ]) ) / 10.0 *  windSpeedUnit );
}


/**
 * Read min/max wind speeds with timestamps
 *
 * @param windSpeedUnit controlling convertion to other units than m/s
 *
 * @param wind_min ( OUT Parameter ) Wind minimum
 * @param wind_max ( OUT Parameter ) Wind maximum
 *                unit defined by config conversion factor
 * @param timeMin ( OUT Parameter ) Timestamps for wind_min
 * @param timeMax ( OUT Parameter ) Timestamps for wind_max
 *
 * @return wind max (double)
 *
 * @note The function is made so that if a pointer to
 *       wind_min/max and timeMin/max is a NULL pointer the function
 *       ignores this parameter. Example: if you only need wind_max
 *       use the function like this..
 *       windmax = wind_minmax(ws2300,METERS_PER_SECOND,NULL,NULL,NULL,NULL);
 *
 */
double Ws2355::windMinMax( double  windSpeedUnit,
                           double *wind_min,
                           double *wind_max,
                           QDateTime *timeMin,
                           QDateTime *timeMax)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( WIND_MINIMUM, 15, data, command) != 15 )
        readErrorExit();

    if( wind_min != NULL )
        *wind_min = (data[ 1 ] * 256 + data[ 0 ]) / 360.0 *  windSpeedUnit;
    if( wind_max != NULL )
        *wind_max = (data[ 4 ] * 256 + data[ 3 ]) / 360.0 *  windSpeedUnit;

    if( timeMin != NULL )
    {
        QTime t(((data[ 6 ] >> 4 ) * 10 ) + ( data[ 6 ] & 0xF ),
                ((data[ 5 ] >> 4 ) * 10 ) + ( data[ 5 ] & 0xF ));
        timeMin->setTime( t );

        QDate d(2000 + ((data[ 9 ] >> 4 ) * 10 ) + ( data[ 9 ] & 0xF ),
                       ((data[ 8 ] >> 4 ) * 10 ) + ( data[ 8 ] & 0xF ),
                       ((data[ 7 ] >> 4 ) * 10 ) + ( data[ 7 ] & 0xF ));
        timeMin->setDate( d );
    }

    if( timeMax != NULL )
    {
        QTime t((( data[ 11 ] >> 4 ) * 10 ) + ( data[ 11 ] & 0xF ),
                (( data[ 10 ] >> 4 ) * 10 ) + ( data[ 10 ] & 0xF ));
        timeMax->setTime( t );

        QDate d(2000 + (( data[ 14 ] >> 4 ) * 10 ) + ( data[ 14 ] & 0xF ),
                       (( data[ 13 ] >> 4 ) * 10 ) + ( data[ 13 ] & 0xF ),
                       (( data[ 12 ] >> 4 ) * 10 ) + ( data[ 12 ] & 0xF ));
        timeMax->setDate( d );
    }

    return (( data[ 4 ] * 256 + data[ 3 ]) / 360.0 *  windSpeedUnit );
}

/**
 * Reset min/max wind with timestamps
 *
 * @param minmax controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 *
 */
int Ws2355::windReset( char minmax )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also
    int i;
    int current_wind;

    for( i = 0; i < MAXWINDRETRIES; i ++ )
    {
        if( readSafe( WIND_SPEED_AND_DIRECTION, 3, data_read, command ) != 3 )
            readErrorExit();

        if(( data_read[ 0 ] != 0x00 ) ||                            //Invalid wind data
          (( data_read[ 1 ] == 0xFF ) && ((( data_read[ 2 ] & 0xF ) == 0 ) || (( data_read[ 2 ] & 0xF ) == 1 ))))
        {
            sleep( 10 ); //wait 10 seconds for new wind measurement
            continue;
        }
        else
        {
            break;
        }
    }
    current_wind = ((( data_read[ 2 ] & 0xF ) << 8 ) + ( data_read[ 1 ])) * 36;

    data_value[ 0 ] =  current_wind          & 0xF;
    data_value[ 1 ] = (current_wind >>  4 )  & 0xF;
    data_value[ 2 ] = (current_wind >>  8 )  & 0xF;
    data_value[ 3 ] = (current_wind >>  12 ) & 0xF;

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    if( minmax & RESET_MIN ) // minimum
    {
        // Set min value to current value
        if( writeSafe( WIND_MINIMUM, 4, WRITENIB, data_value, command ) != 4 )
            writeErrorExit();

        // Set min value timestamp to current time
        if( writeSafe( TIME_MINIMUM_WIND, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    if( minmax & RESET_MAX ) // maximum
    {
        // Set max value to current value
        if( writeSafe( WIND_MAXIMUM, 4, WRITENIB, data_value, command ) != 4 )
            writeErrorExit();

        // Set max value timestamp to current time
        if( writeSafe( TIME_MAXIMUM_WIND, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    return 1;
}


/**
 * Read wind chill, current value only
 *
 * @param temperatureUnit  to control convertion to deg F
 *
 * @return wind chill  (deg C if config.temperatureUnit is not set)
 *                     (deg F if config.temperatureUnit is set)
 *
 * It is recommended to run this right after a wind speed reading
 * to enhance the likelyhood that the wind speed is valid
 *
 */
double Ws2355::readWindchill( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( CURRENT_WINDCHILL, 2, data, command ) != 2 )
        readErrorExit();

    if( temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) -
                  30.0 ) * 9 / 5 + 32 );
    else
        return ((((data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  (data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0) - 30.0 ));
}

/**
 * Read windchill min alarm
 * *
 * @return bool : windchill min alarm
 */
bool Ws2355::readWindchillMinAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( WINDCHILL_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & WINDCHILL_MIN_ALARM_BIT ) == WINDCHILL_MIN_ALARM_BIT );
}

/**
 * Read Windchill min temperature of the alarm
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readWindchillMin( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( LOW_ALARM_WINDCHILL, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}


/**
 * Read windchill max alarm
 * *
 * @return bool : windchill miax alarm
 */
bool Ws2355::readWindchillMaxAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( WINDCHILL_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & WINDCHILL_MAX_ALARM_BIT ) == WINDCHILL_MAX_ALARM_BIT );
}

/**
 * Read Windchill max temperature of the alarm
 *
 * @param temperatureUnit controlling convertion to deg F
 *
 * @return Temperature ( deg C if temperatureUnit is CELCIUS )
 *                     ( deg F if temperatureUnit is FAHRENHEIT )
 *
 */
double Ws2355::readWindchillMax( int temperatureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( HIGH_ALARM_WINDCHILL, 2, data, command ) != 2 )
        readErrorExit();

    if(  temperatureUnit == FAHRENHEIT )
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0) * 9 / 5 + 32 );
    else
        return (((( data[ 1 ] >> 4 ) * 10   + ( data[ 1 ] & 0xF ) +
                  ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) - 30.0 ));
}


/**
 * Read wind chill min/max with timestamps
 *
 * @param temperatureUnit to control convertion to deg F
 *
 * @param wc_min ( OUT Parameter ) Windchill minimum
 * @param wc_max ( OUT Parameter ) Windchill maximum
 *                (deg C if config.temperatureUnit is not set)
 *                (deg F if config.temperatureUnit is set)
 * @param timeMin ( OUT Parameter ) Timestamps for wc_min
 * @param timeMax ( OUT Parameter ) Timestamps for wc_max
 */
void Ws2355::windchillMinMax( int temperatureUnit,
                              double *wc_min,
                              double *wc_max,
                              QDateTime *timeMin,
                              QDateTime *timeMax)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( MINIMUM_WINDCHILL, 15, data, command ) != 15 )
        readErrorExit();

    *wc_min = (( data[ 1 ] >> 4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
               ( data[ 0 ] &  0xF ) / 100.0 ) - 30.0;

    *wc_max = (( data[ 4 ] & 0xF ) * 10 + ( data[ 3 ] >> 4 ) + ( data[ 3 ] & 0xF ) / 10.0 +
               ( data[ 2 ] >> 4 ) / 100.0 ) - 30.0;

    if( temperatureUnit == FAHRENHEIT )
    {
            *wc_min = *wc_min * 9 / 5 + 32;
            *wc_max = *wc_max * 9 / 5 + 32;
    }

    QTime t((( data[ 6 ] & 0xF ) * 10 ) + ( data[ 5 ] >> 4 ),
            (( data[ 5 ] & 0xF ) * 10 ) + ( data[ 4 ] >> 4 ));
    timeMin->setTime( t );

    QDate d( 2000 + (( data[ 9 ] & 0xF ) * 10 ) + ( data[ 8 ] >> 4 ),
                    (( data[ 8 ] & 0xF ) * 10 ) + ( data[ 7 ] >> 4 ),
                    (( data[ 7 ] & 0xF ) * 10 ) + ( data[ 6 ] >> 4 ));
    timeMin->setDate( d );

    QTime t1((( data[ 11 ] & 0xF ) * 10 ) + ( data[ 10 ] >> 4 ),
             (( data[ 10 ] & 0xF ) * 10 ) + ( data[ 9  ] >> 4 ));
    timeMax->setTime( t1 );

    QDate d1( 2000 + (( data[ 14 ] & 0xF ) * 10 ) + ( data[ 13 ] >> 4 ),
                     (( data[ 13 ] & 0xF ) * 10 ) + ( data[ 12 ] >> 4 ),
                     (( data[ 12 ] & 0xF ) * 10 ) + ( data[ 11 ] >> 4 ));
    timeMax->setDate( d1 );

    return;
}


/**
 * Reset min/max windchill with timestamps
 *
 * @param minmax controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 *
 */
int Ws2355::windchillReset( char minmax )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    // First read current windchill into data_value
    if( readSafe( CURRENT_WINDCHILL, 2, data_read, command ) != 2 )
        readErrorExit();

    data_value[ 0 ] = data_read[ 0 ] & 0xF;
    data_value[ 1 ] = data_read[ 0 ] >> 4;
    data_value[ 2 ] = data_read[ 1 ] &  0xF;
    data_value[ 3 ] = data_read[ 1 ] >> 4;

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    if( minmax & RESET_MIN ) // minimum
    {
        // Set min value to current value
        if( writeSafe( MINIMUM_WINDCHILL, 4, WRITENIB, data_value, command ) != 4 )
            writeErrorExit();

        // Set min value timestamp to current time
        if( writeSafe( TIME_MINIMUM_WINDCHILL, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    if( minmax & RESET_MAX ) // maximum
    {
        // Set max value to current value
        if( writeSafe( MAXIMUM_WINDCHILL, 4, WRITENIB, data_value, command ) != 4 )
            writeErrorExit();

        // Set max value timestamp to current time
        if( writeSafe( TIME_MAXIMUM_WINDCHILL, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    return 1;
}


/**
 * Read rain last 1 hour, current value only
 *
 * @param rainUnit controlling convertion to other units than mm
 *
 * @return rain converted to unit given in config
 */
double Ws2355::readRain1h( double rainUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( RAIN_1_HOUR, 3, data, command) != 3 )
            readErrorExit();

    return((( data[ 2 ] >>  4 ) * 1000 + ( data[ 2 ] & 0xF ) * 100 +
            ( data[ 1 ] >>  4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
            ( data[ 0 ] & 0xF ) / 100.0 ) / rainUnit );
}


/**
 * Read rain 1h alarm
 * *
 * @return bool : rain 1h alarm
 */
bool Ws2355::readRain1hAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( RAIN_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & RAIN_1H_ALARM_BIT ) == RAIN_1H_ALARM_BIT );
}

/**
 * Read rain last 1 hour of the alarm
 *
 * @param rain Unit controlling convertion to other units than mm
 *
 * @return rain converted to unit given in config
 */
double Ws2355::readRain1hMax( double rainUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( ALARM_RAIN_1_HOUR, 3, data, command) != 3 )
            readErrorExit();

    return((( data[ 2 ] >>  4 ) * 1000 + ( data[ 2 ] & 0xF ) * 100 +
            ( data[ 1 ] >>  4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
            ( data[ 0 ] & 0xF ) / 100.0 ) / rainUnit );
}

/**
 * Read rain last 1 hour and maximum with timestamp
 *
 * @param rainUnit controlling convertion to other units than mm
 *
 * Output: Rain maximum in rain_max (double)
 *                unit defined by config conversion factor
 *         Timestamps for rain_max in pointers to
 *                timestamp structures for timeMin and timeMax
 *
 * Returns: rain (double) converted to unit given in config
 *
 */
double Ws2355::rain1hAll( double rainUnit,
                          double *rain_max,
                          QDateTime *timeMax )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	// Buffer for write data also

    if( readSafe( RAIN_1_HOUR, 11, data, command) != 11 )
        readErrorExit();

    *rain_max = (( data[ 5 ] >> 4 ) * 1000 + ( data[ 5 ] & 0xF ) * 100 +
                 ( data[ 4 ] >> 4 ) * 10 + ( data[ 4 ] & 0xF ) + ( data[ 3 ] >> 4 ) / 10.0 +
                 ( data[ 3 ] & 0xF ) / 100.0) / rainUnit;

    QTime t((( data[ 7 ] >> 4 ) * 10 ) + (data[ 7 ] & 0xF ),
            (( data[ 6 ] >> 4 ) * 10 ) + (data[ 6 ] & 0xF ));
    timeMax->setTime( t );

    QDate d(2000 + (( data[ 10 ] >> 4 ) * 10 ) + ( data[ 10 ] & 0xF ),
                   (( data[ 9  ] >> 4 ) * 10 ) + ( data[ 9  ] & 0xF ),
                   (( data[ 8  ] >> 4 ) * 10 ) + ( data[ 8  ] & 0xF ));
    timeMax->setDate( d );

    return ((( data[ 2 ] >>  4 ) * 1000 + ( data[ 2 ] & 0xF ) * 100 +
             ( data[ 1 ] >>  4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
             ( data[ 0 ] & 0xF ) / 100.0) / rainUnit );
}


/**
 * Reset max rain 1h with timestamps
 *
 * @param minmax controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 */
int Ws2355::rain1hMaxReset( void )
{
        unsigned char data_read[ 20 ];
        unsigned char data_value[ 20 ];
        unsigned char data_time[ 20 ];
        unsigned char command[ 25 ];	//room for write data also

        // First read current rain 1h into data_value
        if( readSafe( RAIN_1_HOUR, 3, data_read, command ) != 3 )
            readErrorExit();

        data_value[ 0 ] = data_read[ 0 ] &  0xF;
        data_value[ 1 ] = data_read[ 0 ] >> 4;
        data_value[ 2 ] = data_read[ 1 ] &  0xF;
        data_value[ 3 ] = data_read[ 1 ] >> 4;
        data_value[ 4 ] = data_read[ 2 ] &  0xF;
        data_value[ 5 ] = data_read[ 2 ] >> 4;

        // Get current time from station
        if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
            readErrorExit();

        data_time[ 0 ] = data_read[ 0 ] &  0xF;
        data_time[ 1 ] = data_read[ 0 ] >> 4;
        data_time[ 2 ] = data_read[ 1 ] &  0xF;
        data_time[ 3 ] = data_read[ 1 ] >> 4;
        data_time[ 4 ] = data_read[ 2 ] >> 4;
        data_time[ 5 ] = data_read[ 3 ] &  0xF;
        data_time[ 6 ] = data_read[ 3 ] >> 4;
        data_time[ 7 ] = data_read[ 4 ] &  0xF;
        data_time[ 8 ] = data_read[ 4 ] >> 4;
        data_time[ 9 ] = data_read[ 5 ] &  0xF;

        // Set max value to current value
        if( writeSafe( RAIN_MAXIMUM_1_HOUR, 6, WRITENIB, data_value, command ) != 6 )
            writeErrorExit();

        // Set max value timestamp to current time
        if( writeSafe( TIME_MAXIMUM_1_HOUR, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();

        return 1;
}

/**
 * Reset current rain 1h
 *
 * @param minmax controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 *
 */
int Ws2355::rain1hReset( void )
{
        unsigned char data[ 50 ];
        unsigned char command[ 60 ];	//room for write data also

        // First overwrite the 1h rain history with zeros
        memset( &data, 0, sizeof( data ));

        if( writeSafe( LAST_MINUTES_RAINCOUNT, 30, WRITENIB, data, command ) != 30 )
            writeErrorExit();

        // Set value to zero
        if( writeSafe( RAIN_1_HOUR, 6, WRITENIB, data, command ) != 6 )
            writeErrorExit();

        return 1;
}

/**
 * Read rain last 24 hours, current value only
 *
 * @param rainUnit controlling convertion to other units than mm
 *
 * @return rain (double) converted to unit given in config
 */
double Ws2355::readRain24h( double rainUnit)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( RAIN_24_HOURS, 3, data, command ) != 3 )
        readErrorExit();

    return (((data[ 2 ] >> 4 ) * 1000 + ( data[ 2 ] & 0xF ) * 100 +
             (data[ 1 ] >> 4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
             (data[ 0 ] & 0xF ) / 100.0) / rainUnit );
}


/**
 * Read rain 24h alarm
 * *
 * @return bool : rain 24h alarm
 */
bool Ws2355::readRain24hAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( RAIN_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & RAIN_24H_ALARM_BIT ) == RAIN_24H_ALARM_BIT );
}

/**
 * Read rain last 24 hour of the alarm
 *
 * @param rain Unit controlling convertion to other units than mm
 *
 * @return rain converted to unit given in config
 */
double Ws2355::readRain24hMax( double rainUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( ALARM_RAIN_24_HOURS, 3, data, command) != 3 )
            readErrorExit();

    return((( data[ 2 ] >>  4 ) * 1000 + ( data[ 2 ] & 0xF ) * 100 +
            ( data[ 1 ] >>  4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
            ( data[ 0 ] & 0xF ) / 100.0 ) / rainUnit );
}

/**
 * Read rain last 24 hours and maximum with timestamp
 *
 * @param rainUnit ( IN Parameter ) controlling convertion to other units than mm
 *
 * @param rain_max ( OUT Parameter ) maximum in rain_max (double) unit defined by config conversion factor
 * @param timeMax ( OUT Parameter ) timestamp structures for rain_max
 *
 * @return rain (double) converted to unit given in config
 *
 */
double Ws2355::rain24hAll( double rainUnit, double *rain_max, QDateTime *timeMax )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( RAIN_24_HOURS, 11, data, command ) != 11 )
        readErrorExit();

    *rain_max = (( data[ 5 ] >> 4 ) * 1000 + ( data[ 5 ] & 0xF ) * 100 +
                 ( data[ 4 ] >> 4 ) * 10 + ( data[ 4 ] & 0xF ) + ( data[ 3 ] >> 4 ) / 10.0 +
                 ( data[ 3 ] & 0xF ) / 100.0) / rainUnit;

    QTime t((( data[ 7  ] >> 4 ) * 10 ) + ( data[ 7  ] & 0xF ),
            (( data[ 6  ] >> 4 ) * 10 ) + ( data[ 6  ] & 0xF ));
    timeMax->setTime( t );

    QDate d(2000 + (( data[ 10 ] >> 4 ) * 10 ) + ( data[ 10 ] & 0xF ),
            (( data[ 9  ] >> 4 ) * 10 ) + ( data[ 9  ] & 0xF ),
            (( data[ 8  ] >> 4 ) * 10 ) + ( data[ 8  ] & 0xF ));
    timeMax->setDate( d );

    return ((( data[ 2 ] >>  4 ) * 1000 + ( data[ 2 ] & 0xF ) * 100 +
             ( data[ 1 ] >>  4 ) * 10 + ( data[ 1 ] & 0xF ) + ( data[ 0 ] >> 4 ) / 10.0 +
             ( data[ 0 ] & 0xF ) / 100.0 ) / rainUnit );
}


/**
 * Reset max rain 24h with timestamps
 *
 * @return 1 if success
 */
int Ws2355::rain24hMaxReset( void )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also
    int address;
    int number;

    // First read current rain 24h into data_value
    if( readSafe( 0x497, 3, data_read, command ) != 3 )
        readErrorExit();

    data_value[ 0 ] = data_read[ 0 ] &  0xF;
    data_value[ 1 ] = data_read[ 0 ] >> 4;
    data_value[ 2 ] = data_read[ 1 ] &  0xF;
    data_value[ 3 ] = data_read[ 1 ] >> 4;
    data_value[ 4 ] = data_read[ 2 ] &  0xF;
    data_value[ 5 ] = data_read[ 2 ] >> 4;

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    // Set max value to current value
    address = RAIN_MAXIMUM_24_HOURS;
    number = 6;

    if( writeSafe( address, number, WRITENIB, data_value, command ) != number )
        writeErrorExit();

    // Set max value timestamp to current time
    address = TIME_MAXIMUM_24_HOURS;
    number = 10;

    if( writeSafe( address, number, WRITENIB, data_time, command ) != number )
        writeErrorExit();

    return 1;
}


/**
 * Reset current rain 24h
 *
 * @param minmax controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 */
int Ws2355::rain24hReset( void )
{
    unsigned char data[ 50 ];
    unsigned char command[ 60 ];	//room for write data also

    // First overwrite the 24h rain history with zeros
    memset( &data, 0, sizeof( data ));

    if( writeSafe( LAST_HOUR_RAINCOUNT, 48, WRITENIB, data, command ) != 48 )
        writeErrorExit();

    // Set value to zero
    if( writeSafe( RAIN_24_HOURS, 6, WRITENIB, data, command ) != 6 )
        writeErrorExit();

    return 1;
}


/**
 * Read rain accumulated total, current value only
 *
 * @param rainUnit controlling convertion to other units than mm
 *
 * @return rain (double) converted to unit given in config
 *
 */
double Ws2355::readRainTotal( double rainUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if( readSafe( TOTAL_RAIN, 3, data, command ) != 3 )
        readErrorExit();

    return ((( data[ 2 ] >> 4 ) * 1000 + ( data[ 2 ] & 0xF ) * 100 +
             ( data[ 1 ] >> 4 ) * 10 + ( data[ 1 ] & 0xF ) +
             ( data[ 0 ] >> 4 ) / 10.0 + ( data[ 0 ] & 0xF ) / 100.0 ) /
             rainUnit );
}


/**
 * Read rain total accumulated with timestamp
 *
 * @param rainUnit controlling convertion to other units than mm
 *
 * @param time_since ( OUT Parameter ) Timestamp for rain total
 *
 * @return rain (double) converted to unit given in config
 */
double Ws2355::rainTotalAll( double rainUnit, QDateTime *time_since)
{
        unsigned char data[ 20 ];
        unsigned char command[ 25 ];	//room for write data also

        if( readSafe( TOTAL_RAIN, 8, data, command ) != 8 )
                readErrorExit();

        QTime t((( data[ 4 ] >> 4) * 10) + (data[ 4 ] & 0xF),
                (( data[ 3 ] >> 4) * 10) + (data[ 3 ] & 0xF));
        time_since->setTime( t );

        QDate d(2000 + ((data[ 7 ] >> 4 ) * 10 ) + ( data[ 7 ] & 0xF ),
                       ((data[ 6 ] >> 4 ) * 10 ) + ( data[ 6 ] & 0xF ),
                       ((data[ 5 ] >> 4 ) * 10 ) + ( data[ 5 ] & 0xF ));
        time_since->setDate( d );

        return (((data[ 2 ] >> 4) * 1000 + (data[ 2 ] & 0xF) * 100 +
                 (data[ 1 ] >> 4) * 10   + (data[ 1 ] & 0xF) +
                 (data[ 0 ] >> 4) / 10.0 + (data[ 0 ] & 0xF) / 100.0) /
                 rainUnit);
}


/**
 * Reset current total rain
 *
 * @param minmax  controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 */
int Ws2355::rainTotalReset( void )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    // Set value to zero
    memset( &data_value, 0, sizeof( data_value ));

    if( writeSafe( TOTAL_RAIN, 6, WRITENIB, data_value, command ) != 6 )
        writeErrorExit();

    // Set max value timestamp to current time
    if( writeSafe( TIME_TOTAL_RAIN, 10, WRITENIB, data_time, command ) != 10 )
        writeErrorExit();

    return 1;
}


/**
 * Read relative air pressure, current value only
 *
 * @param pressureUnit controlling convertion to other units than hPa
 *
 * @return pressure (double) converted to unit given in config
 */
double Ws2355::readRelPressure( double pressureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( RELATIVE_AIR_PRESSURE_HPA, 3, data, command ) != 3 )
            readErrorExit();

    return ((( data[ 2 ] & 0xF ) * 1000 + ( data[ 1 ] >> 4 ) * 100 +
             ( data[ 1 ] & 0xF ) * 10 + ( data[ 0 ] >> 4 ) +
             ( data[ 0 ] & 0xF ) / 10.0) / pressureUnit );
}

/**
 * Read pressure min alarm
 * *
 * @return bool : pressure min alarm
 */
bool Ws2355::readPressureMinAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( PRESSURE_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & PRESSURE_MIN_ALARM_BIT ) == PRESSURE_MIN_ALARM_BIT );
}

/**
 * Read relative min air pressure of the alarm
 *
 * @param pressureUnit controlling convertion to other units than hPa
 *
 * @return pressure (double) converted to unit given in config
 */
double Ws2355::readPressureMin( double pressureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( LOW_ALARM_AIR_PRESSURE_HPA, 3, data, command ) != 3 )
            readErrorExit();

    return ((( data[ 2 ] & 0xF ) * 1000 + ( data[ 1 ] >> 4 ) * 100 +
             ( data[ 1 ] & 0xF ) * 10 + ( data[ 0 ] >> 4 ) +
             ( data[ 0 ] & 0xF ) / 10.0) / pressureUnit );
}

/**
 * Read pressure max alarm
 * *
 * @return bool : pressure miax alarm
 */
bool Ws2355::readPressureMaxAlarm( void )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    if(  readSafe( PRESSURE_ALARM , 1, data, command ) != 1 )
         readErrorExit();

    return (( data[ 0 ] & PRESSURE_MAX_ALARM_BIT ) == PRESSURE_MAX_ALARM_BIT );
}

/**
 * Read relative max air pressure of the alarm
 *
 * @param pressureUnit controlling convertion to other units than hPa
 *
 * @return pressure (double) converted to unit given in config
 */
double Ws2355::readPressureMax( double pressureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( HIGH_ALARM_AIR_PRESSURE_HPA, 3, data, command ) != 3 )
            readErrorExit();

    return ((( data[ 2 ] & 0xF ) * 1000 + ( data[ 1 ] >> 4 ) * 100 +
             ( data[ 1 ] & 0xF ) * 10 + ( data[ 0 ] >> 4 ) +
             ( data[ 0 ] & 0xF ) / 10.0) / pressureUnit );
}

/**
 * Read relative pressure min/max with timestamps
 *
 * @param pressureUnit controlling convertion to other units than hPa
 *
 * @param pres_min ( OUT Parameter ) Pressure
 * @param pres_max ( OUT Parameter ) Pressure
 *                unit defined by config conversion factor
 * @param timeMin ( OUT Parameter ) Timestamps for pres_min
 * @param timeMax ( OUT Parameter ) Timestamps for pres_max
 */
void Ws2355::relPressureMinMax( double pressureUnit,
                                  double *pres_min,
                                  double *pres_max,
                                  QDateTime *timeMin,
                                  QDateTime *timeMax)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( RELATIVE_AIR_PRESSURE_MINIMUM_HPA, 13, data, command ) != 13 )
        readErrorExit();

    *pres_min = (( data[ 2 ] & 0xF ) * 1000 + ( data[ 1 ] >> 4 ) * 100 +
                 ( data[ 1 ] & 0xF ) * 10 + ( data[ 0 ] >> 4 ) +
                 ( data[ 0 ] & 0xF ) / 10.0) / pressureUnit;

    *pres_max = ((data[12]&0xF)*1000 + (data[11]>>4)*100 +
                (data[11]&0xF)*10 + (data[10]>>4) +
                (data[10]&0xF)/10.0) / pressureUnit;

    //Relative pressure time and date for min/max
    if( readSafe( TIME_MINIMUM_AIR_PRESSURE, 10, data, command ) != 10 )
        readErrorExit();

    QTime t((( data[ 1 ] >> 4 ) * 10 ) + ( data[ 1 ] & 0xF ),
            (( data[ 0 ] >> 4 ) * 10 ) + ( data[ 0 ] & 0xF ));
    timeMin->setTime( t );

    QDate d( 2000 + (( data[ 4 ] >> 4 ) * 10 ) + ( data[ 4 ] & 0xF ),
                    (( data[ 3 ] >> 4 ) * 10 ) + ( data[ 3 ] & 0xF ),
                    (( data[ 2 ] >> 4 ) * 10 ) + ( data[ 2 ] & 0xF ));
    timeMin->setDate( d );

    QTime t1((( data[ 6 ] >> 4 ) * 10 ) + ( data[ 6 ] & 0xF ),
             (( data[ 5 ] >> 4 ) * 10 ) + ( data[ 5 ] & 0xF ));
    timeMax->setTime( t1 );

    QDate d1( 2000 + (( data[ 9 ] >> 4 ) * 10 ) + ( data[ 9 ] & 0xF ),
                     (( data[ 8 ] >> 4 ) * 10 ) + ( data[ 8 ] & 0xF ),
                     (( data[ 7 ] >> 4 ) * 10 ) + ( data[ 7 ] & 0xF ));
    timeMax->setDate( d1 );

    return;
}


/**
 * Read absolute air pressure, current value only
 *
 * @param pressureUnit controlling convertion to other units than hPa
 *
 * @return pressure (double) converted to unit given in config
 */
double Ws2355::readAbsPressure( double pressureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( ABSOLUTE_AIR_PRESSURE_HPA, 3, data, command ) != 3 )
        readErrorExit();

    return ((( data[ 2 ] & 0xF ) * 1000 + ( data[ 1 ] >> 4 ) * 100 +
             ( data[ 1 ] & 0xF ) * 10 + ( data[ 0 ] >> 4 ) +
             ( data[ 0 ] & 0xF ) / 10.0) / pressureUnit );
}


/**
 * Read absolute pressure min/max with timestamps
 *
 * @param pressureUnit controlling convertion to other units than hPa
 *
 * @param pres_min ( OUT Parameter ) Pressure
 * @param pres_max ( OUT Parameter ) Pressure
 *                unit defined by config conversion factor
 * @param timeMin ( OUT Parameter ) Timestamps for pres_min
 * @param timeMax ( OUT Parameter ) Timestamps for pres_max
 */
void Ws2355::absPressureMinMax( double pressureUnit,
                                  double *pres_min,
                                  double *pres_max,
                                  QDateTime *timeMin,
                                  QDateTime *timeMax)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( ABSOLUTE_AIR_PRESSURE_MINIMUM_HPA, 13, data, command ) != 13 )
        readErrorExit();

    *pres_min = (( data[ 2 ] & 0xF ) * 1000 + ( data[ 1 ] >> 4 ) * 100 +
                 ( data[ 1 ] & 0xF ) * 10 + ( data[ 0 ] >> 4) +
                 ( data[ 0 ] & 0xF ) / 10.0 ) / pressureUnit;

    *pres_max = (( data[ 12 ] & 0xF ) * 1000 + ( data[ 11 ] >> 4 ) * 100 +
                 ( data[ 11 ] & 0xF ) * 10 + ( data[ 10 ] >> 4 ) +
                 ( data[ 10 ] & 0xF ) / 10.0) / pressureUnit;

    //Relative pressure time and date for min/max
    if( readSafe( TIME_MINIMUM_AIR_PRESSURE, 10, data, command ) != 10 )
        readErrorExit();

    QTime t((( data[ 1 ] >> 4 ) * 10 ) + ( data[ 1 ] & 0xF ),
            (( data[ 0 ] >> 4 ) * 10 ) + ( data[ 0 ] & 0xF ));
    timeMin->setTime( t );

    QDate d( 2000 + (( data[ 4 ] >> 4 ) * 10 ) + ( data[ 4 ] & 0xF ),
                    (( data[ 3 ] >> 4 ) * 10 ) + ( data[ 3 ] & 0xF ),
                    (( data[ 2 ] >> 4 ) * 10 ) + ( data[ 2 ] & 0xF ));
    timeMin->setDate( d );

    QTime t1((( data[ 6 ] >> 4) * 10 ) + (data[ 6 ] & 0xF ),
             (( data[ 5 ] >> 4) * 10 ) + (data[ 5 ] & 0xF ));
    timeMax->setTime( t1 );

    QDate d1( 2000 + (( data[ 9 ] >> 4 ) * 10 ) + ( data[ 9 ] & 0xF ),
                     (( data[ 8 ] >> 4 ) * 10 ) + ( data[ 8 ] & 0xF ),
                     (( data[ 7 ] >> 4 ) * 10 ) + ( data[ 7 ] & 0xF ));
    timeMax->setDate( d1 );

    return;
}



/**
 * Reset min/max pressure (relative and absolute) with timestamps
 *
 * @param minmax controls if minimum, maximum or both are reset
 *
 * @return 1 if success
 */
int Ws2355::pressureReset( char minmax )
{
    unsigned char data_read[ 20 ];
    unsigned char data_value_abs[ 20 ];
    unsigned char data_value_rel[ 20 ];
    unsigned char data_time[ 20 ];
    unsigned char command[ 25 ];	//room for write data also

    // First read current abs/rel pressure into data_value_abs/rel
    if( readSafe( ABSOLUTE_AIR_PRESSURE_HPA, 8, data_read, command ) != 8 )
        readErrorExit();

    data_value_abs[ 0 ] = data_read[ 0 ] &  0xF;
    data_value_abs[ 1 ] = data_read[ 0 ] >> 4;
    data_value_abs[ 2 ] = data_read[ 1 ] &  0xF;
    data_value_abs[ 3 ] = data_read[ 1 ] >> 4;
    data_value_abs[ 4 ] = data_read[ 2 ] &  0xF;

    data_value_rel[ 0 ] = data_read[ 5 ] &  0xF;
    data_value_rel[ 1 ] = data_read[ 5 ] >> 4;
    data_value_rel[ 2 ] = data_read[ 6 ] &  0xF;
    data_value_rel[ 3 ] = data_read[ 6 ] >> 4;
    data_value_rel[ 4 ] = data_read[ 7 ] &  0xF;

    // Get current time from station
    if( readSafe( LOCAL_CURRENT_TIME, 6, data_read, command ) != 6 )
        readErrorExit();

    data_time[ 0 ] = data_read[ 0 ] &  0xF;
    data_time[ 1 ] = data_read[ 0 ] >> 4;
    data_time[ 2 ] = data_read[ 1 ] &  0xF;
    data_time[ 3 ] = data_read[ 1 ] >> 4;
    data_time[ 4 ] = data_read[ 2 ] >> 4;
    data_time[ 5 ] = data_read[ 3 ] &  0xF;
    data_time[ 6 ] = data_read[ 3 ] >> 4;
    data_time[ 7 ] = data_read[ 4 ] &  0xF;
    data_time[ 8 ] = data_read[ 4 ] >> 4;
    data_time[ 9 ] = data_read[ 5 ] &  0xF;

    if( minmax & RESET_MIN ) // minimum
    {
        // Set min abs value to current abs value
        if( writeSafe( ABSOLUTE_AIR_PRESSURE_MINIMUM_HPA, 5, WRITENIB, data_value_abs, command ) != 5 )
            writeErrorExit();

        // Set min rel value to current rel value
        if( writeSafe( RELATIVE_AIR_PRESSURE_MINIMUM_HPA, 5, WRITENIB, data_value_rel, command ) != 5 )
            writeErrorExit();

        // Set min value timestamp to current time
        if( writeSafe( TIME_MINIMUM_AIR_PRESSURE, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    if( minmax & RESET_MAX ) // maximum
    {
        // Set max abs value to current abs value
        if( writeSafe( ABSOLUTE_AIR_PRESSURE_MAXIMUM_HPA, 5, WRITENIB, data_value_abs, command ) != 5 )
            writeErrorExit();

        // Set max rel value to current rel value
        if( writeSafe( RELATIVE_AIR_PRESSURE_MAXIMUM_HPA, 5, WRITENIB, data_value_rel, command ) != 5 )
            writeErrorExit();

        // Set max value timestamp to current time
        if( writeSafe( TIME_MAXIMUM_AIR_PRESSURE, 10, WRITENIB, data_time, command ) != 10 )
            writeErrorExit();
    }

    return 1;
}


/**
 * Read the correction from absolute to relaive air pressure
 *
 * @param pressureUnit controlling convertion to other units than hPa
 *
 * @return pressure (double) converted to unit given in conv factor
 */
double Ws2355::pressureCorrection( double pressureUnit )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( AIR_PRESSURE_CORRECTION_HPA, 3, data, command ) != 3 )
        readErrorExit();


    return (( data[ 2 ] & 0xF ) * 1000 +
            ( data[ 1 ] >> 4 ) * 100 +
            ( data[ 1 ] & 0xF ) * 10 +
            ( data[ 0 ] >> 4 ) +
            ( data[ 0 ] & 0xF ) / 10.0 -
            1000
           ) / pressureUnit;
}


/**
 * Read Tendency and Forecast
 *
 * @param tendency ( OUT Parameter ) 0:Steady, 1:Rising or 2:Falling
 *        forecast ( OUT Parameter ) 0:Rainy, 1:Cloudy or 2:Sunny
 */
void Ws2355::readTendencyForecast( unsigned char *tendency, unsigned char *forecast )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( FORECAST, 1, data, command ) != 1 )
        readErrorExit();

    *tendency = data[ 0 ] >> 4;
    *forecast = data[ 0 ] & 0xF;

    return;
}


/**
 * Read the history information like interval, countdown, time
 * of last record, pointer to last record.
 *
 * @param interval (OUT Parameter ) Current interval in minutes (integer)
 * @param countdown (OUT Parameter ) Countdown to next measurement (integer)
 * @param timelast (OUT Parameter ) Time/Date for last measurement (timestamp struct)
 * @param nbOfValidRecords (OUT Parameter ) number of valid records (integer)
 *
 * @return integer pointing to last written record. [0x00-0xAE]
 *
 */
int Ws2355::readHistoryInfo( int *interval,
                             int *countdown,
                             QDateTime *timeLast,
                             int *nbOfValidRecords )
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];

    if( readSafe( HISTORY_SAVING_INTERVAL, 10, data, command) != 10 )
        readErrorExit();

    *interval = (data[ 1 ] & 0xF ) * 256 + data[ 0 ] + 1;
    *countdown = data[ 2 ] * 16 + ( data[ 1 ] >> 4 ) + 1;

    QTime t((( data[ 4 ] >> 4) * 10) + (data[ 4 ] & 0xF ),
            (( data[ 3 ] >> 4) * 10) + (data[ 3 ] & 0xF ));
    timeLast->setTime( t );

    QDate d( 2000 + (( data[ 7 ] >> 4 ) * 10 ) + ( data[ 7 ] & 0xF ),
                    (( data[ 6 ] >> 4 ) * 10 ) + ( data[ 6 ] & 0xF ),
                    (( data[ 5 ] >> 4 ) * 10 ) + ( data[ 5 ] & 0xF ));
    timeLast->setDate( d );

    *nbOfValidRecords = data[ 9 ];

    return data[ 8 ];

}

/**
 * initialize resets WS2300 to cold start (rewind and start over)
 *
 * @param Input : Nothing
 *
 * @note device number of the already open serial port
 *
 * @return 0 if fail, 1 if success
 *
 */
int Ws2355::initialize( void )
{
    unsigned char command = 0x06;
    unsigned char answer;

    qDebug() << ">initialize";

    writeDevice(  &command, 1);

    if( readDevice(  &answer, 1) != 1)
            return 0;

    writeDevice(  &command, 1);
    writeDevice(  &command, 1);

    if( readDevice(  &answer, 1) != 1)
            return 0;

    writeDevice(  &command, 1);

    if( readDevice(  &answer, 1) != 1)
            return 0;

    writeDevice(  &command, 1);

    if( readDevice(  &answer, 1) != 1)
            return 0;

    if( answer != 2)
            return 0;

    return 1;
}


/**
 * readData reads data from the WS2300 based on a given address,
 * number of data read, and a an already open serial port
 *
 * @param address ( IN Parameter ) (integer - 16 bit)
 * @param number ( IN Parameter ) number of bytes to read, max value 15
 *
 * @param readdata ( OUT Parameter ) pointer to an array of chars containing
 *                     the just read data, not zero terminated
 * @param commanddata ( OUT Parameter ) pointer to an array of chars containing
 *                     the commands that were sent to the station
 *
 * return number of bytes read, -1 if failed
 *
 */
int Ws2355::readData( int address,
                       int number,
                       unsigned char *readdata,
                       unsigned char *commanddata)
{
    unsigned char answer;
    int i;

    // First 4 bytes are populated with converted address range 0000-13B0
    addressEncoder( address, commanddata );
    // Last populate the 5th byte with the converted number of bytes
    commanddata[ 4 ] = numberOfEncoder( number );

    for( i = 0; i < 4; i++ )
    {
        if( writeDevice( commanddata + i, 1 ) != 1 )
            return -1;
        if( readDevice( &answer, 1 ) != 1 )
            return -1;
        if( answer != commandCheck0123( commanddata + i, i ))
            return -1;
    }

    //Send the final command that asks for 'number' of bytes, check answer
    if( writeDevice( commanddata + 4, 1 ) != 1 )
        return -1;
    if( readDevice( &answer, 1 ) != 1 )
        return -1;
    if( answer != commandCheck4( number ))
        return -1;

    //Read the data bytes
    for( i = 0; i < number; i++ )
    {
        if( readDevice( readdata + i, 1 ) != 1 )
            return -1;
    }

    //Read and verify checksum
    if( readDevice( &answer, 1 ) != 1 )
        return -1;
    if( answer != dataChecksum( readdata, number ))
        return -1;

    return i;
}


/**
 * writeData writes data to the WS2300.
 * It can both write nibbles and set/unset bits
 *
 * @param address ( IN Parameter ) (interger - 16 bit)
 * @param number ( IN Parameter ) number of nibbles to be written/changed
 *                       must 1 for bit modes (SETBIT and UNSETBIT)
 *                       max 80 for nibble mode (WRITENIB)
 * @param encode_constant ( IN Parameter ) unsigned char
 *                                (SETBIT, UNSETBIT or WRITENIB)
 * @param writedata ( IN Parameter ) pointer to an array of chars containing
 *                          data to write, not zero terminated
 *                          data must be in hex - one digit per byte
 *                          If bit mode value must be 0-3 and only
 *                          the first byte can be used.
 *
 * @param commanddata ( OUT Parameter ) pointer to an array of chars containing
 *           the commands that were sent to the station
 *
 * @return number of bytes written, -1 if failed
 *
 */
int Ws2355::writeData( int address,
                        int number,
                        unsigned char encode_constant,
                        unsigned char *writedata,
                        unsigned char *commanddata)
{
    unsigned char answer;
    unsigned char encoded_data[ 80 ];
    int i = 0;
    unsigned char ack_constant = WRITEACK;

    if( encode_constant == SETBIT )
    {
        ack_constant = SETACK;
    }
    else if( encode_constant == UNSETBIT )
    {
        ack_constant = UNSETACK;
    }

    // First 4 bytes are populated with converted address range 0000-13XX
    addressEncoder( address, commanddata );
    // populate the encoded_data array
    dataEncoder( number, encode_constant, writedata, encoded_data );

    //Write the 4 address bytes
    for( i = 0; i < 4; i++ )
    {
        if( writeDevice( commanddata + i, 1 ) != 1 )
            return -1;
        if( readDevice( &answer, 1) != 1 )
            return -1;
        if( answer != commandCheck0123( commanddata + i, i ))
            return -1;
    }

    //Write the data nibbles or set/unset the bits
    for( i = 0; i < number; i++ )
    {
        if( writeDevice( encoded_data + i, 1 ) !=  1)
            return -1;
        if( readDevice(  &answer, 1 ) != 1 )
            return -1;
        if( answer != (writedata[ i ] + ack_constant ))
            return -1;
        commanddata[ i + 4 ] = encoded_data[ i ];
    }

    return i;
}


/**
 * Read data, retry until success or maxretries
 * Reads data from the WS2300 based on a given address,
 * number of data read, and a an already open serial port
 * Uses the readData function and has same interface
 *
 * @param address ( IN Parameter ) (integer - 16 bit)
 * @param number( IN Parameter ) number of bytes to read, max value 15
 *
 * @param readdata ( OUT Parameter ) pointer to an array of chars containing
 *                     the just read data, not zero terminated
 * @param commanddata ( OUT Parameter ) pointer to an array of chars containing
 *                     the commands that were sent to the station
 *
 * @return number of bytes read, -1 if failed
 */
int Ws2355::readSafe( int address,
                      int number,
                      unsigned char *readdata,
                      unsigned char *commanddata)
{
    int j;

    for( j = 0; j < MAXRETRIES; j++ )
    {
        reset06();

        // Read the data. If expected number of bytes read break out of loop.
        if( readData( address, number, readdata, commanddata ) == number )
        {
            break;
        }
    }

    // If we have tried MAXRETRIES times to read we expect not to
    // have valid data
    if( j == MAXRETRIES )
    {
        return -1;
    }

    return number;
}


/**
 * writeSafe Write data, retry until success or maxretries
 * Writes data to the WS2300 based on a given address,
 * number of data to write, and a an already open serial port
 * Uses the writeData function and has same interface
 *
 * @param address ( IN Parameter ) (integer - 16 bit)
 * @param number ( IN Parameter ) number of nibbles to be written/changed
 *                       must 1 for bit modes (SETBIT and UNSETBIT)
 *                       unlimited for nibble mode (WRITENIB)
 * @param encode_constant ( IN Parameter ) unsigned char
 *                               (SETBIT, UNSETBIT or WRITENIB)
 * @param writedata ( IN Parameter ) pointer to an array of chars containing
 *                          data to write, not zero terminated
 *                          data must be in hex - one digit per byte
 *                          If bit mode value must be 0-3 and only
 *                          the first byte can be used.
 *
 * @param commanddata ( OUT Parameter ) pointer to an array of chars containing
 *                            the commands that were sent to the station
 *
 * @return number of bytes written, -1 if failed
 */
int Ws2355::writeSafe( int address,
                       int number,
                       unsigned char encode_constant,
                       unsigned char *writedata,
                       unsigned char *commanddata)
{
    int j;

    for( j = 0; j < MAXRETRIES; j++ )
    {
        //qDebug("Iteration = %d\n",j);
        reset06();

        // Read the data. If expected number of bytes read break out of loop.
        if( writeData( address, number, encode_constant, writedata, commanddata ) == number )
        {
            break;
        }
    }

    // If we have tried MAXRETRIES times to read we expect not to have valid data
    if( j == MAXRETRIES )
    {
        return -1;
    }

    return number;
}


/**
 * readDevice in the Linux version is identical
 * to the standard Linux read()
 *
 * @param buffer ( IN Parameter ) pointer to the buffer to read into (unsigned char)
 * @param size ( IN Parameter ) number of bytes to read
 *
 * @param buffer ( OUT Parameter ) modified on success (pointer to unsigned char)
 *
 * @return number of bytes read
 *
 */
int Ws2355::readDevice( unsigned char *buffer, int size)
{
    int ret;

    for( ;; )
    {
        ret = read( fdStation, buffer, size );

        if( ret == 0 && errno == EINTR )
            continue;
        return ret;
    }
}

/**
 * writeDevice in the Linux version is identical
 * to the standard Linux write()
 *
 * @param buffer ( IN Parameter ) pointer to the buffer to write from
 * @param size ( IN Parameter ) number of bytes to write
 *
 * @return number of bytes written
 *
 */
int Ws2355::writeDevice( unsigned char *buffer, int size)
{
    int ret = write( fdStation, buffer, size );
    tcdrain( fdStation );	// wait for all output written

    return ret;
}

/**
 * reset WS2300 by sending command 06
 */
void Ws2355::reset06( void )
{
    unsigned char command = 0x06;
    unsigned char answer;
    int i;

    for( i = 0; i < 100; i++ )
    {

        // Discard any garbage in the input buffer
        tcflush( fdStation, TCIFLUSH );

        writeDevice( &command, 1 );

        // Occasionally 0, then 2 is returned.  If zero comes back, continue
        // reading as this is more efficient than sending an out-of sync
        // reset and letting the data reads restore synchronization.
        // Occasionally, multiple 2's are returned.  Read with a fast timeout
        // until all data is exhausted, if we got a two back at all, we
        // consider it a success

        while ( 1 == readDevice( &answer, 1 ))
        {
            if( answer == 2 )
            {
                return;
            }
        }
        newThread->msleep( 50 * i );   //we sleep longer and longer for each retry
    }
    fprintf( stderr, "\nCould not reset\n" );
    exit( EXIT_FAILURE );
}

/**
 * Turns display light on and off
 *
 * @param control   0 = off, Anything else = on
 */
void Ws2355::light( int control )
{
    unsigned char data;
    unsigned char command[ 25 ];  //Data returned is just ignored
    int address = 0x016;
    int number = 1;
    unsigned char encode_constant;

    data = 0;
    encode_constant = UNSETBIT;
    if( control != 0 )
        encode_constant = SETBIT;

    if( writeSafe( address, number, encode_constant, &data, command ) != number )
        writeErrorExit();

    return;
}


/**
 * exit location for all calls to readSafe for error exit.
 * includes error reporting.
 */
void Ws2355::readErrorExit(void)
{
    perror("readSafe() error");
    exit(EXIT_FAILURE);
}

/**
 * exit location for all calls to writeSafe for error exit.
 * includes error reporting.
 */
void Ws2355::writeErrorExit(void)
{
    perror("writeSafe() error");
    exit(EXIT_FAILURE);
}

 /**
 * address_encoder converts an 16 bit address to the form needed
 * by the WS-2300 when sending commands.
 *
 * @param address_in (interger - 16 bit)
 *
 * @param address_out ( OUT Parameter ) Pointer to an unsigned character array.
 *          3 bytes, not zero terminated.
 *
 * @return Nothing.
 *
 */
void Ws2355::addressEncoder( int address_in, unsigned char *address_out )
{
    int i = 0;
    int adrbytes = 4;
    unsigned char nibble;

    for( i = 0; i < adrbytes; i++ )
    {
            nibble = ( address_in >> ( 4 * ( 3 - i ))) & 0x0F;
            address_out[ i ] = ( unsigned char ) ( 0x82 + ( nibble * 4 ));
    }

    return;
}

/**
 * data_encoder converts up to 15 data bytes to the form needed
 * by the WS-2300 when sending write commands.
 *
 * @param number - number of databytes (integer)
 * @param encode_constant - unsigned char 0x12=set bit, 0x32=unset bit, 0x42=write nibble
 *
 * @param data_in ( OUT Parameter ) char array with up to 15 hex values
 * @param data_out ( OUT Parameter ) Pointer to an unsigned character array.
 */
void Ws2355::dataEncoder( int number,
                           unsigned char encode_constant,
                           unsigned char *data_in,
                           unsigned char *data_out)
{
    int i = 0;

    for( i = 0; i < number; i++ )
    {
        data_out[ i ] = ( unsigned char ) ( encode_constant + ( data_in[ i ] * 4 ));
    }

    return;
}


/**
 * numberOfEncoder converts the number of bytes we want to read
 * to the form needed by the WS-2300 when sending commands.
 *
 * @param number integer, max value 15
 *
 * @return unsigned char which is the coded number of bytes
 */
unsigned char Ws2355::numberOfEncoder( int number )
{
    int coded_number;

    coded_number = ( unsigned char ) ( 0xC2 + number * 4 );
    if( coded_number > 0xfe )
        coded_number = 0xfe;

    return coded_number;
}


/**
 * command_check0123 calculates the checksum for the first 4
 * commands sent to WS2300.
 *
 * @param pointer to char to check
 * @param sequence of command - i.e. 0, 1, 2 or 3.
 *
 * @return calculated checksum as unsigned char
 */
unsigned char Ws2355::commandCheck0123( unsigned char *command, int sequence )
{
    int response;

    response = sequence * 16 + (( *command ) - 0x82 ) / 4;

    return ( unsigned char ) response;
}


/**
 * command_check4 calculates the checksum for the last command
 * which is sent just before data is received from WS2300
 *
 * @param number of bytes requested
 *
 * @return expected response from requesting number of bytes
 *
 */
unsigned char Ws2355::commandCheck4( int number )
{
    int response;

    response = 0x30 + number;

    return response;
}


/**
 * data_checksum calculates the checksum for the data bytes received
 * from the WS2300
 *
 * @param pointer to array of data to check
 * @param number of bytes in array
 *
 * @return calculated checksum as unsigned char
 *
 */
unsigned char Ws2355::dataChecksum( unsigned char *data, int number )
{
    int checksum = 0;
    int i;

    for( i = 0; i < number; i++ )
    {
            checksum += data[ i ];
    }

    checksum &= 0xFF;

    return ( unsigned char ) checksum;
}

/**
 * Read the history information like interval, countdown, time
 * of last record, pointer to last record.
 *
 * @param record ( IN Parameter ) record index number to be read [0x00-0xAE]
 *
 * @param temperatureIndoor ( OUT Parameter ) (double)
 * @param temperatureIndoor ( OUT Parameter ) (double)
 * @param pressure ( OUT Parameter ) (double)
 * @param humidityIndoor ( OUT Parameter ) (integer)
 * @param humidityOutdoor ( OUT Parameter ) (integer)
 * @param raincount ( OUT Parameter ) (double)
 * @param windspeed ( OUT Parameter ) (double)
 * @param windir_degrees ( OUT Parameter ) (double)
 * @param dewpoint ( OUT Parameter ) (double) - calculated
 * @param windchill ( OUT Parameter ) (double) - calculated, new post 2001 formula
 *
 * @return interger index number pointing to next record
 */
int Ws2355::readHistoryRecord( int record,
                                 double *temperatureIndoor,
                                 double *temperatureOutdoor,
                                 double *pressure,
                                 int *humidityIndoor,
                                 int *humidityOutdoor,
                                 double *raincount,
                                 double *windspeed,
                                 double *windDirDegrees,
                                 double *dewpoint,
                                 double *windchill)
{
    unsigned char data[ 20 ];
    unsigned char command[ 25 ];
    int address;
    int bytes = 10;
    long int tempint;
    double A, B, C;         // Intermediate values used for dewpoint calculation
    double wind_kmph;

    address = 0x6C6 + record * 19;

    if( readSafe( address, bytes, data, command ) != bytes )
        readErrorExit();

    tempint = ( data[ 4 ] << 12 ) + ( data[ 3 ] << 4 ) + ( data[ 2 ] >> 4 );

    *pressure = 1000 + ( tempint % 10000 ) / 10.0;

    if( *pressure >= 1502.2 )
        *pressure = *pressure - 1000;

    *pressure = *pressure /  HECTOPASCAL;

    *humidityIndoor = ( tempint - ( tempint % 10000 )) / 10000.0;

    *humidityOutdoor = ( data[ 5 ] >> 4 ) * 10 + ( data[ 5 ] & 0xF );

    *raincount = (( data[ 7 ] & 0xF ) * 256 + data[ 6 ]) * 0.518 / MILLIMETERS;

    *windspeed = ( data[ 8 ] * 16 + ( data[ 7 ] >> 4 )) / 10.0; //Need metric for WC

    *windDirDegrees = ( data[ 9 ] & 0xF ) * 22.5;

    // Temperatures	in Celcius. Cannot convert until WC is calculated
    tempint = (( data[ 2 ] & 0xF ) << 16 ) + ( data[ 1 ] << 8 ) + data[ 0 ];
    *temperatureIndoor = ( tempint % 1000 ) / 10.0 - 30.0;
    *temperatureOutdoor = ( tempint - ( tempint % 1000 )) / 10000.0 - 30.0;

    // Calculate windchill using new post 2001 USA/Canadian formula
    // Twc = 13.112 + 0.6215*Ta -11.37*V^0.16 + 0.3965*Ta*V^0.16 [Celcius and km/h]

    wind_kmph = 3.6 * *windspeed;
    if( wind_kmph > 4.8 )
    {
        *windchill = 13.112 + 0.6215 * *temperatureOutdoor -
                     11.37 * pow( wind_kmph, 0.16 ) +
                     0.3965 * *temperatureOutdoor * pow( wind_kmph, 0.16 );
    }
    else
    {
        *windchill = *temperatureOutdoor;
    }

    // Calculate dewpoint
    // REF http://www.faqs.org/faqs/meteorology/temp-dewpoint/
    A = 17.2694;
    B = ( *temperatureOutdoor > 0 ) ? 237.3 : 265.5;
    C = ( A * *temperatureOutdoor ) / ( B + *temperatureOutdoor ) + log(( double ) * humidityOutdoor / 100 );
    *dewpoint = B * C / ( A - C );

    // Now that WC/DP is calculated we can convert all temperatures and winds
/*
    if( theGlobalConfiguration->getTemperatureUnit() ==  FAHRENHEIT )
    {
        *temperatureIndoor = *temperatureIndoor * 9 / 5 + 32;
        *temperatureOutdoor = *temperatureOutdoor * 9 / 5 + 32;
        *windchill = *windchill * 9 / 5 + 32;
        *dewpoint = *dewpoint * 9 / 5 + 32;
    }
*/
    *windspeed *= KILOMETERS_PER_HOUR;

    return ( ++record ) % 0xAF;
}


void Ws2355::dataReading( void )
{
    FILE *currentDataFileDescriptor;
    char logline[3000] = "";
    char tempstring[1000] = "";
    int interval, countdown, no_records;
    long counter;
    char ch;
    char datestring[50];        //used to hold the date stamp for the log file
    QDateTime time_last;
    time_t timeLastLog, timeLastRecord;
    struct tm timeLastLog_tm, timeLastRecord_tm;
    int currentRecord, nextRecord, lastLogRecord, newRecords;
    double temperatureIn;
    double temperatureOut;
    double dewpoint;
    double windchill;
    double pressure;
    double pressure_term;
    int humidityIn;
    int humidityOut;
    double rain;
    double windspeed;
    double windDirDegrees;
    const char *directions[]= { "N","NNE","NE","ENE","E","ESE","SE","SSE",
                                "S","SSW","SW","WSW","W","WNW","NW","NNW" };

    int temp_int1, temp_int2, i;

    // Get in-data and select mode.
    // Open the current data file
    QString strFile( theGlobalConfiguration->getFullDataFileName() );

    currentDataFileDescriptor = fopen( strFile.toAscii(), "ab+" );
    if( currentDataFileDescriptor == NULL )
    {
        qDebug() << QObject::trUtf8( "Impossible d'ouvrir le fichier currentDataFile.txt\n" );
        return;
    }

    // Go to the end of file
    fseek( currentDataFileDescriptor, 1L, SEEK_END );

    // Get the last line
    counter = 60;
    do
    {
        counter++;
        if( fseek( currentDataFileDescriptor, -counter, SEEK_END ) < 0 )
            break;
        ch = getc( currentDataFileDescriptor );
    }
    while ( ch != '\n' && ch != '\r' );

    // Read the timestamp of the last line
    if(  fscanf( currentDataFileDescriptor,"%4d%2d%2d%2d%2d", &temp_int1, &temp_int2,
                 &timeLastLog_tm.tm_mday, &timeLastLog_tm.tm_hour,
                 &timeLastLog_tm.tm_min) == 5 )
    {
        timeLastLog_tm.tm_year = temp_int1 - 1900;
        timeLastLog_tm.tm_mon = temp_int2 - 1;
        timeLastLog_tm.tm_sec = 0;
        timeLastLog_tm.tm_isdst = -1;
    }
    else
    {	//if no valid log we set the date to 1 Jan 1990 0:00
        timeLastLog_tm.tm_year = 90;
        timeLastLog_tm.tm_mon = 0;
        timeLastLog_tm.tm_mday = 1;
        timeLastLog_tm.tm_hour = 0;
        timeLastLog_tm.tm_min = 0;
        timeLastLog_tm.tm_sec = 0;
        timeLastLog_tm.tm_isdst = -1;
    }

    timeLastLog = mktime( &timeLastLog_tm );

    currentRecord = readHistoryInfo( &interval, &countdown, &time_last, &no_records );

    timeLastRecord_tm.tm_year = time_last.date().year() - 1900;
    timeLastRecord_tm.tm_mon  = time_last.date().month() - 1;
    timeLastRecord_tm.tm_mday = time_last.date().day();
    timeLastRecord_tm.tm_hour = time_last.time().hour();
    timeLastRecord_tm.tm_min  = time_last.time().minute();
    timeLastRecord_tm.tm_sec  = 0;
    timeLastRecord_tm.tm_isdst = -1;

    timeLastRecord = mktime( &timeLastRecord_tm );

    pressure_term = pressureCorrection( theGlobalConfiguration->getPressureUnit());

    newRecords = (int)difftime( timeLastRecord, timeLastLog ) / ( 60 * interval );

    if(  newRecords > MAX_NUMBER_OF_RECORDS )
         newRecords = MAX_NUMBER_OF_RECORDS;

    if( newRecords > no_records )
        newRecords = no_records;

    lastLogRecord = currentRecord - newRecords;

    if( lastLogRecord < 0)
            lastLogRecord = 0xAE + lastLogRecord + 1;

    timeLastRecord_tm.tm_min -= newRecords * interval;

    for( i = 1; i <= newRecords; i++ )
    {
        qDebug() << newRecords - i;
        getTheDataLogged()->setCurrentLogRead( newRecords - i );

        QCoreApplication::processEvents();

        nextRecord = ( i + lastLogRecord ) % 0xAF;

        readHistoryRecord( nextRecord,
                           &temperatureIn,
                           &temperatureOut,
                           &pressure,
                           &humidityIn,
                           &humidityOut,
                           &rain,
                           &windspeed,
                           &windDirDegrees,
                           &dewpoint,
                           &windchill );

        // Read temperature indoor
        sprintf( logline,"%.1f ", temperatureIn );

        // Read temperature outdoor
        sprintf( tempstring,"%.1f ", temperatureOut );
        strcat( logline, tempstring );

        // Calculate dewpoint
        sprintf( tempstring,"%.1f ", dewpoint );
        strcat( logline, tempstring );

        // Read relatice humidity indoor
        sprintf( tempstring,"%d ", humidityIn );
        strcat( logline, tempstring );

        // Read relatice humidity outdoor
        sprintf( tempstring,"%d ", humidityOut );
        strcat( logline, tempstring );

        // Read wind speed, wind direction
        sprintf( tempstring,"%.1f %.1f %s ", windspeed, windDirDegrees, directions[( int )( windDirDegrees / 22.5 )]);
        strcat( logline, tempstring );

        // Read windchill
        sprintf( tempstring,"%.1f ", windchill );
        strcat( logline, tempstring );

        // Read total rain
        sprintf( tempstring,"%.2f ", rain );
        strcat( logline, tempstring );

        // Read relative pressure
        sprintf( tempstring,"%.3f ", pressure + pressure_term );
        strcat( logline, tempstring );

        // Get date and time for log file, place before all data in the log line
        timeLastRecord_tm.tm_min += interval;
        mktime( &timeLastRecord_tm );                 //normalize timeLastLog_tm

        strftime( datestring, sizeof( datestring ), "%Y%m%d%H%M%S", &timeLastRecord_tm );

        // Print out, add this new line at the end of file
        fseek( currentDataFileDescriptor, 0L, SEEK_END );
        fprintf( currentDataFileDescriptor, "%s %s\n", datestring, logline );
        fflush( NULL );

        if( ( newRecords - i ) == 0 )
            getTheDataLogged()->setLastDataReadingDate( QDateTime::currentDateTime() );

    }

    fclose( currentDataFileDescriptor );
}
