/***************************************************************************
 *   Copyright (C) 2012 by Gil Mardon                                      *
 *   gil.mardon@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <errno.h>
#include <sys/file.h>


#include <unistd.h>
#include <QCoreApplication>

#include "Ws2355Thread.h"
#include "Ws2355.h"
#include "CurrentData.h"
#include "CurrentDataTab.h"
#include "Connection.h"
#include "WsMeteo.h"
#include "Configuration.h"
#include <QDebug>
#include <QTime>

Ws2355Thread::Ws2355Thread( Ws2355 *theStation, Configuration *theGlobalConfiguration, CurrentData *theCurrentData, Connection *theConnection )
    :theStation( theStation ), theGlobalConfiguration( theGlobalConfiguration ), theCurrentData( theCurrentData ), theConnection( theConnection )
{
    start();
}

void Ws2355Thread::run()
{
    double windDirection;
    unsigned char tendency,
                  forecast;

    static int i = 0;

    for(;;)
    {
        sleep( 1 );
        // qDebug() << "readCurrentValues : " << i;
        if( theConnection->openCommunication( theStation->fdStation ) != OK )
            continue;

        if( theCurrentData->dataReadingStateIsStarted() )
        {
            theStation->dataReading();
            theCurrentData->stopDataReading();
        }
        switch( i++ )
        {
            case 0 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setIndoorTemperature";
                theCurrentData->setIndoorTemperature( theStation->readTemperatureIndoor( CELCIUS ) );
                break;
            case 1 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setOutdoorTemperature";
                theCurrentData->setOutdoorTemperature( theStation->readTemperatureOutdoor( CELCIUS ) );
                break;
            case 2 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setIndoorHumidity";
                theCurrentData->setIndoorHumidity( theStation->readHumidityIndoor() );
                break;
            case 3 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setOutdoorHumidity";
                theCurrentData->setOutdoorHumidity( theStation->readHumidityOutdoor() );
                break;
            case 4 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setWindChillTemperature";
                theCurrentData->setWindChillTemperature( theStation->readWindchill( CELCIUS ) );
                break;
            case 5 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setDewPointTemperature";
                theCurrentData->setDewPointTemperature( theStation->readDewpoint( CELCIUS ));
                break;
            case 6 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setPressureAbsolute";
                theCurrentData->setPressureAbsolute( theStation->readAbsPressure( MILLIBARS ));
                break;
            case 7 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setPressureRelative";
                theCurrentData->setPressureRelative( theStation->readRelPressure( MILLIBARS ));
                break;
            case 8 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setWindSpeed";
                theCurrentData->setWindSpeed( theStation->readWindCurrent( KILOMETERS_PER_HOUR, &windDirection ));
                break;
            case 9 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setWindDirection";
                theCurrentData->setWindDirection( windDirection );
                break;
            case 10 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "tendencyForecast";
                theStation->readTendencyForecast( &tendency, &forecast );
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setWeatherTendency";
                theCurrentData->setWeatherTendency( tendency );
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setWeatherForecast";
                theCurrentData->setWeatherForecast( forecast );
                break;
            case 11 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setRain1Hour";
                theCurrentData->setRain1Hour( theStation->readRain1h( MILLIMETERS ));
                break;
            case 12 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setRain1Day";
                theCurrentData->setRain1Day( theStation->readRain24h( MILLIMETERS ));
                break;
            case 13 :
                qDebug() << QTime::currentTime().toString( "hh:mm:ss:zzz" ) << "setRainTotal";
                theCurrentData->setRainTotal( theStation->readRainTotal( MILLIMETERS ));
                break;
            case 14 :
                qDebug() << "DewpointMaxAlarm :" << theStation->readDewpointMaxAlarm();
                theStation->setDewPointTemperatureMaxAlarmConf( theStation->readDewpointMaxAlarm() );
                qDebug() << "DewpointMax :" << theStation->readDewpointMax( CELCIUS );
                theStation->setDewPointTemperatureMaxConf( theStation->readDewpointMax( CELCIUS ) );
                break;
            case 15 :
                qDebug() << "DewpointMinAlarm :" << theStation->readDewpointMinAlarm();
                theStation->setDewPointTemperatureMinAlarmConf( theStation->readDewpointMinAlarm() );
                qDebug() << "DewpointMin :" << theStation->readDewpointMin( CELCIUS );
                theStation->setDewPointTemperatureMinConf( theStation->readDewpointMin( CELCIUS ) );
                break;
            case 16 :
                qDebug() << "IndoorHumidityMaxAlarm :" << theStation->readIndoorHumidityMaxAlarm();
                theStation->setInHumidityMaxAlarmConf( theStation->readIndoorHumidityMaxAlarm() );
                qDebug() << "IndoorHumidityMax :" << theStation->readIndoorHumidityMax();
                theStation->setInHumidityMaxConf( theStation->readIndoorHumidityMax() );
                break;
            case 17 :
                qDebug() << "IndoorHumidityMinAlarm :" << theStation->readIndoorHumidityMinAlarm() ;
                theStation->setInHumidityMinAlarmConf( theStation->readIndoorHumidityMinAlarm() );
                qDebug() << "IndoorHumidityMin :" << theStation->readIndoorHumidityMin() ;
                theStation->setInHumidityMinConf( theStation->readIndoorHumidityMin() );
                break;
            case 18 :
                qDebug() << "IndoorTemperatureMaxAlarm :" << theStation->readIndoorTemperatureMaxAlarm();
                theStation->setInTemperatureMaxAlarmConf( theStation->readIndoorTemperatureMaxAlarm() );
                qDebug() << "IndoorTemperatureMax :" << theStation->readIndoorTemperatureMax( CELCIUS );
                theStation->setInTemperatureMaxConf( theStation->readIndoorTemperatureMax( CELCIUS ) );
                break;
            case 19 :
                qDebug() << "IndoorTemperatureMinAlarm :" << theStation->readIndoorTemperatureMinAlarm();
                theStation->setInTemperatureMinAlarmConf( theStation->readIndoorTemperatureMinAlarm() );
                qDebug() << "IndoorTemperatureMin :" << theStation->readIndoorTemperatureMin( CELCIUS );
                theStation->setInTemperatureMinConf( theStation->readIndoorTemperatureMin( CELCIUS ) );
                break;
            case 20 :
                qDebug() << "OutdoorHumidityMaxAlarm :" << theStation->readOutdoorHumidityMaxAlarm();
                theStation->setOutHumidityMaxAlarmConf( theStation->readOutdoorHumidityMaxAlarm() );
                qDebug() << "OutdoorHumidityMax :" << theStation->readOutdoorHumidityMax();
                theStation->setOutHumidityMaxConf( theStation->readOutdoorHumidityMax() );
                break;
            case 21 :
                qDebug() << "OutdoorHumidityMinAlarm :" << theStation->readOutdoorHumidityMinAlarm();
                theStation->setOutHumidityMinAlarmConf( theStation->readOutdoorHumidityMinAlarm() );
                qDebug() << "OutdoorHumidityMin :" << theStation->readOutdoorHumidityMin();
                theStation->setOutHumidityMinConf( theStation->readOutdoorHumidityMin() );
                break;
            case 22 :
                qDebug() << "OutdoorTemperatureMaxAlarm :" << theStation->readOutdoorTemperatureMaxAlarm();
                theStation->setOutTemperatureMaxAlarmConf( theStation->readOutdoorTemperatureMaxAlarm() );
                qDebug() << "OutdoorTemperatureMax :" << theStation->readOutdoorTemperatureMax( CELCIUS );
                theStation->setOutTemperatureMaxConf( theStation->readOutdoorTemperatureMax( CELCIUS ) );
                break;
            case 23 :
                qDebug() << "OutdoorTemperatureMinAlarm :" << theStation->readOutdoorTemperatureMinAlarm();
                theStation->setOutTemperatureMinAlarmConf( theStation->readOutdoorTemperatureMinAlarm() );
                qDebug() << "OutdoorTemperatureMin :" << theStation->readOutdoorTemperatureMin( CELCIUS );
                theStation->setOutTemperatureMinConf( theStation->readOutdoorTemperatureMin( CELCIUS ) );
                break;
            case 24 :
                qDebug() << "WindchillMaxAlarm :" << theStation->readWindchillMaxAlarm();
                theStation->setWindChillTemperatureMaxAlarmConf( theStation->readWindchillMaxAlarm() );
                qDebug() << "WindchillMax :" << theStation->readWindchillMax( CELCIUS );
                theStation->setWindChillTemperatureMaxConf( theStation->readWindchillMax( CELCIUS ) );
                break;
            case 25 :
                qDebug() << "WindchillMinAlarm :" << theStation->readWindchillMinAlarm();
                theStation->setWindChillTemperatureMinAlarmConf( theStation->readWindchillMinAlarm() );
                qDebug() << "WindchillMin :" << theStation->readWindchillMin( CELCIUS );
                theStation->setWindChillTemperatureMinConf( theStation->readWindchillMin( CELCIUS ) );
                break;
            case 26 :
                qDebug() << "WindDirectionAlarm :" << theStation->readWindDirectionAlarm();
                theStation->setWindDirectionAlarmConf( theStation->readWindDirectionAlarm() );
                qDebug() << "WindDirection :" << theStation->readWindDirection();
                theStation->setWindDirectionConf( theStation->readWindDirection() );
                break;
            case 27 :
                qDebug() << "WindSpeedMaxAlarm :" << theStation->readWindSpeedMaxAlarm();
                theStation->setWindSpeedMaxAlarmConf( theStation->readWindSpeedMaxAlarm() );
                qDebug() << "WindSpeedMax :" << theStation->readWindSpeedMax( KILOMETERS_PER_HOUR );
                theStation->setWindSpeedMaxConf( theStation->readWindSpeedMax(KILOMETERS_PER_HOUR ));
                break;
            case 28 :
                qDebug() << "WindSpeedMinAlarm :" << theStation->readWindSpeedMinAlarm();
                theStation->setWindSpeedMinAlarmConf( theStation->readWindSpeedMinAlarm() );
                qDebug() << "WindSpeedMin :" << theStation->readWindSpeedMin( KILOMETERS_PER_HOUR );
                theStation->setWindSpeedMinConf( theStation->readWindSpeedMin( KILOMETERS_PER_HOUR ));
                break;
            case 29 :
                qDebug() << "PressureMaxAlarm :" << theStation->readPressureMaxAlarm();
                theStation->setPressureMaxAlarmConf( theStation->readPressureMaxAlarm());
                qDebug() << "PressureMax :" << theStation->readPressureMax( MILLIBARS );
                theStation->setPressureMaxConf( theStation->readPressureMax( MILLIBARS ));
                break;
            case 30 :
                qDebug() << "PressureMinAlarm :" << theStation->readPressureMinAlarm();
                theStation->setPressureMinAlarmConf( theStation->readPressureMinAlarm());
                qDebug() << "PressureMin :" << theStation->readPressureMin( MILLIBARS );
                theStation->setPressureMinConf( theStation->readPressureMin( MILLIBARS ));
                break;
            case 31 :
                qDebug() << "Rain1hAlarm :" << theStation->readRain1hAlarm();
                theStation->setRain1HAlarmConf( theStation->readRain1hAlarm());
                qDebug() << "Rain1h :" << theStation->readRain1hMax( MILLIMETERS );
                theStation->setRain1HConf( theStation->readRain1hMax( MILLIMETERS ));
                break;
            case 32 :
                qDebug() << "Rain24hAlarm :" << theStation->readRain24hAlarm();
                theStation->setRain1DayAlarmConf( theStation->readRain24hAlarm());
                qDebug() << "Rain24h :" << theStation->readRain24hMax( MILLIMETERS );
                theStation->setRain1DayConf( theStation->readRain24hMax( MILLIMETERS ));
                    break;
            default :
                i = 0;
                sleep( 60 );        // Wait 1 mn before restart a new total reading
                break;
        }
        close( theStation->fdStation );
    }
}

